package cz.nic.tablexia.game;

import com.badlogic.gdx.scenes.scene2d.Group;

import cz.nic.tablexia.Tablexia;
import cz.nic.tablexia.TablexiaApplication;
import cz.nic.tablexia.bus.ApplicationBus;
import cz.nic.tablexia.bus.ApplicationBus.ApplicationEvent;
import cz.nic.tablexia.game.games.in_the_darkness.InTheDarknessGame;
import cz.nic.tablexia.game.games.kidnapping.KidnappingGame;
import cz.nic.tablexia.game.games.kidnapping.util.KidnappingResultResolver;
import cz.nic.tablexia.game.games.night_watch.NightWatchGame;
import cz.nic.tablexia.game.games.pursuit.PursuitGame;
import cz.nic.tablexia.game.games.robbery.RobberyGame;
import cz.nic.tablexia.game.games.shooting_range.ShootingRangeGame;
import cz.nic.tablexia.game.games.shooting_range.tools.ShootingRangeResultResolver;
import cz.nic.tablexia.loader.application.ApplicationTextManager;
import cz.nic.tablexia.menu.AbstractMenu;
import cz.nic.tablexia.menu.IMenuItem;
import cz.nic.tablexia.shared.model.Game;
import cz.nic.tablexia.screen.gamemenu.GameMenuScreen;
import cz.nic.tablexia.screen.gamemenu.event.ShowGameMenuPageEvent;

public enum GameDefinition implements ApplicationEvent, IMenuItem {

    ROBBERY         (1, ApplicationTextManager.ApplicationTextsAssets.GAME_ROBBERY_TITLE, ApplicationTextManager.ApplicationTextsAssets.GAME_ROBBERY_DESCRIPTION, RobberyGame.class, false, new RobberyGame.GameResultResolver(), ApplicationTextManager.ApplicationTextsAssets.GAME_SCORE, ApplicationTextManager.ApplicationTextsAssets.GAME_AVERAGE_SCORE),
    PURSUIT         (2, ApplicationTextManager.ApplicationTextsAssets.GAME_PURSUIT_TITLE, ApplicationTextManager.ApplicationTextsAssets.GAME_PURSUIT_DESCRIPTION, PursuitGame.class, false, new PursuitGame.GameResultResolver(), ApplicationTextManager.ApplicationTextsAssets.GAME_SCORE_DURATION, ApplicationTextManager.ApplicationTextsAssets.GAME_AVERAGE_SCORE_DURATION),
    KIDNAPPING      (3, ApplicationTextManager.ApplicationTextsAssets.GAME_KIDNAPPING_TITLE, ApplicationTextManager.ApplicationTextsAssets.GAME_KIDNAPPING_DESCRIPTION, KidnappingGame.class, false, new KidnappingResultResolver(), ApplicationTextManager.ApplicationTextsAssets.GAME_SCORE_ERROR, ApplicationTextManager.ApplicationTextsAssets.GAME_AVERAGE_SCORE_ERROR),
    NIGHT_WATCH     (4, ApplicationTextManager.ApplicationTextsAssets.GAME_NIGHT_WATCH_TITLE, ApplicationTextManager.ApplicationTextsAssets.GAME_NIGHT_WATCH_DESCRIPTION, NightWatchGame.class, false, new NightWatchGame.GameResultResolver(), ApplicationTextManager.ApplicationTextsAssets.GAME_SCORE, ApplicationTextManager.ApplicationTextsAssets.GAME_AVERAGE_SCORE),
    SHOOTING_RANGE  (5, ApplicationTextManager.ApplicationTextsAssets.GAME_SHOOTING_RANGE_TITLE, ApplicationTextManager.ApplicationTextsAssets.GAME_SHOOTING_RANGE_DESCRIPTION, ShootingRangeGame.class, false, new ShootingRangeResultResolver(), ApplicationTextManager.ApplicationTextsAssets.GAME_SCORE, ApplicationTextManager.ApplicationTextsAssets.GAME_AVERAGE_SCORE),
    IN_THE_DARKNESS (6, ApplicationTextManager.ApplicationTextsAssets.GAME_IN_THE_DARKNESS_TITLE, ApplicationTextManager.ApplicationTextsAssets.GAME_IN_THE_DARKNESS_DESCRIPTION, InTheDarknessGame.class, true, new InTheDarknessGame.GameResultResolver(),ApplicationTextManager.ApplicationTextsAssets.GAME_SCORE_ERROR, ApplicationTextManager.ApplicationTextsAssets.GAME_AVERAGE_SCORE_ERROR);

    public interface GameResultResolver {
        AbstractTablexiaGame.GameResult getGameCupsResult(Game game);
        float getGameScoreResult(Game game);
    }

    private int                                      gameNumber;
    private String                                   menuTextKey;
    private String                                   descriptionResource;
    private Class<? extends AbstractTablexiaGame<?>> screenClass;
    private boolean                                  hasTutorial;
    private GameResultResolver                       gameResultResolver;
    private String                                   statisticsScoreDialogText;
    private String                                   statisticsAverageScoreDialogText;

    GameDefinition(int gameNumber, String nameResource, String descriptionResource, Class<? extends AbstractTablexiaGame<?>> screenClass, boolean hasTutorial, GameResultResolver gameResultResolver, String statisticsDialogText, String statisticsAverageScoreDialogText) {
        this.gameNumber = gameNumber;
        this.menuTextKey = nameResource;
        this.descriptionResource = descriptionResource;
        this.screenClass = screenClass;
        this.hasTutorial = hasTutorial;
        this.gameResultResolver = gameResultResolver;
        this.statisticsScoreDialogText = statisticsDialogText;
        this.statisticsAverageScoreDialogText = statisticsAverageScoreDialogText;
    }

    public int getGameNumber() {
        return gameNumber;
    }

    @Override
    public String getTitle() {
        return ApplicationTextManager.getInstance().getResult().get(menuTextKey);
    }

    @Override
    public String getDescription() {
        return ApplicationTextManager.getInstance().getResult().get(descriptionResource);
    }


    public String getStatisticsScoreDialogText() {
        return statisticsScoreDialogText;
    }


    public String getStatisticsAverageScoreDialogText() {
        return statisticsAverageScoreDialogText;
    }


    public Class<? extends AbstractTablexiaGame<?>> getScreenClass() {
        return screenClass;
    }

    public boolean hasTutorial() {
        return hasTutorial;
    }

    @Override
    public void performAction() {
        ApplicationBus.getInstance().publishAsync(this);
        if (TablexiaApplication.getActualScreenClass() == GameMenuScreen.class) {
            ApplicationBus.getInstance().publishAsync(new ShowGameMenuPageEvent(GameDefinition.this, true));
        } else {
            ApplicationBus.getInstance().publishAsync(new Tablexia.ChangeScreenEvent(GameMenuScreen.class,
                    TablexiaApplication.ScreenTransaction.FADE,
                    GameMenuScreen.createScreenState(GameMenuScreen.GameMenuPageDefinition.getGameMenuPageForGameDefinition(this))));
        }
    }

    @Override
    public AbstractMenu.MenuAction getMenuAction() {
        return AbstractMenu.MenuAction.CLOSE;
    }

    @Override
    public IMenuItem[] getSubmenu() {
        return null;
    }

    public GameResultResolver getGameResultResolver(){
        return gameResultResolver;
    }

    public static GameDefinition getGameDefinitionForClass(Class<? extends AbstractTablexiaGame> screenClass) {
        for (GameDefinition gameDefinition : GameDefinition.values()) {
            if (gameDefinition.screenClass.equals(screenClass)) {
                return gameDefinition;
            }
        }
        return null;
    }

    public static GameDefinition getGameDefinitionForGameNumber(int gameNumber) {
        for (GameDefinition gameDefinition : GameDefinition.values()) {
            if (gameDefinition.gameNumber == gameNumber) {
                return gameDefinition;
            }
        }
        return null;
    }

    @Override
    public Class<? extends Group> getItemGroupClass() {
        return null; // TODO
    }

    @Override
    public String[] getIcons() {
        return new String[0]; // TODO
    }
}
