/*******************************************************************************
 *     Tablexia	
 * 
 *     Copyright (C) 2013  CZ NIC z.s.p.o. <podpora at nic dot cz>
 * 
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/
package cz.nic.tablexia.game.difficulty;


import java.util.ArrayList;
import java.util.List;

import cz.nic.tablexia.TablexiaSettings;
import cz.nic.tablexia.loader.application.ApplicationTextManager;
import cz.nic.tablexia.shared.model.Game;

/**
 * Game difficulty types
 * 
 * @author Matyáš Latner
 */
public enum GameDifficulty {

    TUTORIAL(0, TablexiaSettings.EXPERIENCE_POINTS_TUTORIAL, "gamedifficulty_tutorial",  false, false),
    EASY    (1, TablexiaSettings.EXPERIENCE_POINTS_EASY,     "gamedifficulty_easy",      true, true),
    MEDIUM  (2, TablexiaSettings.EXPERIENCE_POINTS_MEDIUM,   "gamedifficulty_medium",    true, true),
    HARD    (3, TablexiaSettings.EXPERIENCE_POINTS_HARD,     "gamedifficulty_hard",      true, true);

    public static final GameDifficulty DEFAULT_DIFFICULTY = EASY;

    private int difficultyNumber;
    private int experiencePointsMultiplier;
    private String descriptionResourceKey;
    private boolean visible;
    private boolean hasScore;

    GameDifficulty(int difficultyNumber, int experiencePointsMultiplier, String descriptionResourceKey, boolean visible, boolean hasScore) {
        this.difficultyNumber = difficultyNumber;
        this.experiencePointsMultiplier = experiencePointsMultiplier;
        this.descriptionResourceKey = descriptionResourceKey;
        this.visible = visible;
        this.hasScore = hasScore;
    }

    public int getDifficultyNumber() {
        return difficultyNumber;
    }

    public String getTextDescription() {
        return ApplicationTextManager.getInstance().getText(getDescriptionResourceKey());
    }

    public boolean isVisible() {
        return visible;
    }

    public static List<GameDifficulty> getVisibleGameDifficultyList() {
        List<GameDifficulty> gameDifficultyList = new ArrayList<GameDifficulty>();
        for (GameDifficulty gameDifficulty: GameDifficulty.values()) {
            if (gameDifficulty.isVisible()) {
                gameDifficultyList.add(gameDifficulty);
            }
        }
        return gameDifficultyList;
    }

    /**
     * Returns key of description for current difficulty
     * 
     * @return key of description string resource
     */
    public String getDescriptionResourceKey() {
        return descriptionResourceKey;
    }

    public static GameDifficulty getGameDifficultyForDifficultyNumber(int difficultyNumber) {
        for (GameDifficulty gameDifficulty: GameDifficulty.values()) {
            if (gameDifficulty.difficultyNumber == difficultyNumber) {
                return gameDifficulty;
            }
        }
        return null;
    }

    public static GameDifficulty getByGame(Game game) {
        return getGameDifficultyForDifficultyNumber(game.getGameDifficulty());
    }

    public int getExperiencePointsMultiplier() {
        return experiencePointsMultiplier;
    }

    public boolean hasScore() {
        return hasScore;
    }
}
