package cz.nic.tablexia.game.games.in_the_darkness;

import java.util.Arrays;
import java.util.List;
import java.util.Random;

import cz.nic.tablexia.game.difficulty.GameDifficulty;
import cz.nic.tablexia.game.games.in_the_darkness.action.ActionType;
import cz.nic.tablexia.game.games.in_the_darkness.map.IMapProvider;
import cz.nic.tablexia.game.games.in_the_darkness.map.MapGenerator;
import cz.nic.tablexia.game.games.in_the_darkness.map.MapTutorial;
import cz.nic.tablexia.util.Log;

/**
 * Game InTheDarkness difficulty definition and mapping to game difficulty
 * 
 * @author Matyáš Latner
 *
 */
public enum InTheDarknessDifficulty {
	
	TUTORIAL(GameDifficulty.TUTORIAL,	false,	MapTutorial.class,  1, Arrays.asList(new ActionType[]{})),
	EASY	(GameDifficulty.EASY, 		true, 	MapGenerator.class, 1, Arrays.asList(new ActionType[]{})),
	MEDIUM	(GameDifficulty.MEDIUM, 	true, 	MapGenerator.class, 1, Arrays.asList(new ActionType[]{ActionType.KEY})),
	HARD	(GameDifficulty.HARD, 		true, 	MapGenerator.class, 2, Arrays.asList(new ActionType[]{ActionType.KEY}));
	
	private GameDifficulty gameDifficulty;
	private int floorCount;
	private List<ActionType> difficultyActionTypes;
	private Class<? extends IMapProvider> mapProviderClass;
	private boolean hasResults;

	InTheDarknessDifficulty(GameDifficulty gameDifficulty, boolean hasResults, Class<? extends IMapProvider> mapProviderClass, int floorCount, List<ActionType> difficultyActionTypes) {
		this.gameDifficulty			= gameDifficulty;
		this.hasResults 			= hasResults;
		this.mapProviderClass 		= mapProviderClass;
		this.floorCount 			= floorCount;
		this.difficultyActionTypes 	= difficultyActionTypes;
	}
	
	public boolean hasResults() {
		return hasResults;
	}
	
	public int getFloorCount() {
		return floorCount;
	}
	
	public boolean hasDifficultyActionType(ActionType actionType) {
		return difficultyActionTypes.contains(actionType);
	}
	
	public static InTheDarknessDifficulty getInTheDarknessDifficultyForGameDifficulty(GameDifficulty gameDifficulty) {
		for (InTheDarknessDifficulty inTheDarknessDifficulty : InTheDarknessDifficulty.values()) {
			if (inTheDarknessDifficulty.gameDifficulty == gameDifficulty) {
				return inTheDarknessDifficulty;
			}
		}
		return null;
	}
	
	public IMapProvider getMapProviderNewInstance(Random random) {
		try {
			return mapProviderClass.getConstructor(Random.class).newInstance(random);
		} catch (Exception e) {
			Log.err(InTheDarknessDifficulty.class, "Cannot create instance for IMapProvider class: " + mapProviderClass, e);
		}
		return null;
	}
	
}
