package cz.nic.tablexia.game.games.kidnapping.actors;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.ui.Image;

import java.util.HashMap;

import cz.nic.tablexia.game.common.media.GfxLibrary;
import cz.nic.tablexia.game.games.kidnapping.KidnappingGame;
import cz.nic.tablexia.game.games.kidnapping.Properties;
import cz.nic.tablexia.game.games.kidnapping.media.assets.KidnappingTextAssets;
import cz.nic.tablexia.game.games.kidnapping.media.assets.TextureTypes;
import cz.nic.tablexia.game.games.kidnapping.model.Direction;
import cz.nic.tablexia.game.games.kidnapping.model.GameState;
import cz.nic.tablexia.game.games.kidnapping.model.Position;
import cz.nic.tablexia.loader.application.ApplicationAtlasManager;
import cz.nic.tablexia.util.ScaleUtil;
import cz.nic.tablexia.util.listener.DragActorListener;

/**
 * Created by lhoracek on 5/19/15.
 */
public class Map extends Group {

    private Group                         tiles   = new ParentCullingGroup();
    private java.util.Map<Position, Tile> tileMap = new HashMap<Position, Tile>();
    private EarOverlay   overlay;
    private ReplayButton replayButton;
    private Image        correct, wrong;
    private DragActorListener debugListener = new DragActorListener(tiles);

    public Map(KidnappingGame screen, GfxLibrary library) {
        addActor(tiles);
        addActor(replayButton = new ReplayButton(screen));
        replayButton.setVisible(false);
        addActor(overlay = new EarOverlay(screen.getText(KidnappingTextAssets.INSTRUCTIONS_PAY_ATTENTION), ApplicationAtlasManager.getInstance().getColorTexture(Color.BLACK), screen.getScreenTextureRegion(screen.getGfxLibrary().get(TextureTypes.EAR))));
        addActor(correct = new Image(library.getTextureRegion(TextureTypes.CORRECT)));
        addActor(wrong = new Image(library.getTextureRegion(TextureTypes.WRONG)));
        wrong.setVisible(false);
        correct.setVisible(false);
    }

    public Tile getTile(Position position) {
        return tileMap.get(position);
    }

    public EarOverlay getOverlay() {
        return overlay;
    }

    public Image getCorrect() {
        return correct;
    }

    public Image getWrong() {
        return wrong;
    }

    public ReplayButton getReplayButton() {
        return replayButton;
    }

    public void addTile(Position position, Tile tile) {
        int baseX = getBaseX(position);
        int baseY = getBaseY(position) + (tile.getTileType().getYOffset() / 2);
        tile.setPosition(baseX, baseY);
        tile.setPosition(position);
        tileMap.put(position, tile);
        tiles.addActor(tile);
    }

    @Override
    protected void sizeChanged() {
        super.sizeChanged();
        overlay.setSize(getWidth(), getHeight());
        replayButton.setPosition(getWidth() - (replayButton.getWidth() + 50), 20);

        correct.setSize(getWidth() / 4, ScaleUtil.getHeight(correct.getWidth(), correct.getHeight(), getWidth() / 4));
        wrong.setSize(getWidth() / 4, ScaleUtil.getHeight(wrong.getWidth(), wrong.getHeight(), getWidth() / 4));
        correct.setPosition(getWidth() / 2 - correct.getWidth() / 2, getHeight() / 2 - correct.getHeight() / 2);
        wrong.setPosition(getWidth() / 2 - wrong.getWidth() / 2, getHeight() / 2 - wrong.getHeight() / 2);
    }

    public void draw(Batch batch, float parentAlpha) {
        clipBegin();
        super.draw(batch, parentAlpha);
        clipEnd();
    }

    public void setMapScale(float scaleXY) {
        tiles.setScale(scaleXY);
    }

    public float getMapScale() {
        if (tiles.getScaleY() != tiles.getScaleX()) {
            throw new IllegalStateException("Different scale");
        }
        return tiles.getScaleX();
    }

    public Group getTiles() {
        return tiles;
    }

    private static int getBaseX(Position position) {
        return (position.getX() * (Properties.TILE_BASE_WIDTH + Properties.ROAD_WIDTH)) / 2;
    }

    private static int getBaseY(Position position) {
        return -1 * ((position.getY() * (Properties.TILE_BASE_HEIGHT + Properties.ROAD_HEIGHT)) - ((position.getX() % 2) * ((Properties.TILE_BASE_HEIGHT + Properties.ROAD_HEIGHT) / 2)));
    }

    public Arrow getDirectionArrow(Position position, Direction direction) {
        Tile tile = getTile(position);
        return direction == Direction.NE ? tile.getNorthEastOut() : direction == Direction.NW ? tile.getNorthWestOut() : direction == Direction.SE ? getTile(position.getSouthEast()).getNorthWestIn() : getTile(position.getSouthWest()).getNorthEastIn();
    }

    public Arrow[] getDirectionArrow(Position position, Direction[] directions) {
        Arrow[] arrows = new Arrow[directions.length];
        for (int i = 0; i < directions.length; i++) {
            arrows[i] = getDirectionArrow(position, directions[i]);
        }
        return arrows;
    }

    @Override
    public void setDebug(boolean enabled) {
        super.setDebug(enabled);
        if (enabled) {
            tiles.addListener(debugListener);
        } else {
            tiles.removeListener(debugListener);
        }
    }

    public static class MapFactory {
        public static Map createInstance(KidnappingGame game, GfxLibrary gfxLibrary, GameState gameState) {
            Map map = new Map(game, gfxLibrary);
            for (Position position : gameState.getMap().keySet()) {
                map.addTile(position, Tile.TileFactory.createInstance(gfxLibrary, gameState.getMap().get(position)));
            }
            map.getOverlay().setVisible(false);
            return map;
        }
    }
}