/**
 * ****************************************************************************
 * Tablexia
 * <p/>
 * Copyright (C) 2013  CZ NIC z.s.p.o. <podpora at nic dot cz>
 * <p/>
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * <p/>
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p/>
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************
 */

package cz.nic.tablexia.game.games.kidnapping.actors;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Label;

import cz.nic.tablexia.TablexiaSettings;
import cz.nic.tablexia.game.common.media.GfxLibrary;
import cz.nic.tablexia.game.games.kidnapping.media.assets.TextureTypes;
import cz.nic.tablexia.game.games.kidnapping.media.assets.TileType;
import cz.nic.tablexia.game.games.kidnapping.model.Position;

/**
 * @author lhoracek
 */
public class Tile extends Group {
    TileType tileType;
    Image    street;
    Image    flag;
    Image    pathNorthWest, pathNorthEast;
    Arrow northWestIn, northEastIn, northWestOut, northEastOut;
    Label positionLabel;

    public void setPosition(Position position) {
        BitmapFont bf = new BitmapFont();
        bf.getData().setScale(3);
        if (TablexiaSettings.getInstance().isShowBoundingBoxes()) {
            addActor(positionLabel = new Label(position.toString(), new Label.LabelStyle(bf, Color.GREEN)));
        }
        sizeChanged();
    }

    @Override
    protected void sizeChanged() {
        super.sizeChanged();
        if (positionLabel != null) {
            positionLabel.setPosition(street.getWidth() / 2 - positionLabel.getWidth() / 2, street.getHeight() / 2 - positionLabel.getHeight() / 2);
        }
    }

    @Override
    public void setDebug(boolean enabled) {
        super.setDebug(enabled);
    }

    @Override
    public void setDebug(boolean enabled, boolean recursively) {
        super.setDebug(enabled, recursively);
    }

    private Tile() {
    }

    public Image getStreet() {
        return street;
    }

    public Image getFlag() {
        return flag;
    }

    public Image getPathNorthWest() {
        return pathNorthWest;
    }

    public Image getPathNorthEast() {
        return pathNorthEast;
    }

    public Arrow getNorthWestIn() {
        return northWestIn;
    }

    public Arrow getNorthEastIn() {
        return northEastIn;
    }

    public Arrow getNorthWestOut() {
        return northWestOut;
    }

    public Arrow getNorthEastOut() {
        return northEastOut;
    }

    public TileType getTileType() {
        return tileType;
    }

    public float getCenterX() {
        return (getX() + getStreet().getWidth() / 2);

    }

    public float getCenterY() {
        return (getY() + getStreet().getHeight() / 4);
    }

    public static class TileFactory {
        public static Tile createInstance(GfxLibrary gfxLibrary, TileType tileType) {
            Tile tile = new Tile();
            tile.tileType = tileType;

            TextureRegion tileTexture = gfxLibrary.getTextureRegion(tileType);
            tile.street = new Image(tileTexture);
            tile.addActor(tile.street);
            tile.street.setTouchable(Touchable.disabled);

            addPath(gfxLibrary, tile);
            addArrows(gfxLibrary, tile);

            return tile;
        }

        private static void addPath(GfxLibrary gfxLibrary, Tile tile) {
            TextureRegion flagTexture = gfxLibrary.getTextureRegion(TextureTypes.POSITION_CURRENT);
            tile.addActor(tile.flag = new Image(flagTexture));
            tile.flag.setPosition(351, -29);
            tile.flag.setVisible(false);

            TextureRegion lineNE = gfxLibrary.getTextureRegion(TextureTypes.POSITION_LINE_NESW);
            tile.addActor(tile.pathNorthEast = new Image(lineNE));
            tile.pathNorthEast.setPosition(464, 86);
            tile.pathNorthEast.setVisible(false);

            TextureRegion lineNW = gfxLibrary.getTextureRegion(TextureTypes.POSITION_LINE_NWSE);
            tile.addActor(tile.pathNorthWest = new Image(lineNW));
            tile.pathNorthWest.setPosition(59, 88);
            tile.pathNorthWest.setVisible(false);
        }

        private static void addArrows(GfxLibrary gfxLibrary, Tile tile) {
            TextureRegion arrowNE = gfxLibrary.getTextureRegion(TextureTypes.POSITION_NEXT_NE);
            TextureRegion arrowNEPressed = gfxLibrary.getTextureRegion(TextureTypes.POSITION_NEXT_NE_PRESSED);
            tile.addActor(tile.northEastOut = new Arrow(arrowNE, arrowNEPressed));
            tile.northEastOut.setPosition(531, 110);
            tile.northEastOut.setVisible(false);

            TextureRegion arrowNW = gfxLibrary.getTextureRegion(TextureTypes.POSITION_NEXT_NW);
            TextureRegion arrowNWPressed = gfxLibrary.getTextureRegion(TextureTypes.POSITION_NEXT_NW_PRESSED);
            tile.addActor(tile.northWestOut = new Arrow(arrowNW, arrowNWPressed));
            tile.northWestOut.setPosition(127, 112);
            tile.northWestOut.setVisible(false);

            TextureRegion arrowSE = gfxLibrary.getTextureRegion(TextureTypes.POSITION_NEXT_SE);
            TextureRegion arrowSEPressed = gfxLibrary.getTextureRegion(TextureTypes.POSITION_NEXT_SE_PRESSED);
            tile.addActor(tile.northWestIn = new Arrow(arrowSE, arrowSEPressed));
            tile.northWestIn.setPosition(137, 112);
            tile.northWestIn.setVisible(false);

            TextureRegion arrowSW = gfxLibrary.getTextureRegion(TextureTypes.POSITION_NEXT_SW);
            TextureRegion arrowSWPressed = gfxLibrary.getTextureRegion(TextureTypes.POSITION_NEXT_SW_PRESSED);
            tile.addActor(tile.northEastIn = new Arrow(arrowSW, arrowSWPressed));
            tile.northEastIn.setPosition(515, 106);
            tile.northEastIn.setVisible(false);
        }
    }
}
