package cz.nic.tablexia.game.games.night_watch;

import com.badlogic.gdx.audio.Music;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.actions.RunnableAction;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Stack;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.utils.Align;
import com.badlogic.gdx.utils.Scaling;

import net.dermetfan.utils.Pair;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import cz.nic.tablexia.TablexiaApplication;
import cz.nic.tablexia.TablexiaSettings;
import cz.nic.tablexia.game.AbstractTablexiaGame;
import cz.nic.tablexia.game.GameDefinition;
import cz.nic.tablexia.game.games.night_watch.assets.NightWatchAssets;
import cz.nic.tablexia.game.games.night_watch.helper.GameRulesHelper;
import cz.nic.tablexia.game.games.night_watch.helper.TextureHelper;
import cz.nic.tablexia.game.games.night_watch.solution.GameSolutionGenerator;
import cz.nic.tablexia.game.games.night_watch.solution.Solution;
import cz.nic.tablexia.game.games.night_watch.subscene.Watch;
import cz.nic.tablexia.loader.application.ApplicationTextManager;
import cz.nic.tablexia.shared.model.Game;
import cz.nic.tablexia.util.Log;
import cz.nic.tablexia.util.Utility;
import cz.nic.tablexia.util.ui.AnimatedImage;
import cz.nic.tablexia.util.ui.button.StandardTablexiaButton;
import cz.nic.tablexia.util.ui.button.TablexiaButton;
import cz.nic.tablexia.util.ui.dialog.components.AnimatedImageContentDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.TablexiaDialogComponentAdapter;
import cz.nic.tablexia.util.ui.dialog.components.TextContentDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.TwoColumnContentDialogComponent;

/**
 * Created by Václav Tarantík on 6.3.15.
 */
public class NightWatchGame extends AbstractTablexiaGame<int[][]> {
    public static class GameResultResolver implements GameDefinition.GameResultResolver {
        @Override
        public GameResult getGameCupsResult(Game game) {
            int correctWindows = LEVELS_COUNT - Integer.valueOf(game.getGameScoreValue(SCORE_WRONG_WINDOWS));
            int correctTimeCount = LEVELS_COUNT - Integer.valueOf(game.getGameScoreValue(SCORE_WRONG_TIME));
            return GameRulesHelper.getNumberOfStarsForResult(correctWindows + correctTimeCount);
        }

        @Override
        public float getGameScoreResult(Game game) {
            int wrongTime = Integer.parseInt(game.getGameScore(NightWatchGame.SCORE_WRONG_TIME, Integer.toString(LEVELS_COUNT)));
            int wrongWindows = Integer.parseInt(game.getGameScore(NightWatchGame.SCORE_WRONG_WINDOWS, Integer.toString(LEVELS_COUNT)));
            return 2 * LEVELS_COUNT - wrongTime - wrongWindows;
        }
    }

    private enum ResultMapping {

        NO_STAR_TEXT	(GameResult.NO_STAR, 	NightWatchAssets.VICTORYTEXT_NOSTAR, 		NightWatchAssets.VICTORYSPEECH_NOSTAR),
        ONE_STAR_TEXT	(GameResult.ONE_STAR, 	NightWatchAssets.VICTORYTEXT_ONESTAR, 		NightWatchAssets.VICTORYSPEECH_ONESTAR),
        TWO_STAR_TEXT	(GameResult.TWO_STAR, 	NightWatchAssets.VICTORYTEXT_TWOSTARS, 	    NightWatchAssets.VICTORYSPEECH_TWOSTAR),
        THREE_STAR_TEXT	(GameResult.THREE_STAR, NightWatchAssets.VICTORYTEXT_THREESTARS,	NightWatchAssets.VICTORYSPEECH_THREESTAR);

        private final GameResult gameResult;
        private final String textKey;
        private final String soundName;

        ResultMapping(GameResult gameResult, String textKey, String soundName) {
            this.gameResult = gameResult;
            this.textKey = textKey;
            this.soundName = soundName;
        }

        public String getTextKey() {
            return textKey;
        }

        public String getSoundName() {
            return soundName;
        }

        public static ResultMapping getResultTextMappingForGameResult(GameResult gameResult) {
            for (ResultMapping resultMapping : ResultMapping.values()) {
                if (resultMapping.gameResult.equals(gameResult)) {
                    return resultMapping;
                }
            }
            return null;
        }
    }
    private static final int LEVELS_COUNT = 8;//mox correct windows and time
    private static final int SCREEN_WIDTH = TablexiaSettings.getWorldSize();
    private static final int SCREEN_MIN_HEIGHT = TablexiaSettings.getMinWorldHeight();
    private static final int INITIAL_BG_TEXTURE_INDEX       = 0;
    private static final float SOLUTION_VISIBLE_SECONDS     = 2.0f;
    private static final float SOLUTION_DELAY               = 0.5f;
    private static final float TRANSITION_DELAY             = 2;
    private static final float TRANSITION_ANIMATION_DELAY   = 0.2f;
    private static final float TRANSITION_ANIMATION_FADE_IN = 0.2f;
    private static final float WINDOW_IMAGE_RESIZE          = 1.03f;

    private static final String SUMMARY_TEXT_WINDOWS_KEY = "victory_text_window_mistakes";
    private static final String SUMMARY_TEXT_TIME_KEY = "victory_text_time_mistakes";

    private  static final String SCORE_WRONG_WINDOWS = "windows_count";
    private  static final String SCORE_WRONG_TIME = "time_count";

    private static final int WATCH_WIDTH    = 300;
    private static final int BUTTON_X       = 800;
    private static final int BUTTON_Y       = 20;
    private static final int BUTTON_WIDTH   = 150;
    private static final int BUTTON_HEIGHT  = 80;

    private int wrongWindows = 0;
    private int wrongTime = 0;
    private boolean userPlaying = false;
    private int numberOfSelectedWindows;
    private Set<Integer> selectedWindows;

    private int currentLevel = 0;
    private int currentRound;
    private Group windowsGroup;
    private Group contentGroup;
    private Stack backgroundStack;
    private Map<Integer,List<Solution>> gameSolutions;
    private Watch watch;
    private TablexiaButton button;
    private TextureRegion clickmap;
    private Music backgroundSound;
    private String actualBackgroundSound;
    private Image backgroundImage;

    @Override
    protected void gameLoaded(Map<String, String> gameState) {
        setGameScore(SCORE_WRONG_WINDOWS, 0);
        setGameScore(SCORE_WRONG_TIME, 0);
        currentLevel = 0;
        currentRound = 0;
        contentGroup = new Group();
        contentGroup.setSize(SCREEN_WIDTH, SCREEN_MIN_HEIGHT);
        getStage().addActor(contentGroup);

        gameSolutions = GameSolutionGenerator.generateSolutions(getGameDifficulty(), getRandom());
        selectedWindows = new HashSet<Integer>();

        prepareBackground();
        prepareWindows();
        prepareWatch();
        prepareButton();
    }

    @Override
    protected void gameVisible() {
        startRound();
    }

    @Override
    protected int[][] prepareGameData(Map<String, String> gameState) {
        clickmap = getScreenTextureRegion(NightWatchAssets.WINDOWS_FOLDER + NightWatchAssets.CLICK_MAP);
        if (!clickmap.getTexture().getTextureData().isPrepared()) {
            clickmap.getTexture().getTextureData().prepare();
        }
        return Utility.createColorMap(clickmap);
    }

    private void prepareBackground() {
        backgroundStack = new Stack();
        TextureRegion backgroundTexture = getScreenTextureRegion(TextureHelper.getBackgroundTexturesNames().get(INITIAL_BG_TEXTURE_INDEX));
        backgroundImage = new Image(backgroundTexture);

        backgroundStack.addListener(new ClickListener() {
            @Override
            public boolean touchDown(InputEvent event, float x, float y, int pointer, int button) {
                Log.info(getClass(), "TOUCHED COLOR: " + getTouchedColor(x, y));
                Log.info(getClass(), "TOUCHED WINDOW: " + TextureHelper.colorMap.get(getTouchedColor(x, y)));
                final Integer touchedWindowIndex = TextureHelper.colorMap.get(getTouchedColor(x, y));
                if (touchedWindowIndex != null) {
                    int maxNumberOfWindows = GameRulesHelper.getNumberOfWindowsLightenedInCurrentRound(getGameDifficulty(), currentLevel);
                    if (numberOfSelectedWindows < maxNumberOfWindows) {
                        lightUpWindow(touchedWindowIndex).addListener(new ClickListener() {
                            @Override
                            public boolean touchDown(InputEvent event, float x, float y, int pointer, int button) {
                                selectedWindows.remove(touchedWindowIndex);
                                event.getListenerActor().remove();
                                numberOfSelectedWindows--;
                                NightWatchGame.this.button.setVisible(false);
                                return true;
                            }
                        });

                        if (numberOfSelectedWindows == maxNumberOfWindows) {
                            NightWatchGame.this.button.setVisible(true);
                            NightWatchGame.this.button.setChecked(false);
                        }
                    }
                }
                return true;
            }
        });
        backgroundStack.addActor(backgroundImage);
        contentGroup.addActor(backgroundStack);

        setActorToFullScene(backgroundStack);
    }

    private void prepareWatch() {
        watch = new Watch(WATCH_WIDTH, getScreenTextureRegion(NightWatchAssets.WATCH), getScreenTextureRegion(NightWatchAssets.HOUR_HAND), getScreenTextureRegion(NightWatchAssets.MINUTE_HAND), getScreenTextureRegion(NightWatchAssets.WATCH_ONLY), getSound(NightWatchAssets.SOUND_WATCH_1));
        watch.setPosition(getSceneWidth() / 2 - watch.getWidth() / 2, getViewportBottomY());
        watch.setDebug(TablexiaSettings.getInstance().isShowBoundingBoxes());
        contentGroup.addActor(watch);
    }

    private void prepareWindows() {
       windowsGroup = new Group();
       contentGroup.addActor(windowsGroup);
    }

    private void prepareButton() {
        String buttonText = getText("game_nocnisledovani_btncontinue_text");

        button = new StandardTablexiaButton(buttonText)
                .setButtonBounds(BUTTON_X, getViewportBottomY() + BUTTON_Y, BUTTON_WIDTH, BUTTON_HEIGHT)
                .setInputListener(new ClickListener() {
                    @Override
                    public void clicked(InputEvent event, float x, float y) {
                        userPlaying = false;
                        button.setVisible(false);
                        //disabling background so windows cannot be clicked until
                        enableClickables(false);
                        addAction(Actions.sequence(evaluate(), Actions.delay(SOLUTION_VISIBLE_SECONDS), resetScene(), Actions.delay(SOLUTION_DELAY), decideWhatFollows()));
                    }
                });
        button.setVisible(false);

        contentGroup.addActor(button);
    }

    //initial animation
    private void startRound() {
        backgroundStack.clearChildren();
        backgroundStack.addActor(backgroundImage);
        enableClickables(false);
        playBackgroundSound(NightWatchAssets.SOUND_DAY);
        float currentLidTime = GameRulesHelper.getCurrentLidTime(getGameDifficulty(), currentLevel, currentRound);
        addAction(Actions.sequence(Actions.delay(TRANSITION_DELAY), animateDaytimeTransition(true), Actions.delay(TRANSITION_DELAY), showSolution(),
                Actions.delay(currentLidTime), resetScene(), animateDaytimeTransition(false), play()));
    }

    //shows generated solution for this round
    private RunnableAction showSolution() {
        return new RunnableAction() {
            @Override
            public void run() {
                Log.info(getClass(), "SHOW SOLUTION");

                Solution currentSolution = gameSolutions.get(currentLevel).get(currentRound);
                Log.info(getClass(), "Solution: ");
                for (int i = 0; i < currentSolution.getWindowSequence().size(); i++) {
                    Log.info(getClass(),currentSolution.getWindowSequence().get(i).toString());
                    lightUpWindow(currentSolution.getWindowSequence().get(i));
                }
                Log.info(getClass(), "TIME: " + currentSolution.getTime());
                watch.setTime(currentSolution.getTime());
            }
        };
    }

    private RunnableAction resetScene() {
        return new RunnableAction() {
            @Override
            public void run() {
                Log.info(getClass(), "RESET SCENE");
                selectedWindows.clear();
                resetWindows();
                watch.resetWatch();
            }
        };
    }

    private RunnableAction play() {
        return new RunnableAction() {
            @Override
            public void run() {
                Log.info(getClass(), "PLAY");
                userPlaying = true;
                enableClickables(true);
            }
        };
    }

    private RunnableAction evaluate() {
        return new RunnableAction() {
            @Override
            public void run() {
                Log.info(getClass(), "EVALUATE");
                int userSetTime = watch.getCurrentTime();
                List<Integer> windows = new ArrayList<Integer>(selectedWindows);

                Solution correctSolution = gameSolutions.get(currentLevel).get(currentRound);
                boolean correct = correctSolution.check(windows, userSetTime);
                if (!correctSolution.checkWindows(windows) && !isGameFinished()) {
                    setGameScore(SCORE_WRONG_WINDOWS, ++wrongWindows);
                }
                if(!correctSolution.checkTime(userSetTime) && !isGameFinished()){
                    setGameScore(SCORE_WRONG_TIME, ++wrongTime);
                }
                Log.info(getClass(), "CORRECT: " + correct);

                for (int i = 0; i < correctSolution.getWindowSequence().size(); i++) {
                    lightUpWindowWithColor(correctSolution.getWindowSequence().get(i), Color.GREEN);
                }

                watch.showCorrectTime(correctSolution.getTime());

                if (!correct) {
                    for (int i = 0; i < windows.size(); i++) {
                        if (!correctSolution.getWindowSequence().contains(windows.get(i))) {
                            lightUpWindowWithColor(windows.get(i), Color.RED);
                        }
                    }
                    if (correctSolution.getTime() != userSetTime) {
                        watch.showWrongTime(userSetTime);
                    }
                }
            }
        };
    }

    private RunnableAction animateDaytimeTransition(final boolean dayToNight) {
        return new RunnableAction() {
            @Override
            public void run() {
                Log.info(getClass(), "ANIMATE DAY TO NIGHT: " + dayToNight);
                List<String> dayTimeAnimationTextures = TextureHelper.getDaytimeTransitionTextureNames(dayToNight);
                for (int i = 1; i < dayTimeAnimationTextures.size(); i++) {
                    Image replacingImage = new Image(getScreenTextureRegion(dayTimeAnimationTextures.get(i)));
                    replacingImage.setSize(SCREEN_WIDTH, SCREEN_MIN_HEIGHT);
                    replacingImage.getColor().a = 0f;
                    backgroundStack.addActor(replacingImage);
                    replacingImage.addAction(Actions.sequence(Actions.delay(i * TRANSITION_ANIMATION_DELAY), Actions.fadeIn(TRANSITION_ANIMATION_FADE_IN)));
                }
                playBackgroundSound(dayToNight ? NightWatchAssets.SOUND_NIGHT : NightWatchAssets.SOUND_DAY);
                playRandomTransitionSound(dayToNight);
            }
        };
    }


    private RunnableAction decideWhatFollows() {
        return new RunnableAction() {

            @Override
            public void run() {
                int numberOfRoundsInThisLevel = GameRulesHelper.getRoundsPerLevelCount(getGameDifficulty(), currentLevel);
                int numberOfLevelsForCurrentDifficulty = GameRulesHelper.getNumberOfLevelsPerGame();
                currentRound++;
                if (currentRound < numberOfRoundsInThisLevel) {
                    startRound();
                } else {
                    currentLevel++;
                    if (currentLevel < numberOfLevelsForCurrentDifficulty) {
                        currentRound = 0;
                        startRound();
                    } else {
                        Log.info(this.getClass(), "Finishing...");
                        gameComplete();
                    }
                }
            }
        };
    }

    private Image lightUpWindow(final int index) {
        return lightUpWindowWithColor(index, null);
    }

    private Image lightUpWindowWithColor(final int index, Color color) {
        List<Pair<Integer, Integer>> windowPositions = TextureHelper.getWindowsPositions(getGameDifficulty());
        final List<String> windowsNamesList = TextureHelper.getWindowsNames(getGameDifficulty());
        final Image windowImage = new Image(getScreenTextureRegion(windowsNamesList.get(index)));
        if (color != null) {
            windowImage.setColor(color);
        }
        windowImage.setPosition(windowPositions.get(index).getKey(), windowPositions.get(index).getValue());

        windowImage.setHeight(windowImage.getHeight() * WINDOW_IMAGE_RESIZE);
        windowImage.setWidth(windowImage.getWidth() * WINDOW_IMAGE_RESIZE);

        selectedWindows.add(index);
        windowsGroup.addActor(windowImage);
        //TODO add reset scene action here

        //user is playing, increment number of lid windows
        int maxNumberOfWindows = GameRulesHelper.getNumberOfWindowsLightenedInCurrentRound(getGameDifficulty(), currentLevel);
        numberOfSelectedWindows++;
        if (numberOfSelectedWindows == maxNumberOfWindows && userPlaying) {
            button.setVisible(true);
        }

        return windowImage;
    }

    //method enables or disables all clickables so it cannot be clicked while animating or during screen transitions
    private void enableClickables(boolean enable) {
        Touchable touchable;
        if (enable) {
            touchable = Touchable.enabled;
        } else {
            touchable = Touchable.disabled;
        }
        backgroundStack.setTouchable(touchable);
        watch.enable(enable);
    }

    private void resetWindows() {
        windowsGroup.clearChildren();
        numberOfSelectedWindows = 0;
    }

    private Color getTouchedColor(float x, float y) {
        int clickX = (int) (x / getSceneWidth() * clickmap.getRegionWidth());
        int clickY = clickmap.getRegionHeight() - (int) (y / getSceneOuterHeight() * clickmap.getRegionHeight());

        if (clickY < 0 || clickY > clickmap.getRegionHeight()) {
            return Color.BLACK;
        }

        return new Color(getData()[clickX][clickY]);
    }

    private void playBackgroundSound(String name) {
        // don't play again background sound, which is actually playing
        if (actualBackgroundSound != null && actualBackgroundSound.equals(name) && backgroundSound.isPlaying()) {
            return;
        }

        if (backgroundSound != null && backgroundSound.isPlaying()) {
            backgroundSound.stop();
        }

        backgroundSound = getMusic(name);
        backgroundSound.setLooping(true);
        backgroundSound.play();
        actualBackgroundSound = name;
    }

    private void playRandomTransitionSound(boolean dayToNight) {
        List<String> sounds = dayToNight ? NightWatchAssets.getEveningTransitionSounds() : NightWatchAssets.getMorningTransitionSounds();
        getSound(sounds.get(getRandom().nextInt(sounds.size()))).play();
    }

    @Override
    protected List<SummaryMessage> getSummaryMessageForGameResult(Game game) {
        return Arrays.asList(new SummaryMessage(SummaryImage.STATS, getFormattedText(SUMMARY_TEXT_WINDOWS_KEY, game.getGameScore(SCORE_WRONG_WINDOWS, "0"))),
                new SummaryMessage(SummaryImage.TIME, getFormattedText(SUMMARY_TEXT_TIME_KEY, game.getGameScore(SCORE_WRONG_TIME, "0"))));
    }

    @Override
    protected String getTextKeyForGameResult(GameResult gameResult) {
        ResultMapping mapping = ResultMapping.getResultTextMappingForGameResult(gameResult);
        if (mapping != null) {
            return mapping.getTextKey();
        }

        return null;
    }

    @Override
    protected String getSoundNameForGameResult(GameResult gameResult) {
        ResultMapping mapping = ResultMapping.getResultTextMappingForGameResult(gameResult);
        if (mapping != null) {
            return mapping.getSoundName();
        }
        return null;
    }

    @Override
    protected void screenResized(int width, int height) {
        if (watch != null) {
            watch.setY(getViewportBottomY());
        }

        if (button != null) {
            button.setY(getViewportBottomY() + BUTTON_Y);
        }
    }

    @Override
    protected void gameDisposed() {
        if (backgroundSound != null) {
            backgroundSound.stop();
            backgroundSound.dispose();
        }
    }

    @Override
    protected void prepareGameSoundAssetNames(List<String> soundsFileNames) {
        soundsFileNames.addAll(NightWatchAssets.getSoundsToLoad());
    }

    @Override
    public void gameComplete() {
        if (backgroundSound != null) {
            backgroundSound.stop();
        }
        super.gameComplete();
    }

    @Override
    protected void gamePaused(Map<String, String> gameState) {
        if (backgroundSound != null) {
            backgroundSound.pause();
        }
    }

    @Override
    protected void gameResumed() {
        if (backgroundSound != null) {
            backgroundSound.play();
        }
    }


//////////////////////////////////////////// PRELOADER

    private static final String PRELOADER_ANIM_IMAGE            = "preloader_anim";
    private static final int    PRELOADER_ANIM_FRAMES           = 19;
    private static final float  PRELOADER_ANIM_FRAME_DURATION   = 0.5f;
    private static final String PRELOADER_TEXT_KEY             	= ApplicationTextManager.ApplicationTextsAssets.GAME_NIGHT_WATCH_PRELOADER_TEXT;

    private static final Scaling        PRELOADER_IMAGE_SCALING 		    = Scaling.fit;
    private static final int 			PRELOADER_TEXT_ALIGN 				= Align.left;
    private static final float 			PRELOADER_TEXT_PADDING 				= 10f;
    private static final float 			PRELOADER_LEFT_COLUMN_WIDTH_RATIO 	= 1f / 4;
    private static final float 			PRELOADER_RIGHT_COLUMN_RATIO 		= 5f / 8;
    private static final float 			PRELOADER_ROW_HEIGHT 				= 1f / 3;

    @Override
    public void preparePreloaderContent(float width, float height, TablexiaApplication.PreloaderAssetsManager preloaderAssetsManager, List<TablexiaDialogComponentAdapter> components) {
        AnimatedImage preloaderImage = new AnimatedImage(preloaderAssetsManager.getAnimation(PRELOADER_ANIM_IMAGE, PRELOADER_ANIM_FRAMES, PRELOADER_ANIM_FRAME_DURATION), false);
        preloaderImage.startAnimationLoop();
        String preloaderText = preloaderAssetsManager.getText(PRELOADER_TEXT_KEY);
        components.add(new TwoColumnContentDialogComponent(
                new AnimatedImageContentDialogComponent(preloaderImage, PRELOADER_IMAGE_SCALING),
                new TextContentDialogComponent(preloaderText, Integer.valueOf(PRELOADER_TEXT_ALIGN), Float.valueOf(PRELOADER_TEXT_PADDING)),
                PRELOADER_LEFT_COLUMN_WIDTH_RATIO,
                PRELOADER_RIGHT_COLUMN_RATIO,
                PRELOADER_ROW_HEIGHT));
    }

    @Override
    public boolean preloaderHasSpeech() {
        return true;
    }


//////////////////////////////////////////// DEBUG - FORCE GAME END

    @Override
    protected void forceGameEnd(GameResult result) {
        int wrongWindows;
        int wrongTimeCount;

        switch (result) {
            case NO_STAR:
                wrongWindows = LEVELS_COUNT - 1;
                wrongTimeCount = LEVELS_COUNT - 1;
                break;
            case ONE_STAR:
                wrongWindows = ((2 * LEVELS_COUNT) - GameRulesHelper.SCORE_ONE_STAR - 1) / 2;
                wrongTimeCount = wrongWindows;
                break;
            case TWO_STAR:
                wrongWindows = ((2 * LEVELS_COUNT) - GameRulesHelper.SCORE_TWO_STARS - 1) / 2;
                wrongTimeCount = wrongWindows;
                break;
            default:
                wrongWindows = ((2 * LEVELS_COUNT) - GameRulesHelper.SCORE_THREE_STARS - 1) / 2;
                wrongTimeCount = wrongWindows;
                break;
        }

        setGameScore(SCORE_WRONG_WINDOWS, Integer.toString(wrongWindows));
        setGameScore(SCORE_WRONG_TIME,    Integer.toString(wrongTimeCount));

        waitForActualGameScore();

        endGame();
        showGameResultDialog();
    }
}
