package cz.nic.tablexia.game.games.night_watch.helper;

import cz.nic.tablexia.game.AbstractTablexiaGame;
import cz.nic.tablexia.game.difficulty.GameDifficulty;

/**
 * Created by Vendik on 30.4.2015.
 */
public class GameRulesHelper {
    //time difference is divided by number of rounds in each level showing the same number of windows
    private static final float  STARTING_LID_TIME   = 1.5f;
    private static final float  END_LID_TIME        = 0.7f;
    private static final int    NUMBER_OF_LEVELS    = 2;


    public static final int     SCORE_THREE_STARS   = 13;
    public static final int     SCORE_TWO_STARS     = 8;
    public static final int     SCORE_ONE_STAR      = 4;

    public static int getRoundsPerLevelCount(GameDifficulty gameDifficulty, int level) {
        switch (gameDifficulty) {
            case EASY:
                if (level == 0) {
                    return 4;
                } else if (level == 1) {
                    return 4;
                }
            case MEDIUM:
                if (level == 0) {
                    return 3;
                } else if (level == 1) {
                    return 5;
                }
            case HARD:
                if (level == 0) {
                    return 5;
                } else if (level == 1) {
                    return 3;
                }
            default:
                throw new IllegalArgumentException();
        }
    }

    public static int getNumberOfWindowsLightenedInCurrentRound(GameDifficulty gameDifficulty, int level) {
        switch (gameDifficulty) {
            case EASY:
                if (level == 0) {
                    return 1;
                } else if (level == 1) {
                    return 2;
                }
            case MEDIUM:
                if (level == 0) {
                    return 1;
                } else if (level == 1) {
                    return 2;
                }
            case HARD:
                if (level == 0) {
                    return 2;
                } else if (level == 1) {
                    return 3;
                }
            default:
                throw new IllegalArgumentException();
        }
    }

    /*
 * returns the number of the windows in scene according to current scene
 * number
 */
    public static int getNumberOfWindowsForDifficulty(GameDifficulty gameDifficulty) {
        switch (gameDifficulty) {
            case EASY:
                return 16;
            case MEDIUM:
                return 24;
            case HARD:
                return 32;
            default:
                throw new IllegalStateException("Illegal difficulty parameter value!");
        }
    }

    public static int getNumberOfLevelsPerGame(){
        return NUMBER_OF_LEVELS;
    }

    public static float getCurrentLidTime(GameDifficulty gameDifficulty,int level,int currentRound){
        int numberOfRoundsInCurrentLevel = getRoundsPerLevelCount(gameDifficulty, level);
        float timeUnit = (STARTING_LID_TIME - END_LID_TIME) / numberOfRoundsInCurrentLevel;
        return STARTING_LID_TIME - (timeUnit * currentRound);
    }

    public static AbstractTablexiaGame.GameResult getNumberOfStarsForResult(int points){
        if (points > SCORE_THREE_STARS) {
            return AbstractTablexiaGame.GameResult.THREE_STAR;
        } else if (points > SCORE_TWO_STARS) {
            return AbstractTablexiaGame.GameResult.TWO_STAR;
        } else if (points > SCORE_ONE_STAR) {
            return AbstractTablexiaGame.GameResult.ONE_STAR;
        }
        return AbstractTablexiaGame.GameResult.NO_STAR;
    }

}
