package cz.nic.tablexia.game.games.shooting_range.tools;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Pixmap;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.scenes.scene2d.utils.TextureRegionDrawable;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import cz.nic.tablexia.game.common.media.GfxLibrary;
import cz.nic.tablexia.game.games.shooting_range.actors.Target;
import cz.nic.tablexia.game.games.shooting_range.media.TextureType;

/**
 * Created by lhoracek on 7/14/15.
 */
public class PixelPerfectHitEvaluator implements HitEvaluator {
    private Map<Texture, Pixmap> pixmapCache = new HashMap<Texture, Pixmap>();

    @Override
    public void dispose() {
        for (Pixmap pixmap : pixmapCache.values()) {
            pixmap.dispose();
        }
    }

    public PixelPerfectHitEvaluator(GfxLibrary gfxLibrary, List<TextureType> targetTypes) {
        init(gfxLibrary, targetTypes);
    }

    private void init(GfxLibrary gfxLibrary, List<TextureType> targetTypes) {
        for (TextureType targetType : targetTypes) {
            getPixmap(gfxLibrary.getTextureRegion(targetType).getTexture());
        }
    }

    private Pixmap getPixmap(Texture texture) {
        if (pixmapCache.get(texture) == null) {
            texture.getTextureData().prepare();
            Pixmap pixmap = texture.getTextureData().consumePixmap();
            pixmapCache.put(texture, pixmap);
        }
        return pixmapCache.get(texture);
    }

    @Override
    public boolean hit(Target target, float x, float y) {
        // Pixel perfect hit decision
        if (!target.isShot()) {
            TextureRegion textureRegion = ((TextureRegionDrawable) target.getDrawable()).getRegion();
            Texture texture = textureRegion.getTexture();
            Pixmap pixmap = getPixmap(texture);
            int pixel = pixmap.getPixel(textureRegion.getRegionX() + (int) x, textureRegion.getRegionY() + textureRegion.getRegionHeight() - (int) y);
            if (new Color(pixel).a != 0) {
                return true;
            }
        }
        return false;
    }
}
