package cz.nic.tablexia.game.ranksystem;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import cz.nic.tablexia.bus.ApplicationBus;
import cz.nic.tablexia.game.AbstractTablexiaGame;
import cz.nic.tablexia.game.GameDefinition;
import cz.nic.tablexia.game.difficulty.GameDifficulty;
import cz.nic.tablexia.loader.application.ApplicationAtlasManager;
import cz.nic.tablexia.loader.application.ApplicationTextManager;
import cz.nic.tablexia.model.game.GameDAO;
import cz.nic.tablexia.shared.model.Game;
import cz.nic.tablexia.shared.model.User;
import cz.nic.tablexia.util.Log;

/**
 * Created by bcx on 4/15/16.
 */
public class UserRankManager {
    public enum UserRank {
        RANK_NONE ( 0, false, false, null,                                                       ApplicationAtlasManager.BADGE_NONE, ApplicationAtlasManager.BADGE_NONE_ICON, new int[]{  0,   0,   0,   0,   0,   0}),
        RANK_I    ( 1, true,  true,  ApplicationTextManager.ApplicationTextsAssets.USER_RANK_1,  ApplicationAtlasManager.BADGE_1,    ApplicationAtlasManager.BADGE_1_ICON,    new int[]{  3,   3,   3,   3,   3,   3}),
        RANK_II   ( 2, true,  true,  ApplicationTextManager.ApplicationTextsAssets.USER_RANK_2,  ApplicationAtlasManager.BADGE_2,    ApplicationAtlasManager.BADGE_2_ICON,    new int[]{  9,   9,   9,   9,   9,   9}),
        RANK_III  ( 3, true,  true,  ApplicationTextManager.ApplicationTextsAssets.USER_RANK_3,  ApplicationAtlasManager.BADGE_3,    ApplicationAtlasManager.BADGE_3_ICON,    new int[]{ 18,  18,  18,  18,  18,  18}),
        RANK_IV   ( 4, true,  true,  ApplicationTextManager.ApplicationTextsAssets.USER_RANK_4,  ApplicationAtlasManager.BADGE_4,    ApplicationAtlasManager.BADGE_4_ICON,    new int[]{ 30,  30,  30,  30,  30,  30}),
        RANK_V    ( 5, true,  true,  ApplicationTextManager.ApplicationTextsAssets.USER_RANK_5,  ApplicationAtlasManager.BADGE_5,    ApplicationAtlasManager.BADGE_5_ICON,    new int[]{ 48,  48,  48,  48,  48,  48}),
        RANK_VI   ( 6, true,  true,  ApplicationTextManager.ApplicationTextsAssets.USER_RANK_6,  ApplicationAtlasManager.BADGE_6,    ApplicationAtlasManager.BADGE_6_ICON,    new int[]{ 73,  73,  73,  73,  73,  73}),
        RANK_VII  ( 7, true,  true,  ApplicationTextManager.ApplicationTextsAssets.USER_RANK_7,  ApplicationAtlasManager.BADGE_7,    ApplicationAtlasManager.BADGE_7_ICON,    new int[]{105, 105, 105, 105, 105, 105}),
        RANK_VIII ( 8, true,  true,  ApplicationTextManager.ApplicationTextsAssets.USER_RANK_8,  ApplicationAtlasManager.BADGE_8,    ApplicationAtlasManager.BADGE_8_ICON,    new int[]{145, 145, 145, 145, 145, 145}),
        RANK_IX   ( 9, true,  true,  ApplicationTextManager.ApplicationTextsAssets.USER_RANK_9,  ApplicationAtlasManager.BADGE_9,    ApplicationAtlasManager.BADGE_9_ICON,    new int[]{193, 193, 193, 193, 193, 193}),
        RANK_X    (10, true,  true,  ApplicationTextManager.ApplicationTextsAssets.USER_RANK_10, ApplicationAtlasManager.BADGE_10,   ApplicationAtlasManager.BADGE_10_ICON,   new int[]{248, 248, 248, 248, 248, 248}),
        RANK_XI   (11, true,  true,  ApplicationTextManager.ApplicationTextsAssets.USER_RANK_11, ApplicationAtlasManager.BADGE_11,   ApplicationAtlasManager.BADGE_11_ICON,   new int[]{316, 316, 316, 316, 316, 316});

        private int id;
        private String nameKey;
        private String badgeImage, badgeIcon;
        private int[] experiencePointsNeeded;

        private boolean userMenu, badgeBoard;

        UserRank(int id, boolean userMenu, boolean badgeBoard, String nameKey, String badgeImage, String badgeIcon, int[] experiencePointsNeeded) {
            this.id = id;
            this.nameKey = nameKey;
            this.badgeImage = badgeImage;
            this.badgeIcon = badgeIcon;
            this.experiencePointsNeeded = experiencePointsNeeded;

            this.userMenu = userMenu;
            this.badgeBoard = badgeBoard;
        }

        public int getId() {
            return id;
        }

        public static UserRank getInitialRank() {
            return RANK_NONE;
        }

        public static UserRank getRank(int id) {
            for(UserRank rank : values()) {
                if(rank.getId() == id)
                    return rank;
            }

            return null;
        }

        public static UserRank getNextRank(UserRank rank) {
            UserRank next = UserRank.getRank(rank.getId() + 1);
            return next != null ? next : rank;
        }

        public boolean hasBadgeName() {
            return nameKey != null;
        }

        public boolean isShownOnUserMenu() {
            return userMenu;
        }

        public boolean isShownOnBadgeBoard() {
            return badgeBoard;
        }

        //TODO - Don't return keys, return actual texture regions and texts...
        public String getBadgeImageKey() {
            return badgeImage;
        }

        public String getBadgeIconKey() {
            return badgeIcon;
        }

        public String getNameKey() {
            return nameKey;
        }

        public int[] getExperiencePointsNeeded() {
            return experiencePointsNeeded;
        }
    }

    public static class RankProgress {
        int minXP, currXP, nextXP;

        public RankProgress(int minXP, int currXP, int nextXP) {
            this.minXP = minXP;
            this.currXP = currXP;
            this.nextXP = nextXP;
        }

        public int getMinXP() {
            return minXP;
        }

        public void setMinXP(int minXP) {
            this.minXP = minXP;
        }

        public int getCurrXP() {
            return currXP;
        }

        public void setCurrXP(int currXP) {
            this.currXP = currXP;
        }

        public int getNextXP() {
            return nextXP;
        }

        public void setNextXP(int nextXP) {
            this.nextXP = nextXP;
        }


        public float getPercentDone() {
            if(nextXP - minXP == 0) return 1f; //divide by zero case
            return (currXP - minXP) / (float) (nextXP - minXP);
        }

        @Override
        public String toString() {
            return "Min: " + minXP + "   Curr: " + currXP + "   NextRank: " + nextXP;
        }
    }

    //FIELDS
    private static UserRankManager instance;

    //Caches users progress
    private Map<Long, RankProgress[]> gameRankProgressMap;

    //Caches users ranks
    private Map<Long, UserRank> userRankMap;

    //Keeps track of user that need to be refreshed
    private Map<Long, Boolean> userRefreshMap;

    //When was the last time user ranked up
    private Map<Long, Long> userLastRankUpTime;

    private UserRankManager() {
        gameRankProgressMap = new HashMap<Long, RankProgress[]>();
        userRankMap         = new HashMap<Long, UserRank>();
        userRefreshMap      = new HashMap<Long, Boolean>();
        userLastRankUpTime  = new HashMap<Long, Long>();
    }

    public static final UserRankManager getInstance() {
        if(instance == null)
            instance = new UserRankManager();

        return instance;
    }

    private boolean isRankRefreshNeeded(User user) {
        if(userRefreshMap.get(user.getId()) == null || userRefreshMap.get(user.getId())) return true;
        return false;
    }

    public void setRankRefreshNeeded(User user, boolean refreshNeeded) {
        userRefreshMap.put(user.getId(), refreshNeeded);
    }

    public RankProgress[] getGameRankProgresses(User user) {
        if(isRankRefreshNeeded(user))
            refreshUserRank(user);

        RankProgress[] result = gameRankProgressMap.get(user.getId());
        if(result == null) {
            refreshUserRank(user);
            result = gameRankProgressMap.get(user.getId());
        }

        return result;
    }

    public RankProgress getGameRankProgress(User user, GameDefinition game) {
        if(isRankRefreshNeeded(user))
            refreshUserRank(user);


        RankProgress[] result = gameRankProgressMap.get(user.getId());
        if(result == null) {
            refreshUserRank(user);
            result = gameRankProgressMap.get(user.getId());
        }

        return result[game.getGameNumber() - 1];
    }

    public RankProgress getTotalUserRankProgress(User user) {
        RankProgress resultRankProgress = new RankProgress(0, 0, 0);

        RankProgress[] gameRankProgresses =  UserRankManager.getInstance().getGameRankProgresses(user);

        for(RankProgress rankProgress : gameRankProgresses) {
            int min = rankProgress.getMinXP();
            int curr = rankProgress.getCurrXP() > rankProgress.getNextXP() ? rankProgress.getNextXP() : rankProgress.getCurrXP();
            int next = rankProgress.getNextXP();

            resultRankProgress.setMinXP(resultRankProgress.getMinXP() + min);
            resultRankProgress.setCurrXP(resultRankProgress.getCurrXP() + curr);
            resultRankProgress.setNextXP(resultRankProgress.getNextXP() + next);
        }

        return resultRankProgress;
    }

    public UserRank getRank(User user) {
        if(isRankRefreshNeeded(user))
            refreshUserRank(user);

        return userRankMap.get(user.getId());
    }

    public long getLastTimeRankedUp(User user) {
        long result = 0;

        if(userLastRankUpTime.containsKey(user.getId()))
            result = userLastRankUpTime.get(user.getId());

        return result;
    }

    private int getExperiencePoints(int cupsCount, int difficulty) {
        return cupsCount * difficulty;
    }

    public synchronized void forceRefreshUserRank(User user) {
        setRankRefreshNeeded(user, true);
        refreshUserRank(user);
    }

    private synchronized void refreshUserRank(User user) {
        if(!isRankRefreshNeeded(user)) {
            return;
        }

        setRankRefreshNeeded(user, false);

        UserRank oldRank;
        boolean firstRefresh = false;
        if((oldRank = userRankMap.get(user.getId())) == null) {
            oldRank = UserRank.getInitialRank();
            userRankMap.put(user.getId(), oldRank);
            firstRefresh = true;
        }

        UserRank currRank = UserRank.getInitialRank();
        UserRank nextRank = UserRank.getNextRank(currRank);

        //TODO cache 'em for currently logged user
        List<Game> userGames = GameDAO.selectUserGamesForRankManager(user.getId());

        int[] gamesExperiencePoints = new int[GameDefinition.values().length];

        Log.info(getClass(), "Calculating rank for user: " + user.getName() + " from " + userGames.size() + " games.");
        for(Game game : userGames) {
            if(!game.isFinished()) continue;
            if(game.getGameDifficulty() == GameDifficulty.TUTORIAL.getDifficultyNumber()) continue;

            AbstractTablexiaGame.GameResult gameResult = GameDAO.getGameResult(game);
            int experiencePoints = getExperiencePoints(gameResult.getStarCount(), game.getGameDifficulty());
            int gameNumber = game.getGameNumber();

            //Increases games cups by starCount
            gamesExperiencePoints[gameNumber - 1] += experiencePoints;

            //Ignore experience points If user got more than is requested for the next rank
            if(gamesExperiencePoints[gameNumber - 1] > nextRank.getExperiencePointsNeeded()[gameNumber - 1])
                gamesExperiencePoints[gameNumber - 1] = nextRank.getExperiencePointsNeeded()[gameNumber - 1];

            //checking rank...
            boolean result = true;

            for(int i = 0; i < nextRank.getExperiencePointsNeeded().length; i++) {
                if(gamesExperiencePoints[i] < nextRank.getExperiencePointsNeeded()[i]) {
                    result = false;
                    break;
                }
            }

            //Rank up
            if(result) {
                currRank = nextRank;
                nextRank = UserRank.getNextRank(currRank);
                userLastRankUpTime.put(user.getId(), game.getEndTime());
            }
        }

        RankProgress[] gameRankProgresses = new RankProgress[GameDefinition.values().length];
        for(int i = 0; i < gameRankProgresses.length; i++) {
            gameRankProgresses[i] = new RankProgress(currRank.getExperiencePointsNeeded()[i], gamesExperiencePoints[i], nextRank.getExperiencePointsNeeded()[i]);
        }

        gameRankProgressMap.put(user.getId(), gameRankProgresses);
        userRankMap.put(user.getId(), currRank);

        if(oldRank != null && currRank != null && oldRank != currRank && !firstRefresh && currRank.getId() > oldRank.getId()) {
            //User has ranked up since the last time we checked....
            ApplicationBus.getInstance().post(new UserRankUpEvent(user, oldRank, currRank)).asynchronously();
        }
    }

    public static class UserRankUpEvent implements ApplicationBus.ApplicationEvent {
        private User user;
        private UserRank previousRank, newRank;

        private UserRankUpEvent(User user, UserRank previousRank, UserRank newRank) {
            this.previousRank = previousRank;
            this.newRank = newRank;
            this.user = user;
        }

        public User getUser() {
            return user;
        }

        public UserRank getPreviousRank() {
            return previousRank;
        }

        public UserRank getNewRank() {
            return newRank;
        }
    }
}