/*******************************************************************************
 *     Tablexia
 *
 *     Copyright (C) 2013  CZ NIC z.s.p.o. <podpora at nic dot cz>
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/

package cz.nic.tablexia.game.trophy;

import cz.nic.tablexia.game.GameDefinition;
import cz.nic.tablexia.game.difficulty.GameDifficulty;
import cz.nic.tablexia.loader.application.ApplicationTextManager;
import cz.nic.tablexia.shared.model.User;
import cz.nic.tablexia.model.game.GameTrophy;

/**
 * Definitions of game trophies
 *
 * @author Matyáš Latner, Vaclav Tarantik
 */
public enum GameTrophyDefinition implements ITrophyDefinition {

    ROBBERY_PLAY1(GameDefinition.ROBBERY, GameTrophyTypeDefinition.PLAY_LIMIT_1, "robbery_1_finished", "robbery_1_finished_description", 1, new GameTrophy.NumberOfTimesFinished()), //0
    ROBBERY_PLAY2(GameDefinition.ROBBERY, GameTrophyTypeDefinition.PLAY_LIMIT_2, "robbery_5_finished", "robbery_5_finished_description", 5, new GameTrophy.NumberOfTimesFinished()), //1
    ROBBERY_PLAY3(GameDefinition.ROBBERY, GameTrophyTypeDefinition.PLAY_LIMIT_3, "robbery_10_finished", "robbery_10_finished_description", 10, new GameTrophy.NumberOfTimesFinished()), //2
    ROBBERY_DIFF1(GameDefinition.ROBBERY, GameTrophyTypeDefinition.DIFFICULTY_LIMIT_1, "robbery_1_limit", "robbery_1_limit_description", GameDifficulty.MEDIUM.getDifficultyNumber(), new GameTrophy.ThreeStarsReceived()), //3
    ROBBERY_DIFF2(GameDefinition.ROBBERY, GameTrophyTypeDefinition.DIFFICULTY_LIMIT_2, "robbery_2_limit", "robbery_2_limit_description", GameDifficulty.HARD.getDifficultyNumber(), new GameTrophy.ThreeStarsReceived()),//4

    PURSUIT_PLAY1(GameDefinition.PURSUIT, GameTrophyTypeDefinition.PLAY_LIMIT_1, "pursuit_1_finished",  "pursuit_1_finished_description", 1, new GameTrophy.NumberOfTimesFinished()), //5
    PURSUIT_PLAY2(GameDefinition.PURSUIT, GameTrophyTypeDefinition.PLAY_LIMIT_2, "pursuit_5_finished",  "pursuit_5_finished_description", 5, new GameTrophy.NumberOfTimesFinished()), //6
    PURSUIT_PLAY3(GameDefinition.PURSUIT, GameTrophyTypeDefinition.PLAY_LIMIT_3, "pursuit_10_finished",  "pursuit_10_finished_description", 10, new GameTrophy.NumberOfTimesFinished()), //7
    PURSUIT_DIFF1(GameDefinition.PURSUIT, GameTrophyTypeDefinition.DIFFICULTY_LIMIT_1, "pursuit_1_limit", "pursuit_1_limit_description", GameDifficulty.MEDIUM.getDifficultyNumber(), new GameTrophy.ThreeStarsReceived()), //8
    PURSUIT_DIFF2(GameDefinition.PURSUIT, GameTrophyTypeDefinition.DIFFICULTY_LIMIT_2, "pursuit_2_limit", "pursuit_2_limit_description", GameDifficulty.HARD.getDifficultyNumber(), new GameTrophy.ThreeStarsReceived()),//9

    KIDNAPPING_PLAY1(GameDefinition.KIDNAPPING, GameTrophyTypeDefinition.PLAY_LIMIT_1, "kidnapping_1_finished", "kidnapping_1_finished_description", 1, new GameTrophy.NumberOfTimesFinished()), //10
    KIDNAPPING_PLAY2(GameDefinition.KIDNAPPING, GameTrophyTypeDefinition.PLAY_LIMIT_2, "kidnapping_5_finished", "kidnapping_5_finished_description", 5, new GameTrophy.NumberOfTimesFinished()), //11
    KIDNAPPING_PLAY3(GameDefinition.KIDNAPPING, GameTrophyTypeDefinition.PLAY_LIMIT_3, "kidnapping_10_finished", "kidnapping_10_finished_description", 10, new GameTrophy.NumberOfTimesFinished()), //12
    KIDNAPPING_DIFF1(GameDefinition.KIDNAPPING, GameTrophyTypeDefinition.DIFFICULTY_LIMIT_1, "kidnapping_1_limit",  "kidnapping_1_limit_description", GameDifficulty.MEDIUM.getDifficultyNumber(), new GameTrophy.ThreeStarsReceived()), //13
    KIDNAPPING_DIFF2(GameDefinition.KIDNAPPING, GameTrophyTypeDefinition.DIFFICULTY_LIMIT_2, "kidnapping_2_limit", "kidnapping_2_limit_description", GameDifficulty.HARD.getDifficultyNumber(), new GameTrophy.ThreeStarsReceived()),//14

    NIGHT_WATCH_PLAY1(GameDefinition.NIGHT_WATCH, GameTrophyTypeDefinition.PLAY_LIMIT_1, "night_watch_1_finished", "night_watch_1_finished_description", 1, new GameTrophy.NumberOfTimesFinished()), //15
    NIGHT_WATCH_PLAY2(GameDefinition.NIGHT_WATCH, GameTrophyTypeDefinition.PLAY_LIMIT_2, "night_watch_5_finished", "night_watch_5_finished_description",  5, new GameTrophy.NumberOfTimesFinished()), //16
    NIGHT_WATCH_PLAY3(GameDefinition.NIGHT_WATCH, GameTrophyTypeDefinition.PLAY_LIMIT_3, "night_watch_10_finished", "night_watch_10_finished_description",10, new GameTrophy.NumberOfTimesFinished()), //17
    NIGHT_WATCH_DIFF1(GameDefinition.NIGHT_WATCH, GameTrophyTypeDefinition.DIFFICULTY_LIMIT_1, "night_watch_1_limit", "night_watch_1_limit_description",  GameDifficulty.MEDIUM.getDifficultyNumber(), new GameTrophy.ThreeStarsReceived()), //18
    NIGHT_WATCH_DIFF2(GameDefinition.NIGHT_WATCH, GameTrophyTypeDefinition.DIFFICULTY_LIMIT_2, "night_watch_2_limit", "night_watch_2_limit_description",GameDifficulty.HARD.getDifficultyNumber(), new GameTrophy.ThreeStarsReceived()),//19

    SHOOTING_RANGE_PLAY1(GameDefinition.SHOOTING_RANGE, GameTrophyTypeDefinition.PLAY_LIMIT_1, "shooting_range_1_finished", "shooting_range_1_finished_description", 1, new GameTrophy.NumberOfTimesFinished()), //20
    SHOOTING_RANGE_PLAY2(GameDefinition.SHOOTING_RANGE, GameTrophyTypeDefinition.PLAY_LIMIT_2, "shooting_range_5_finished","shooting_range_5_finished_description", 5, new GameTrophy.NumberOfTimesFinished()), //21
    SHOOTING_RANGE_PLAY3(GameDefinition.SHOOTING_RANGE, GameTrophyTypeDefinition.PLAY_LIMIT_3, "shooting_range_10_finished", "shooting_range_10_finished_description", 10, new GameTrophy.NumberOfTimesFinished()), //22
    SHOOTING_RANGE_DIFF1(GameDefinition.SHOOTING_RANGE, GameTrophyTypeDefinition.DIFFICULTY_LIMIT_1, "shooting_range_1_limit",  "shooting_range_1_limit_description", GameDifficulty.MEDIUM.getDifficultyNumber(), new GameTrophy.ThreeStarsReceived()), //23
    SHOOTING_RANGE_DIFF2(GameDefinition.SHOOTING_RANGE, GameTrophyTypeDefinition.DIFFICULTY_LIMIT_2, "shooting_range_2_limit", "shooting_range_2_limit_description", GameDifficulty.HARD.getDifficultyNumber(), new GameTrophy.ThreeStarsReceived()),//24

    IN_THE_DARKNESS_PLAY1(GameDefinition.IN_THE_DARKNESS, GameTrophyTypeDefinition.PLAY_LIMIT_1, "in_the_darkness_1_finished", "in_the_darkness_1_finished_description", 1, new GameTrophy.NumberOfTimesFinished()), //25
    IN_THE_DARKNESS_PLAY2(GameDefinition.IN_THE_DARKNESS, GameTrophyTypeDefinition.PLAY_LIMIT_2, "in_the_darkness_5_finished","in_the_darkness_5_finished_description", 5, new GameTrophy.NumberOfTimesFinished()), //26
    IN_THE_DARKNESS_PLAY3(GameDefinition.IN_THE_DARKNESS, GameTrophyTypeDefinition.PLAY_LIMIT_3, "in_the_darkness_10_finished", "in_the_darkness_10_finished_description", 10, new GameTrophy.NumberOfTimesFinished()), //27
    IN_THE_DARKNESS_DIFF1(GameDefinition.IN_THE_DARKNESS, GameTrophyTypeDefinition.DIFFICULTY_LIMIT_1, "in_the_darkness_1_limit", "in_the_darkness_1_limit_description", GameDifficulty.MEDIUM.getDifficultyNumber(), new GameTrophy.ThreeStarsReceived()), //28
    IN_THE_DARKNESS_DIFF2(GameDefinition.IN_THE_DARKNESS, GameTrophyTypeDefinition.DIFFICULTY_LIMIT_2, "in_the_darkness_2_limit", "in_the_darkness_2_limit_description", GameDifficulty.HARD.getDifficultyNumber(), new GameTrophy.ThreeStarsReceived());//29


    private GameDefinition                  gameDefinition;
    private GameTrophyTypeDefinition        trophyTypeDefinition;
    private String                          trophyName;
    private String                          trophyDescription;
    private int                             limit;
    private GameTrophy.GameTrophyResolver   gameTrophyResolver;

    private GameTrophyDefinition(GameDefinition gameDefinition, GameTrophyTypeDefinition trophyTypeDefinition, String trophyName, String trophyDescription, int limit, GameTrophy.GameTrophyResolver gameTrophyResolver) {
        this.gameDefinition = gameDefinition;
        this.trophyTypeDefinition = trophyTypeDefinition;
        this.trophyName = trophyName;
        this.trophyDescription = trophyDescription;
        this.limit = limit;
        this.gameTrophyResolver = gameTrophyResolver;
    }

    public GameDefinition getGameDefinition() {
        return gameDefinition;
    }

    public GameTrophyTypeDefinition getTrophyTypeDefinition() {
        return trophyTypeDefinition;
    }

    @Override
    public String getTrophyNameKey() {
        return trophyName;
    }

    @Override
    public String getTrophyName() {
        return ApplicationTextManager.getInstance().getText(getTrophyNameKey());
    }

    @Override
    public String getTrophyDescriptionKey() {
        return trophyDescription;
    }

    @Override
    public String getTrophyDescription() {
        return String.format("%s %s", ApplicationTextManager.getInstance().getText(getTrophyDescriptionKey()), getGameDefinition().getTitle());
    }

    public int getLimit() {
        return limit;
    }

    public boolean hasTrophy(User user) {
        return gameTrophyResolver.hasGameTrophy(user, this);
    }

    public static GameTrophyDefinition getTrophyDefinitionByGameAndTrophyType(GameDefinition gameDefinition, GameTrophyTypeDefinition trophyTypeDefinition) {
        for (GameTrophyDefinition trophyDefinition : GameTrophyDefinition.values()) {
            if (trophyDefinition.getGameDefinition().equals(gameDefinition) && (trophyDefinition.getTrophyTypeDefinition() == trophyTypeDefinition)) {
                return trophyDefinition;
            }
        }
        return null;
    }

    public static GameTrophyDefinition getTrophyDefinitionByGameAndLimit(GameDefinition gameDefinition, TrophyGlobalTypeDefinition trophyGlobalTypeDefinition, int limit) {
        for (GameTrophyDefinition trophyDefinition : GameTrophyDefinition.values()) {
            if (trophyDefinition.getGameDefinition().equals(gameDefinition) && (trophyDefinition.getTrophyTypeDefinition().getTrophyGlobalTypeDefinition() == trophyGlobalTypeDefinition) && (trophyDefinition.getLimit() == limit)) {
                return trophyDefinition;
            }
        }
        return null;
    }
}
