package cz.nic.tablexia.loader;

import com.badlogic.gdx.utils.Disposable;
import com.badlogic.gdx.utils.async.AsyncExecutor;
import com.badlogic.gdx.utils.async.AsyncResult;
import com.badlogic.gdx.utils.async.AsyncTask;
import com.badlogic.gdx.utils.async.ThreadUtils;

public class TablexiaDataManager<T> implements Disposable {
	
	private AsyncExecutor 	asyncExecutor;
	private AsyncTask<T>	asyncTask;
	private AsyncResult<T> 	asyncResult;
	
	private boolean 		started;
	private boolean 		complete;
	
	private T 				result;
	
	protected void setAsyncTask(AsyncTask<T> asyncTask) {
		this.asyncTask 	= asyncTask;
		result 			= null;
		asyncExecutor 	= new AsyncExecutor(1);
		started			= false;
		complete 		= false;
	}
	
	public boolean update() {
		if (started) {
			if (asyncResult == null || asyncResult.isDone()) {
				if (!complete) {
					complete = true;	
					result = asyncResult.get();
					asyncResult = null;
					asyncExecutor.dispose();
					asyncExecutor = null;
					asyncTask = null;
				}
				return true;
			}
		} else {
			if (asyncTask == null) {
				return true;
			}
			asyncResult = asyncExecutor.submit(asyncTask);
			started = true;
		}
		return false;
	}

    public void finishLoading () {
        while (!update()) {
            ThreadUtils.yield();
        }
    }
	
	public T getResult() {
		if (result == null) {
			throw new IllegalStateException("Loading not complete or didnt start!");
		}
		return result;
	}
	
	@Override
	public void dispose() {
		result = null;
	}
}