package cz.nic.tablexia.loader.application;

import com.badlogic.gdx.audio.Sound;

import cz.nic.tablexia.loader.IApplicationLoader;
import cz.nic.tablexia.loader.TablexiaSoundManager;

/**
 * Application Sounds Manager, which is loading sounds from the external path...
 */
public class ApplicationInternalSoundManager extends TablexiaSoundManager implements IApplicationLoader {

	private static ApplicationInternalSoundManager instance;

	private ApplicationInternalSoundManager() {
		super(AssetsStorageType.INTERNAL);
	}
	
	public static ApplicationInternalSoundManager getInstance() {
		if (instance == null) {
			instance = new ApplicationInternalSoundManager();
		}
		return instance;
	}
	
	@Override
	public synchronized void dispose() {
		super.dispose();
		instance = null;
	}

	private static final String MP3_SUFFIX 			= ".mp3";
	private static final String BASE_PATH 			= "sfx/";
	public 	static final String BUTTON_CLICKED		= BASE_PATH + "button_clicked" + MP3_SUFFIX;
	public 	static final String ALERT				= BASE_PATH + "alert" + MP3_SUFFIX;

    public void load() {
		loadSound(BUTTON_CLICKED);
		loadSound(ALERT);
    }

	@Override
	public void loadSound(String soundName) {
		// don't check sound file -> all assets needn't be unzipped yet
		super.loadSound(soundName, false);
	}

	@Override
	public Sound getSound(String soundName) {
		return super.getSound(soundName);
	}
}