package cz.nic.tablexia.loader.application;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.utils.I18NBundle;
import com.badlogic.gdx.utils.async.AsyncTask;

import java.util.Locale;

import cz.nic.tablexia.loader.IApplicationLoader;
import cz.nic.tablexia.loader.TablexiaDataManager;

/**
 * Texture loader and manager for application context.
 * 
 * @author Matyáš Latner
 *
 */
public class ApplicationTextManager extends TablexiaDataManager<I18NBundle> implements IApplicationLoader {

    public static final class ApplicationTextsAssets {

        public static final String VICTORYSCREEN_BUTTON_REPLAY      = "victoryscreen_button_replay";
        public static final String VICTORYSCREEN_BUTTON_CHANGEGAME  = "victoryscreen_button_changegame";
		public static final String VICTORYSCREEN_NEW_TROPHY  		= "victoryscreen_new_trophy";

		public static final String USERMENU_NEWUSER  				= "usermenu_newuser";
		public static final String USERMENU_DELETE_USER  			= "usermenu_deleteuser";
		public static final String USERMENU_DIALOGUE_YES  			= "usermenu_dialog_yes";
		public static final String USERMENU_DIALOGUE_NO  			= "usermenu_dialog_no";

        public static final String USER_LOGOUT_MESSAGE              = "user_logout_message";

		public static final String PRELOADER_TITLE					= "preloader_title";
		public static final String CONFIRM_BUTTON  					= "confirm_button";

		public static final String SYSTEM_CONFIRM					= "system_confirm";
		public static final String SYSTEM_DECLINE					= "system_decline";
		public static final String SYSTEM_YES 						= "system_yes";
		public static final String SYSTEM_NO 						= "system_no";
		public static final String SYSTEM_EXIT 						= "system_exit";
		public static final String SYSTEM_RETRY						= "system_retry";
		public static final String SYSTEM_BACK						= "system_back";

		public static final String ZIPASSETLOADER_ERROR				= "zipassetloader_error";
		public static final String ZIPASSETLOADER_DOWNLOAD_REQUEST	= "zipassetloader_download_request";
		public static final String ZIPASSETLOADER_NOT_ENOUGH_SPACE  = "zipassetloader_not_enough_space";

		public static final String LANGUAGE_SYSTEM 					= "language_system";
		public static final String LANGUAGE_CZECH  					= "language_czech";
		public static final String LANGUAGE_SLOVAK 					= "language_slovak";
		public static final String LANGUAGE_GERMAN 					= "language_german";

		public static final String MAINMENU_GAMES 					= "mainmenu_games";
		public static final String MAINMENU_HALLOFFAME 				= "mainmenu_halloffame";
		public static final String MAINMENU_STATISTICS 				= "mainmenu_statistics";
		public static final String MAINMENU_ENCYCLOPEDIA			= "mainmenu_encyclopedia";
		public static final String MAINMENU_ABOUT					= "mainmenu_about";
		public static final String MAINMENU_PROFILE					= "mainmenu_profile";
		public static final String MAINMENU_LOGOUT					= "mainmenu_logout";
		public static final String MAINMENU_DEBUG					= "mainmenu_screendebug";

		public static final String GAME_ROBBERY_TITLE 					= "game_robbery_title";
		public static final String GAME_ROBBERY_DESCRIPTION				= "game_robbery_description";
		public static final String GAME_ROBBERY_PRELOADER_TEXT			= "game_robbery_preloader";
		public static final String GAME_PURSUIT_TITLE 					= "game_pursuit_title";
		public static final String GAME_PURSUIT_DESCRIPTION				= "game_pursuit_description";
		public static final String GAME_PURSUIT_PRELOADER_TEXT			= "game_pursuit_preloader";
		public static final String GAME_KIDNAPPING_TITLE 				= "game_kidnapping_title";
		public static final String GAME_KIDNAPPING_DESCRIPTION			= "game_kidnapping_description";
		public static final String GAME_KIDNAPPING_PRELOADER_TEXT1		= "game_kidnapping_preloader1";
		public static final String GAME_KIDNAPPING_PRELOADER_TEXT2		= "game_kidnapping_preloader2";
		public static final String GAME_KIDNAPPING_PRELOADER_TEXT3		= "game_kidnapping_preloader3";
		public static final String GAME_NIGHT_WATCH_TITLE 				= "game_night_watch_title";
		public static final String GAME_NIGHT_WATCH_DESCRIPTION			= "game_night_watch_description";
		public static final String GAME_NIGHT_WATCH_PRELOADER_TEXT		= "game_night_watch_preloader";
		public static final String GAME_SHOOTING_RANGE_TITLE 			= "game_shooting_range_title";
		public static final String GAME_SHOOTING_RANGE_DESCRIPTION		= "game_shooting_range_description";
		public static final String GAME_SHOOTING_RANGE_PRELOADER_TEXT1	= "game_shooting_range_preloader1";
		public static final String GAME_SHOOTING_RANGE_PRELOADER_TEXT2	= "game_shooting_range_preloader2";
		public static final String GAME_IN_THE_DARKNESS_TITLE 			= "game_in_the_darkness_title";
		public static final String GAME_IN_THE_DARKNESS_DESCRIPTION		= "game_in_the_darkness_description";
		public static final String GAME_IN_THE_DARKNESS_PRELOADER_TEXT1	= "game_in_the_darkness_preloader1";
		public static final String GAME_IN_THE_DARKNESS_PRELOADER_TEXT2	= "game_in_the_darkness_preloader2";

		public static final String GAME_DIFFICULTY_NAME					= "gamedifficulty_name";
		public static final String GAME_SCORE 							= "game_score";
		public static final String GAME_SCORE_ERROR 					= "game_score_error";
		public static final String GAME_SCORE_DURATION 					= "game_score_duration";
		public static final String GAME_AVERAGE_SCORE 					= "game_averagescore";
		public static final String GAME_AVERAGE_SCORE_ERROR 			= "game_averagescore_error";
		public static final String GAME_AVERAGE_SCORE_DURATION			= "game_averagescore_duration";
		public static final String GAME_QUIT_QUESTION					= "game_quit_question";

		public static final String SOUND_MUTED_QUESTION 				= "sound_muted_question";
		public static final String SOUND_MUTED_GAME_QUESTION 			= "sound_muted_game_question";

		public static final String SYNC_REQUEST_DIALOG_TEXT				= "sync_request_dialog_text";
		public static final String SYNC_REQUEST_BUTTON					= "sync_request_button";

		public static final String USER_RANK_1							= "user_rank_1";
		public static final String USER_RANK_2							= "user_rank_2";
		public static final String USER_RANK_3							= "user_rank_3";
		public static final String USER_RANK_4							= "user_rank_4";
		public static final String USER_RANK_5							= "user_rank_5";
		public static final String USER_RANK_6							= "user_rank_6";
		public static final String USER_RANK_7							= "user_rank_7";
		public static final String USER_RANK_8							= "user_rank_8";
		public static final String USER_RANK_9							= "user_rank_9";
		public static final String USER_RANK_10							= "user_rank_10";
		public static final String USER_RANK_11							= "user_rank_11";
    }
	
	private static final String APPLICATION_TEXT_RESOURCE_FILE = "text/application/application";

	private static class ApplicationTextLoader implements AsyncTask<I18NBundle> {

		private String textResourceFileName;
		private Locale locale;

		public ApplicationTextLoader(String textResourceFileName, Locale locale) {
			this.textResourceFileName = textResourceFileName;
			this.locale = locale;
		}

		@Override
		public I18NBundle call() throws Exception {
			FileHandle baseFileHandle = Gdx.files.internal(textResourceFileName);
			return I18NBundle.createBundle(baseFileHandle, locale);
		}
	}

	private static ApplicationTextManager instance;
	
	private ApplicationTextManager() {}
	
	public static ApplicationTextManager getInstance() {
		if (instance == null) {
			instance = new ApplicationTextManager();
		}
		return instance;
	}
	
	public void load(Locale locale) {
        setAsyncTask(new ApplicationTextLoader(APPLICATION_TEXT_RESOURCE_FILE, locale));
	}

    public String getText(String textKey) {
        return getResult().get(textKey);
    }

    public String getFormattedText(String textKey, Object... args) {
        return getResult().format(textKey, args);
    }
	
	@Override
	public void dispose() {
		super.dispose();
		instance = null;
	}
	
}