package cz.nic.tablexia.menu.game;

import com.badlogic.gdx.scenes.scene2d.Group;

import cz.nic.tablexia.Tablexia;
import cz.nic.tablexia.TablexiaApplication;
import cz.nic.tablexia.bus.ApplicationBus;
import cz.nic.tablexia.game.GameDefinition;
import cz.nic.tablexia.loader.application.ApplicationTextManager;
import cz.nic.tablexia.menu.AbstractMenu;
import cz.nic.tablexia.menu.IMenuItem;
import cz.nic.tablexia.screen.gamemenu.GameMenuScreen;
import cz.nic.tablexia.screen.gamemenu.event.ShowGameMenuPageEvent;

/**
 * Created by Matyáš Latner.
 */
public enum GameMenuDefinition implements ApplicationBus.ApplicationEvent, IMenuItem {

    ROBBERY         (GameDefinition.ROBBERY,            ApplicationTextManager.ApplicationTextsAssets.GAME_ROBBERY_TITLE, 			ApplicationTextManager.ApplicationTextsAssets.GAME_ROBBERY_DESCRIPTION),
    PURSUIT         (GameDefinition.PURSUIT,            ApplicationTextManager.ApplicationTextsAssets.GAME_PURSUIT_TITLE, 			ApplicationTextManager.ApplicationTextsAssets.GAME_PURSUIT_DESCRIPTION),
    KIDNAPPING      (GameDefinition.KIDNAPPING,         ApplicationTextManager.ApplicationTextsAssets.GAME_KIDNAPPING_TITLE, 		ApplicationTextManager.ApplicationTextsAssets.GAME_KIDNAPPING_DESCRIPTION),
    NIGHT_WATCH     (GameDefinition.NIGHT_WATCH,        ApplicationTextManager.ApplicationTextsAssets.GAME_NIGHT_WATCH_TITLE, 		ApplicationTextManager.ApplicationTextsAssets.GAME_NIGHT_WATCH_DESCRIPTION),
    SHOOTING_RANGE  (GameDefinition.SHOOTING_RANGE,     ApplicationTextManager.ApplicationTextsAssets.GAME_SHOOTING_RANGE_TITLE, 	ApplicationTextManager.ApplicationTextsAssets.GAME_SHOOTING_RANGE_DESCRIPTION),
    IN_THE_DARKNESS (GameDefinition.IN_THE_DARKNESS,    ApplicationTextManager.ApplicationTextsAssets.GAME_IN_THE_DARKNESS_TITLE,	ApplicationTextManager.ApplicationTextsAssets.GAME_IN_THE_DARKNESS_DESCRIPTION);

    private static final Class<GameMenuItemGroup> MENU_ITEM_GROUP_CLASS = GameMenuItemGroup.class;

    private GameDefinition  gameDefinition;
    private String          titleKey;
    private String          descriptionKey;

    GameMenuDefinition(GameDefinition gameDefinition, String titleKey, String descriptionKey){
        this.gameDefinition = gameDefinition;
        this.titleKey = titleKey;
        this.descriptionKey = descriptionKey;
    }

    public GameDefinition getGameDefinition() {
        return gameDefinition;
    }

    @Override
    public String getTitle() {
        return ApplicationTextManager.getInstance().getResult().get(titleKey);
    }

    @Override
    public String getDescription() {
        return ApplicationTextManager.getInstance().getResult().get(descriptionKey);
    }

    @Override
    public String[] getIcons() {
        return null;
    }

    @Override
    public Class<? extends Group> getItemGroupClass() {
        return MENU_ITEM_GROUP_CLASS;
    }

    @Override
    public AbstractMenu.MenuAction getMenuAction() {
        return AbstractMenu.MenuAction.CLOSE;
    }

    @Override
    public IMenuItem[] getSubmenu() {
        return null;
    }

    @Override
    public void performAction() {
        ApplicationBus.getInstance().publishAsync(this);
        if (TablexiaApplication.getActualScreenClass() == GameMenuScreen.class) {
            ApplicationBus.getInstance().publishAsync(new ShowGameMenuPageEvent(gameDefinition, true));
        } else {
            ApplicationBus.getInstance().publishAsync(new Tablexia.ChangeScreenEvent(GameMenuScreen.class,
                                                                                     TablexiaApplication.ScreenTransaction.FADE,
                                                                                     GameMenuScreen.createScreenState(GameMenuScreen.GameMenuPageDefinition.getGameMenuPageForGameDefinition(gameDefinition))));
        }
    }

    public static GameMenuDefinition getGameMenuDefinitionForGameDefinition(GameDefinition gameDefinition) {
        for (GameMenuDefinition gameMenuDefinition : GameMenuDefinition.values()) {
            if (gameDefinition == gameMenuDefinition.gameDefinition) {
                return gameMenuDefinition;
            }
        }
        return null;
    }
}
