package cz.nic.tablexia.menu.game;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.ui.Cell;
import com.badlogic.gdx.scenes.scene2d.ui.Container;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.scenes.scene2d.utils.NinePatchDrawable;
import com.badlogic.gdx.scenes.scene2d.utils.TextureRegionDrawable;
import com.badlogic.gdx.utils.Align;

import net.engio.mbassy.listener.Handler;

import cz.nic.tablexia.TablexiaApplication;
import cz.nic.tablexia.bus.ApplicationBus;
import cz.nic.tablexia.game.AbstractTablexiaGame;
import cz.nic.tablexia.game.GameDefinition;
import cz.nic.tablexia.loader.application.ApplicationAtlasManager;
import cz.nic.tablexia.loader.application.ApplicationFontManager;
import cz.nic.tablexia.menu.IMenuItem;
import cz.nic.tablexia.screen.gamemenu.GameMenuScreen;
import cz.nic.tablexia.screen.gamemenu.pages.GameMenuPage;
import cz.nic.tablexia.util.ui.TablexiaLabel;

/**
 * Created by Matyáš Latner.
 */
public class GameMenuItemGroup extends Container<Table> {

    private static final Color  HIGHLITED_COLOR             = new Color(0.780f, 0.698f, 0.600f, 1);

    private static final ApplicationFontManager.FontType_NEW    TITLE_FONT_TYPE             = ApplicationFontManager.FontType_NEW.BOLD_18;
    private static final Color                              TITLE_FONT_COLOR            = new Color(0.325f, 0.278f, 0.255f, 1);

    private static final ApplicationFontManager.FontType_NEW    DESCRIPTION_FONT_TYPE       = ApplicationFontManager.FontType_NEW.REGULAR_12;
    private static final Color                              DESCRIPTION_FONT_COLOR      = new Color(0.325f, 0.278f, 0.255f, 1);

    private static final float  LABLES_PADDING_LEFT_RATIO   = 1f / 40;
    private static final float  LABELS_PADDING_DEFAULT      = 6f;
    private static final Color  HIGHLITER_COLOR             = new Color(0.647f, 0.22f, 0.22f, 1);
    private static final float  ARROW_PADING_RIGHT          = 15f;
    private static final float  LAYOUT_PADDING              = 2f;
    private static final float  CONTAINER_BOTTOM_PADDING    = 4f;
    private static final float  LEFT_PADDING                = 5f;

    private final Table             layoutContainer;
    private final Image             arrowIcon;
    private final Image             restartIcon;
    private final Cell<Image>       iconCell;
    private final Image             highliter;
    private NinePatchDrawable       background;
    private TextureRegionDrawable   highlitedBackground;

    private Class<? extends AbstractTablexiaGame> abstractTablexiaGameClass;
    private GameMenuScreen.GameMenuPageDefinition gameMenuPageDefinition;

    public GameMenuItemGroup(IMenuItem menuItem, float width) {
        this.abstractTablexiaGameClass = ((GameMenuDefinition) menuItem).getGameDefinition().getScreenClass();
        this.gameMenuPageDefinition = GameMenuScreen.GameMenuPageDefinition.getGameMenuPageForGameDefinition(GameDefinition.getGameDefinitionForClass(abstractTablexiaGameClass));

        padLeft(LEFT_PADDING);
        layoutContainer = new Table();
        layoutContainer.pad(LAYOUT_PADDING);
        layoutContainer.setTouchable(Touchable.enabled);
        layoutContainer.addListener(new ClickListener() {

            @Override
            public boolean touchDown(InputEvent event, float x, float y, int pointer, int button) {
                setPressed();
                return super.touchDown(event, x, y, pointer, button);
            }

            @Override
            public void touchUp(InputEvent event, float x, float y, int pointer, int button) {
                setUnpressed();
                super.touchUp(event, x, y, pointer, button);
            }
        });

        background          = new NinePatchDrawable(ApplicationAtlasManager.getInstance().getPatch(ApplicationAtlasManager.MAINMENU_CHILDITEM_BACKGROUND));
        highlitedBackground = new TextureRegionDrawable(ApplicationAtlasManager.getInstance().getColorTextureRegion(HIGHLITED_COLOR));

        fill(true, false);
        setActor(layoutContainer);
        padBottom(CONTAINER_BOTTOM_PADDING);

        // highliter
        highliter = new Image(ApplicationAtlasManager.getInstance().getColorTexture(HIGHLITER_COLOR));
        layoutContainer.add(highliter);

        // texts
        TablexiaLabel titleLabel = new TablexiaLabel(menuItem.getTitle(), new TablexiaLabel.TablexiaLabelStyle(TITLE_FONT_TYPE, TITLE_FONT_COLOR));
        titleLabel.setWrap(true);

        TablexiaLabel descriptionLabel = new TablexiaLabel(menuItem.getDescription(), new TablexiaLabel.TablexiaLabelStyle(DESCRIPTION_FONT_TYPE, DESCRIPTION_FONT_COLOR));
        descriptionLabel.setWrap(true);

        Table labelsContainer = new Table();
        labelsContainer.add(titleLabel).align(Align.left);
        labelsContainer.row();
        labelsContainer.add(descriptionLabel).align(Align.left);
        labelsContainer.pad(LABELS_PADDING_DEFAULT, width * LABLES_PADDING_LEFT_RATIO, LABELS_PADDING_DEFAULT, LABELS_PADDING_DEFAULT);

        //TODO - Check this and make it better!
        float cellHeight =  titleLabel.getTablexiaLabelStyle().getFontType().getSize() +
                            descriptionLabel.getTablexiaLabelStyle().getFontType().getSize() +
                            2 * LABELS_PADDING_DEFAULT;

        layoutContainer.add(labelsContainer).height(cellHeight);

        // arrow
        arrowIcon = new Image(ApplicationAtlasManager.getInstance().getTextureRegion(ApplicationAtlasManager.MAINMENU_CHILDITEM_ARROW));
        restartIcon = new Image(ApplicationAtlasManager.getInstance().getTextureRegion(ApplicationAtlasManager.MAINMENU_CHILDITEM_RESTART));
        iconCell = layoutContainer.add();
        iconCell.expandX().align(Align.right).padRight(ARROW_PADING_RIGHT);
        changeIconToArrow();

        // highliter size
        highliter.getDrawable().setMinWidth(5);
        highliter.getDrawable().setMinHeight(layoutContainer.getMinHeight());
        unhighlite();

        setUnpressed();
        ApplicationBus.getInstance().subscribe(this);
    }

    private void highlite() {
        highliter.setVisible(true);
    }

    private void unhighlite() {
        highliter.setVisible(false);
    }

    private void changeIconToArrow() {
        restartIcon.remove();
        iconCell.setActor(arrowIcon);
    }

    private void changeIconToRestart() {
        arrowIcon.remove();
        iconCell.setActor(restartIcon);
    }

    private void setUnpressed() {
        layoutContainer.setBackground(background);
    }

    private void setPressed() {
        layoutContainer.setBackground(highlitedBackground);
    }

    @Handler
    public void onScreenChangedEvent(TablexiaApplication.ScreenChangedEvent screenChangedEvent) {
        if (screenChangedEvent.getScreenClass() == abstractTablexiaGameClass) {
            highlite();
            changeIconToRestart();
        } else {
            unhighlite();
            changeIconToArrow();
        }
    }

    @Handler
    public void onGameMenuPageScrollEvent(GameMenuPage.GameMenuPageScrolledEvent gameMenuPageScrolledEvent) {
        if (gameMenuPageScrolledEvent.getGameMenuPageDefinition() == gameMenuPageDefinition) {
            highlite();
        } else {
            unhighlite();
        }
    }
}
