package cz.nic.tablexia.menu.main;

import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.ui.Container;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.ui.VerticalGroup;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.utils.Align;

import net.engio.mbassy.listener.Handler;

import java.util.HashMap;
import java.util.Map;

import cz.nic.tablexia.bus.ApplicationBus;
import cz.nic.tablexia.bus.event.SubMenuControlEvent;
import cz.nic.tablexia.loader.application.ApplicationInternalSoundManager;
import cz.nic.tablexia.menu.AbstractMenu;
import cz.nic.tablexia.menu.IMenuItem;
import cz.nic.tablexia.menu.main.user.UserSelectBox;

/**
 * MainMenu container
 *
 * @author Matyáš Latner
 */
public class MainMenu extends AbstractMenu {
    private static final MenuControlType                    MENU_CONTROL_TYPE               = MenuControlType.MANUAL;
    private static final boolean                            MENU_SCREEN_PAUSE               = true;

    private static final float                              SCROLLBAR_PADDING_TOP           = 10f;
    private static final int                                SELECTBOX_USER_HEIGHT           = 70;

    private Map<IMenuItem, Group>   subMenus = new HashMap<IMenuItem, Group>();
    private VerticalGroup           menuItemsLayout;

    public MainMenu(Float width, Float height) {
        super(width, height);
        ApplicationBus.getInstance().subscribe(this);
    }

    @Override
    protected boolean isScreenPause() {
        return MENU_SCREEN_PAUSE;
    }

    @Override
    protected MenuControlType getMenuControlType() {
        return MENU_CONTROL_TYPE;
    }

    @Override
    public void initMenuItems(Container<Group> layoutContainer) {
        subMenus.clear();
        menuItemsLayout = new VerticalGroup();
        menuItemsLayout.fill();

        Table layoutTable = new Table();
        layoutTable.setFillParent(true);
        layoutTable.padTop(MAINMENU_PADDING);

        // User SelectBox
        layoutTable.add(new UserSelectBox(SELECTBOX_USER_HEIGHT)).fillX();
        layoutTable.row();

        // Menu ScrollPane
        ScrollPane scrollPane = new ScrollPane(menuItemsLayout);
        scrollPane.setCancelTouchFocus(false);
        scrollPane.setScrollingDisabled(true, false);
        layoutTable.add(scrollPane).fill().padTop(SCROLLBAR_PADDING_TOP).expandY();
        layoutTable.row();


        for (final IMenuItem menuItem : MainMenuDefinition.getItemsForMenu()) {
            Group menuItemContainer = getGroupForMenuItem(menuItem, getWidth());
            menuItemContainer.setWidth(getWidth());
            menuItemContainer.addListener(new ClickListener() {
                @Override
                public void clicked(InputEvent event, float x, float y) {
                    performMenuAction(menuItem);
                }
            });
            menuItemsLayout.addActor(menuItemContainer);
            IMenuItem[] subMenu = menuItem.getSubmenu();
            if (subMenu != null) {
                VerticalGroup subMenuLayout = new VerticalGroup();
                subMenus.put(menuItem, subMenuLayout);
                menuItemsLayout.addActor(subMenuLayout);
                subMenuLayout.fill();
                for (final IMenuItem submenuItem : subMenu) {
                    Group submenuItemContainer = getGroupForMenuItem(submenuItem, getWidth());
                    submenuItemContainer.addListener(new ClickListener() {
                        @Override
                        public void clicked(InputEvent event, float x, float y) {
                            performMenuAction(submenuItem);
                        }
                    });
                    subMenuLayout.addActor(submenuItemContainer);
                }
            }
        }

        float menuWidth = getBackground().getDrawable().getMinWidth() - MAINMENU_PADDING - APP_NAME_TEXT_PADDING_LEFT;

        layoutContainer.align(Align.bottom);
        layoutTable.getCells().get(0).width(menuWidth);
        layoutContainer.setActor(layoutTable);
    }

    private void performMenuAction(IMenuItem menuItem) {
        if (!isDisableControl()) {
            ApplicationInternalSoundManager.getInstance().getSound(ApplicationInternalSoundManager.BUTTON_CLICKED).play();
            if (menuItem.getMenuAction() != null) {
                disableMenuGravity(true);
            }
            doMenuAction(menuItem.getMenuAction(), true, true);
            menuItem.performAction();
        }
    }

    @Override
    public void dispose() {
        super.dispose();
        ApplicationBus.getInstance().unsubscribe(this);
    }


//////////////////////////// SUBMENU CONTROL EVENT HANDLING

    @Handler
    public void onSubMenuControlEvent(SubMenuControlEvent subMenuControlEvent) {
        Group subMenu = subMenus.get(subMenuControlEvent.getMenuItem());
        if (subMenu != null) {
            if (subMenuControlEvent.getSubMenuAction().isOpen(subMenu.hasParent()) && !subMenu.hasParent()) {
                menuItemsLayout.addActorAt(((MainMenuDefinition) subMenuControlEvent.getMenuItem()).ordinal() + 1, subMenu);
            } else {
                subMenu.remove();
            }
        }
    }
}
