package cz.nic.tablexia.menu.main;

import com.badlogic.gdx.scenes.scene2d.Group;

import java.util.ArrayList;
import java.util.List;

import cz.nic.tablexia.Tablexia;
import cz.nic.tablexia.TablexiaApplication;
import cz.nic.tablexia.TablexiaSettings;
import cz.nic.tablexia.bus.ApplicationBus;
import cz.nic.tablexia.bus.ApplicationBus.ApplicationEvent;
import cz.nic.tablexia.bus.event.SubMenuControlEvent;
import cz.nic.tablexia.loader.application.ApplicationAtlasManager;
import cz.nic.tablexia.loader.application.ApplicationTextManager;
import cz.nic.tablexia.menu.AbstractMenu;
import cz.nic.tablexia.menu.IMenuItem;
import cz.nic.tablexia.menu.game.GameMenuDefinition;
import cz.nic.tablexia.screen.about.AboutScreen;
import cz.nic.tablexia.screen.encyclopedia.EncyclopediaScreen;
import cz.nic.tablexia.screen.halloffame.HallOfFameScreen;
import cz.nic.tablexia.screen.profile.ProfileScreen;
import cz.nic.tablexia.screen.statistics.StatisticsScreen;
import cz.nic.tablexia.screen.viewport.ViewportDebugScreen;


public enum MainMenuDefinition implements IMenuItem {

	GAMES				(false, new String[]{ApplicationAtlasManager.MAINMENU_ICON_GAMES_UNPRESSED, ApplicationAtlasManager.MAINMENU_ICON_GAMES_PRESSED}, ApplicationTextManager.ApplicationTextsAssets.MAINMENU_GAMES, GameMenuDefinition.values(), null, null) {
		@Override
		public void performAction() {
			ApplicationBus.getInstance().post(new SubMenuControlEvent(MainMenu.class, this, SubMenuControlEvent.SubMenuAction.TOGGLE)).asynchronously();
		}
	},
	HALL_OF_FAME		(false, new String[]{ApplicationAtlasManager.MAINMENU_ICON_HALLOFFAME_UNPRESSED,    ApplicationAtlasManager.MAINMENU_ICON_HALLOFFAME_PRESSED},      ApplicationTextManager.ApplicationTextsAssets.MAINMENU_HALLOFFAME,      null, AbstractMenu.MenuAction.CLOSE,    new Tablexia.ChangeScreenEvent(HallOfFameScreen.class,      TablexiaApplication.ScreenTransaction.FADE)),
	STATISTICS			(false, new String[]{ApplicationAtlasManager.MAINMENU_ICON_STATISTICS_UNPRESSED,    ApplicationAtlasManager.MAINMENU_ICON_STATISTICS_PRESSED},      ApplicationTextManager.ApplicationTextsAssets.MAINMENU_STATISTICS,      null, AbstractMenu.MenuAction.CLOSE,    new Tablexia.ChangeScreenEvent(StatisticsScreen.class,      TablexiaApplication.ScreenTransaction.FADE)),
	ENCYCLOPEDIA		(false, new String[]{ApplicationAtlasManager.MAINMENU_ICON_ENCYCLOPEDIA_UNPRESSED,  ApplicationAtlasManager.MAINMENU_ICON_ENCYCLOPEDIA_PRESSED},    ApplicationTextManager.ApplicationTextsAssets.MAINMENU_ENCYCLOPEDIA,    null, AbstractMenu.MenuAction.CLOSE,    new Tablexia.ChangeScreenEvent(EncyclopediaScreen.class,    TablexiaApplication.ScreenTransaction.FADE)),
	ABOUT_APPLICATION	(false, new String[]{ApplicationAtlasManager.MAINMENU_ICON_ABOUT_UNPRESSED,         ApplicationAtlasManager.MAINMENU_ICON_ABOUT_PRESSED},           ApplicationTextManager.ApplicationTextsAssets.MAINMENU_ABOUT,           null, AbstractMenu.MenuAction.CLOSE,    new Tablexia.ChangeScreenEvent(AboutScreen.class,           TablexiaApplication.ScreenTransaction.FADE)),
	PROFILE				(false, new String[]{ApplicationAtlasManager.MAINMENU_ICON_PROFILE_UNPRESSED,       ApplicationAtlasManager.MAINMENU_ICON_PROFILE_PRESSED},         ApplicationTextManager.ApplicationTextsAssets.MAINMENU_PROFILE,         null, AbstractMenu.MenuAction.CLOSE,    new Tablexia.ChangeScreenEvent(ProfileScreen.class,         TablexiaApplication.ScreenTransaction.FADE)),
	LOGOUT				(false, new String[]{ApplicationAtlasManager.MAINMENU_ICON_LOGOUT_UNPRESSED,        ApplicationAtlasManager.MAINMENU_ICON_LOGOUT_PRESSED},          ApplicationTextManager.ApplicationTextsAssets.MAINMENU_LOGOUT,          null, AbstractMenu.MenuAction.CLOSE,    null) {
		@Override
		public void performAction() {
            TablexiaSettings.getInstance().changeUser(null, new TablexiaSettings.LogoutAcceptListener() {
                @Override
                public void onLogoutAccepted() {
                    ApplicationBus.getInstance().post(new Tablexia.ChangeScreenEvent(TablexiaSettings.LOADER_SCREEN, TablexiaApplication.ScreenTransaction.FADE)).asynchronously();
                }
            });

        }
	},
    SCREEN_DEBUG    	(true, new String[]{ApplicationAtlasManager.MAINMENU_ICON_ABOUT_UNPRESSED, ApplicationAtlasManager.MAINMENU_ICON_ABOUT_PRESSED}, ApplicationTextManager.ApplicationTextsAssets.MAINMENU_DEBUG, null, AbstractMenu.MenuAction.CLOSE, new Tablexia.ChangeScreenEvent(ViewportDebugScreen.class, TablexiaApplication.ScreenTransaction.FADE));

	private static final Class<MainMenuItemGroup> MENU_ITEM_GROUP_CLASS = MainMenuItemGroup.class;

	private boolean debug;
	private String[] icons;
	private String menuTextKey;
	private IMenuItem[] submenu;
	private AbstractMenu.MenuAction menuAction;
	private ApplicationEvent action;

	MainMenuDefinition(boolean debug, String[] icons, String nameResource, IMenuItem[] submenu, AbstractMenu.MenuAction menuAction, ApplicationEvent action) {
        this.debug = debug;
		this.icons = icons;
		this.menuTextKey = nameResource;
		this.submenu = submenu;
		this.menuAction = menuAction;
		this.action = action;
	}

    public ApplicationEvent getAction() {
		return action;
	}

	@Override
	public String getTitle() {
		return ApplicationTextManager.getInstance().getText(menuTextKey);
	}

	@Override
	public String getDescription() {
		return null;
	}

	@Override
	public Class<? extends Group> getItemGroupClass() {
		return MENU_ITEM_GROUP_CLASS;
	}

	@Override
	public void performAction() {
		ApplicationBus.getInstance().publishAsync(this.getAction());
	}

	public String[] getIcons() {
		return icons;
	}

	public IMenuItem[] getSubmenu() {
		return submenu;
	}

	@Override
	public AbstractMenu.MenuAction getMenuAction() {
		return menuAction;
	}

    public static List<MainMenuDefinition> getItemsForMenu() {
        List<MainMenuDefinition> mainMenu = new ArrayList<MainMenuDefinition>();
        for (MainMenuDefinition mainMenuDefinition : MainMenuDefinition.values()) {
            if (TablexiaSettings.getInstance().getBuildType().isDebugMenu() || !mainMenuDefinition.debug) {
                mainMenu.add(mainMenuDefinition);
            }
        }
        return mainMenu;
    }
}
