package cz.nic.tablexia.menu.main.locale;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.GlyphLayout;
import com.badlogic.gdx.graphics.g2d.NinePatch;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;
import com.badlogic.gdx.utils.Align;

import java.util.ArrayList;
import java.util.List;

import cz.nic.tablexia.TablexiaSettings;
import cz.nic.tablexia.bus.ApplicationBus;
import cz.nic.tablexia.loader.application.ApplicationAtlasManager;
import cz.nic.tablexia.loader.application.ApplicationFontManager;
import cz.nic.tablexia.loader.application.ApplicationTextManager;
import cz.nic.tablexia.util.ui.AbstractTablexiaSelectBox;
import cz.nic.tablexia.util.ui.TablexiaLabel;

/**
 * Created by Matyáš Latner.
 */
public class LocaleSelectBox extends AbstractTablexiaSelectBox<LocaleSelectBox.LocaleItem> {

    private static final ApplicationFontManager.FontType_NEW    LOCALE_TEXT_FONT    = ApplicationFontManager.FontType_NEW.REGULAR_14;
    private static final Color                                  LOCALE_TEXT_COLOR   = new Color(0.098f, 0.086f, 0.075f, 1f);
    private static final int                                    LOCALE_TEXT_ALIGN   = Align.center;

    public class LocaleItem extends Actor {
        private final NinePatch                         background;
        private final String                            text;
        private final GlyphLayout                       glyphLayout;
        private final TablexiaSettings.LocaleDefinition localeDefinition;
        private final TablexiaLabel                     textLabel;

        public LocaleItem(TablexiaSettings.LocaleDefinition localeDefinition) {
            this.localeDefinition = localeDefinition;

            background  = ApplicationAtlasManager.getInstance().getPatch(ApplicationAtlasManager.USERMENU_MENUITEM_BACKGROUND);
            text        = ApplicationTextManager.getInstance().getText(localeDefinition.getDescriptionKey());
            glyphLayout = new GlyphLayout();

            textLabel   = new TablexiaLabel(text, new TablexiaLabel.TablexiaLabelStyle(LOCALE_TEXT_FONT, LOCALE_TEXT_COLOR));
            textLabel.setAlignment(LOCALE_TEXT_ALIGN);
        }

        public TablexiaSettings.LocaleDefinition getLocaleDefinition() {
            return localeDefinition;
        }

        @Override
        public void draw(Batch batch, float parentAlpha) {
            LOCALE_TEXT_COLOR.a = parentAlpha;

            background.draw(batch, getX(), getY(), getWidth(), getHeight());

            textLabel.setBounds(getX(), getY(), getWidth(), getHeight());
            textLabel.draw(batch, parentAlpha);
        }
    }

    private final ChangeListener changeListener = new ChangeListener() {
        @Override
        public void changed(ChangeEvent event, Actor actor) {
            TablexiaSettings.getInstance().setLocale(((LocaleSelectBox)event.getTarget()).getSelected().getLocaleDefinition());
        }
    };

    public LocaleSelectBox(float itemHeight) {
        super(itemHeight);
        ApplicationBus.getInstance().subscribe(this);
        setCheckValidity(false);
        prepareLocales();
    }

    private void prepareLocales() {
        removeCaptureListener(changeListener);
        LocaleItem selectedLocaleItem = null;

        List<LocaleItem> localeItems = new ArrayList<LocaleItem>();
        for (TablexiaSettings.LocaleDefinition localeDefinition: TablexiaSettings.LocaleDefinition.getEnabledLocaleDefinitions()) {
            LocaleItem localeItem = new LocaleItem(localeDefinition);
            if (localeDefinition.equals(TablexiaSettings.getInstance().getLocaleDefinition())) {
                selectedLocaleItem = localeItem;
            }
            else {
                localeItems.add(localeItem);
            }
        }
        setItems(localeItems.toArray(new LocaleItem[]{}));
        if (selectedLocaleItem != null) {
            setSelected(selectedLocaleItem);
        }
        addCaptureListener(changeListener);
    }

    @Override
    public boolean onSelectBoxItemSelected(LocaleItem item) {
        getSelectBoxList().changeToSelected();
        prepareLocales();
        return true;
    }

    @Override
    protected void drawSelectedItem(Batch batch, float parentAlpha, LocaleItem selected, float width, float height) {
        selected.setBounds(getX(), getY(), width, height);
        selected.draw(batch, parentAlpha);
    }

    @Override
    protected void drawListItem(Batch batch, float parentAlpha, LocaleItem item, float x, float y, float width, float height) {
        item.setBounds(x, y, width, height);
        item.draw(batch, parentAlpha);
    }
}
