package cz.nic.tablexia.menu.main.sound;

import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.utils.TextureRegionDrawable;

import net.engio.mbassy.listener.Handler;

import cz.nic.tablexia.TablexiaApplication;
import cz.nic.tablexia.TablexiaSettings;
import cz.nic.tablexia.bus.ApplicationBus;
import cz.nic.tablexia.loader.application.ApplicationAtlasManager;
import cz.nic.tablexia.util.ui.ClickListenerWithSound;

/**
 * Created by drahomir on 7/15/16.
 */
public class SoundMuteButton extends Group {
    private static final float DEFAULT_OPACITY  = 1.0f;
    private static final float DISABLED_OPACITY = 0.25f;

    private static final float ICON_SIZE_SCALE  = 0.6f;

    private final ClickListenerWithSound clickListener = new ClickListenerWithSound() {
        @Override
        public void onClick(InputEvent event, float x, float y) {
            TablexiaSettings.getInstance().toggleSoundMute();
        }
    };

    private TextureRegionDrawable buttonIconPressed;
    private TextureRegionDrawable buttonIconUnpressed;

    private Image   buttonBackground;
    private Image   buttonIcon;
    private Image   disabledOverlay;

    public SoundMuteButton(float height) {
        setSize(height, height);
        initialize();
    }

    private void initialize() {
        ApplicationBus.getInstance().subscribe(this);

        buttonIconPressed   = new TextureRegionDrawable(ApplicationAtlasManager.getInstance().getTextureRegion(ApplicationAtlasManager.MAINMENU_ICON_SOUNDS_PRESSED));
        buttonIconUnpressed = new TextureRegionDrawable(ApplicationAtlasManager.getInstance().getTextureRegion(ApplicationAtlasManager.MAINMENU_ICON_SOUNDS_UNPRESSED));

        buttonBackground = new Image(ApplicationAtlasManager.getInstance().getPatch(ApplicationAtlasManager.USERMENU_MENUITEM_BACKGROUND));
        buttonBackground.setSize(getWidth(), getHeight());
        addActor(buttonBackground);

        buttonIcon = new Image();
        buttonIcon.setSize(getWidth() * ICON_SIZE_SCALE, getHeight() * ICON_SIZE_SCALE);
        buttonIcon.setPosition((getWidth() - buttonIcon.getWidth()) / 2f, (getHeight() - buttonIcon.getHeight()) / 2);
        addActor(buttonIcon);

        disabledOverlay = new Image(ApplicationAtlasManager.getInstance().getTextureRegion(ApplicationAtlasManager.MAINMENU_ICON_SOUNDS_DISABLED));
        disabledOverlay.setPosition(0, 0);
        disabledOverlay.setSize(getWidth(), getHeight());
        disabledOverlay.setVisible(false);
        addActor(disabledOverlay);

        addListener(clickListener);

        //Change this in case you wanna save the sound mute state across app launches
        setUnmuted();
    }

    private void setMuted() {
        buttonIcon.setDrawable(buttonIconPressed);
    }

    private void setUnmuted() {
        buttonIcon.setDrawable(buttonIconUnpressed);
    }

    private void setDisabled() {
        removeListener(clickListener);
        disabledOverlay.setVisible(true);
        setColor(getColor().r, getColor().g, getColor().b, DISABLED_OPACITY);
    }

    private void setEnabled() {
        addListener(clickListener);
        disabledOverlay.setVisible(false);
        setColor(getColor().r, getColor().g, getColor().b, DEFAULT_OPACITY);
    }

    @Handler
    public void handleSoundMuteEvent(TablexiaSettings.SoundMuteEvent soundMuteEvent) {
        if(soundMuteEvent.isSoundMuted()) setMuted();
        else setUnmuted();
    }

    @Handler
    public void handleScreenChangedEvent(TablexiaApplication.ScreenChangedEvent screenChangedEvent) {
        if(TablexiaApplication.getActualScreen().isSoundMandatory())
            setDisabled();
        else
            setEnabled();
    }
}
