package cz.nic.tablexia.menu.user;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.ui.Container;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;
import com.badlogic.gdx.scenes.scene2d.ui.VerticalGroup;
import com.badlogic.gdx.scenes.scene2d.utils.ActorGestureListener;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;

import net.engio.mbassy.listener.Handler;

import java.util.ArrayList;
import java.util.List;

import cz.nic.tablexia.Tablexia;
import cz.nic.tablexia.TablexiaApplication;
import cz.nic.tablexia.bus.ApplicationBus;
import cz.nic.tablexia.bus.event.DeleteUserSynchronizationEvent;
import cz.nic.tablexia.bus.event.MenuControlEvent;
import cz.nic.tablexia.loader.application.ApplicationInternalSoundManager;
import cz.nic.tablexia.loader.application.ApplicationTextManager;
import cz.nic.tablexia.menu.AbstractMenu;
import cz.nic.tablexia.model.UserDAO;
import cz.nic.tablexia.screen.createuser.PanoramaScreen;
import cz.nic.tablexia.shared.model.User;
import cz.nic.tablexia.sync.RestSynchronizationService;
import cz.nic.tablexia.sync.work.DownloadUser;
import cz.nic.tablexia.sync.work.PushDataToServer;
import cz.nic.tablexia.sync.work.SyncWork;
import cz.nic.tablexia.util.ui.ClickListenerWithSound;
import cz.nic.tablexia.util.ui.dialog.TablexiaComponentDialogFactory;
import cz.nic.tablexia.util.ui.dialog.components.AdaptiveSizeDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.BackButtonHideComponent;
import cz.nic.tablexia.util.ui.dialog.components.CenterPositionDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.DimmerDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.FixedSpaceContentDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.PositiveNegativeButtonContentDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.ResizableSpaceContentDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.TextContentDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.TextFiledDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.ViewportMaximumSizeComponent;

/**
 * UserMenu container
 *
 * @author Matyáš Latner
 */
public class UserMenu extends AbstractMenu {

    private static final MenuControlType    MENU_CONTROL_TYPE   				= MenuControlType.AUTOMATIC;
    private static final boolean            MENU_SCREEN_PAUSE  				 	= false;
    private static final int                USER_ITEM_HEIGHT    				= 80;
	private static final float				DOWNLOAD_DIALOG_INPUT_WIDTH_PERCENT	= 0.8f;

    private static final int    SYNC_REQUEST_DIALOG_WIDTH            = 450;
    private static final int    SYNC_REQUEST_DIALOG_HEIGHT           = 300;
    private static final String SYNC_REQUEST_DIALOG_PLACEHOLDER_TEXT = "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx";
    private static final int    SYNC_REQUEST_DIALOG_MAXIMUM_LENGTH   = 36;
    private static final float  SYNC_DIALOG_TITLE_PAD                = 0.15f;
    private static final float  SYNC_DIALOG_TEXT_FIELD_PAD           = 0.05f;
    
    public static final String  CODE_REGEX                           = "(([0-9]|[a-f]){8})-(([0-9]|[a-f]){4})-(([0-9]|[a-f]){4})-(([0-9]|[a-f]){4})-(([0-9]|[a-f]){12})$";

    private List<User> users;

    public UserMenu(Float width, Float height) {
        super(width, height);
        users = new ArrayList<User>();
        ApplicationBus.getInstance().subscribe(this);
    }

    @Override
    protected boolean isScreenPause() {
        return MENU_SCREEN_PAUSE;
    }

    @Override
    protected MenuControlType getMenuControlType() {
        return MENU_CONTROL_TYPE;
    }

    @Override
    public void initMenuItems(Container<Group> layoutContainer) {
        final VerticalGroup menuItemsLayout = new VerticalGroup();

        final ScrollPane scrollPane = new ScrollPane(menuItemsLayout);
        final Container<ScrollPane> container = new Container<ScrollPane>(scrollPane);
        container.setFillParent(true);
        final float userMenuItemWidth = getBackground().getWidth();
        this.users = UserDAO.selectActiveUsers();
        for (final User user : users) {
            final UserMenuItem menuItem = new UserMenuItem(user);
            final Group userItemGroup = getGroupForMenuItem(menuItem, getWidth());
            userItemGroup.setSize(userMenuItemWidth, USER_ITEM_HEIGHT);
            userItemGroup.addListener(new ActorGestureListener() {
                @Override
                public boolean longPress(Actor actor, float x, float y) {
                    int index = menuItemsLayout.getChildren().indexOf(actor, false);
                    final User selected = UserDAO.selectActiveUsers().get(index);
                    TablexiaComponentDialogFactory.getInstance().createWarningYesNoDialog(
                            String.format("%s %s?", ApplicationTextManager.getInstance().getText(ApplicationTextManager.ApplicationTextsAssets.USERMENU_DELETE_USER), selected.getName()),
                            new ClickListenerWithSound() {
                                @Override
                                public void onClick(InputEvent event, float x, float y) {
                                    selected.setDeleted(true);
                                    UserDAO.updateUser(selected);
                                    menuItemsLayout.removeActor(userItemGroup);
                                    ApplicationBus.getInstance().publishAsync(new DeleteUserSynchronizationEvent(selected));
                                }
                            },
							true
                    ).show(TablexiaComponentDialogFactory.WARNING_DIALOG_WIDTH, TablexiaComponentDialogFactory.WARNING_DIALOG_HEIGHT);
                    return true;
                }

                @Override
                public void tap(InputEvent event, float x, float y, int count, int button) {
                    ApplicationInternalSoundManager.getInstance().getSound(ApplicationInternalSoundManager.BUTTON_CLICKED).play();
                    doMenuAction(menuItem.getMenuAction(), true, true);
                    menuItem.performAction();
                }
            });
            menuItemsLayout.addActor(userItemGroup);
        }
        final UserMenuNewSelectBoxItemGroup userMenuNewSelectBoxItemGroup = new UserMenuNewSelectBoxItemGroup() {
            @Override
            public void performAction() {
                ApplicationBus.getInstance().post(new MenuControlEvent(UserMenu.class, MenuAction.HIDE, true)).asynchronously();
                ApplicationBus.getInstance().post(new Tablexia.ChangeScreenEvent(PanoramaScreen.class, TablexiaApplication.ScreenTransaction.MOVE_UP)).asynchronously();
            }

            @Override
            public User getUser() {
                return null;
            }
        };
        userMenuNewSelectBoxItemGroup.setSize(userMenuItemWidth, USER_ITEM_HEIGHT);
        userMenuNewSelectBoxItemGroup.addListener(new ClickListener() {
            @Override
            public void clicked(InputEvent event, float x, float y) {
                ApplicationInternalSoundManager.getInstance().getSound(ApplicationInternalSoundManager.BUTTON_CLICKED).play();
                userMenuNewSelectBoxItemGroup.performAction();
            }
        });
        menuItemsLayout.addActor(userMenuNewSelectBoxItemGroup);

        final UserMenuDownloadSelectBoxItemGroup userDownload = new UserMenuDownloadSelectBoxItemGroup() {
            @Override
            public void performAction() {
                final TextFiledDialogComponent textFiledDialogComponent = new TextFiledDialogComponent(SYNC_REQUEST_DIALOG_MAXIMUM_LENGTH, SYNC_REQUEST_DIALOG_PLACEHOLDER_TEXT);
				textFiledDialogComponent.setTextFieldWidthPercent(DOWNLOAD_DIALOG_INPUT_WIDTH_PERCENT);
                textFiledDialogComponent.setRepositionOnFocusOnMobileDevices(true);

                TablexiaComponentDialogFactory.getInstance().createDialog(
                        new CenterPositionDialogComponent(),
                        new DimmerDialogComponent(),
                        new ViewportMaximumSizeComponent(),
                        new AdaptiveSizeDialogComponent(),
                        new FixedSpaceContentDialogComponent(SYNC_DIALOG_TITLE_PAD),
                        new TextContentDialogComponent(ApplicationTextManager.getInstance().getText(ApplicationTextManager.ApplicationTextsAssets.SYNC_REQUEST_DIALOG_TEXT)),
                        new FixedSpaceContentDialogComponent(SYNC_DIALOG_TEXT_FIELD_PAD),
                        textFiledDialogComponent,
                        new ResizableSpaceContentDialogComponent(),
                        new PositiveNegativeButtonContentDialogComponent(new ClickListener() {
                                @Override
                                public void clicked(InputEvent event, float x, float y) {
                                    super.clicked(event, x, y);
                                    if (textFiledDialogComponent.getText().length()>0 && textFiledDialogComponent.getText().toLowerCase().matches(CODE_REGEX)) {
                                        DownloadUser syncWork = new DownloadUser(textFiledDialogComponent.getText());
                                        syncWork.registerListener(new SyncWork.RestSyncListener() {
                                            @Override
                                            public void onSuccess(User user) {
                                                ApplicationBus.getInstance().post(new RefreshUserMenu()).asynchronously();
                                            }
                                        });
                                        RestSynchronizationService.doSyncWork(syncWork);
                                    }
                                    //Prevents keyboard getting stuck
                                    Gdx.input.setOnscreenKeyboardVisible(false);
                                }
                            },
                            new ClickListener() {
                                @Override
                                public void clicked(InputEvent event, float x, float y) {
                                    super.clicked(event, x, y);
                                    //Prevents keyboard getting stuck
                                    Gdx.input.setOnscreenKeyboardVisible(false);
                                }
                            },
                            PositiveNegativeButtonContentDialogComponent.PositiveNegativeButtonType.CONFIRM_DECLINE),
                        new FixedSpaceContentDialogComponent(),
						new BackButtonHideComponent()
                ).show(SYNC_REQUEST_DIALOG_WIDTH, SYNC_REQUEST_DIALOG_HEIGHT);
            }

            @Override
            public User getUser() {
                return null;
            }
        };

        userDownload.setSize(userMenuItemWidth, USER_ITEM_HEIGHT);
        userDownload.addListener(new ClickListener() {
            @Override
            public void clicked(InputEvent event, float x, float y) {
                ApplicationInternalSoundManager.getInstance().getSound(ApplicationInternalSoundManager.BUTTON_CLICKED).play();
                userDownload.performAction();
            }
        });
        menuItemsLayout.addActor(userDownload);

        layoutContainer.setActor(container);
    }

    @Override
    public void dispose() {
        ApplicationBus.getInstance().unsubscribe(this);
    }


//////////////////////////// CREATE USER EVENT HANDLING

    @Handler
    public void handleCreatedUserEvent(UserDAO.CreatedUserEvent createdUserEvent) {
        initMenuItems();
    }

    @Handler
    public void handleRefreshUserMenuEvent(RefreshUserMenu refreshEvent) {
        Gdx.app.postRunnable(new Runnable() {
            @Override
            public void run() {
                initMenuItems();
            }
        });
    }

    @Handler
    public void updateUserUuid(PushDataToServer.UserUuidUpdatedEvent event) {
        for (final User user : users) {
            if (user != null && user.getId() == event.getUserId()) {
                user.setUuid(event.getUserUuid());
            }
        }
    }

    public static class RefreshUserMenu implements ApplicationBus.ApplicationEvent {

    }
}
