package cz.nic.tablexia.screen.about;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.ui.Container;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.scenes.scene2d.utils.TextureRegionDrawable;
import com.badlogic.gdx.utils.Align;
import com.badlogic.gdx.utils.viewport.ExtendViewport;
import com.badlogic.gdx.utils.viewport.Viewport;

import java.util.List;
import java.util.Map;

import cz.nic.tablexia.TablexiaSettings;
import cz.nic.tablexia.loader.application.ApplicationFontManager;
import cz.nic.tablexia.screen.AbstractTablexiaScreen;
import cz.nic.tablexia.screen.about.assets.AboutAssets;
import cz.nic.tablexia.util.ui.ClickListenerWithSound;
import cz.nic.tablexia.util.ui.TablexiaLabel;
import cz.nic.tablexia.util.ui.button.TablexiaButton;

public class AboutScreen extends AbstractTablexiaScreen<Void> {
    private static final Color   DEFAULT_FONT_COLOR       = TablexiaSettings.getDefaultFontColor();
    private static final Color   DEFAULT_BACKGROUND_COLOR = TablexiaSettings.getDefaultBackgroundColor();
    private static final Color   DEFAULT_LINK_COLOR       = new Color(0.27f, 0.53f, 0.8f,1);

    private static final float CONTENT_RELATIVE_WIDTH   = 0.9f;
    private static final float CONTENT_RELATIVE_HEIGHT  = 0.85f;
    private static final int   LOGO_TABLEXIA_MAX_HEIGHT = 60;
    private static final int   LOGO_CZ_NIC_MAX_HEIGHT   = 20;
    private static final int   SOCIAL_MAX_HEIGHT        = 40;
    private static final int   CONTENT_PAD              = 10;
    private static final int   SMALL_PAD                = 5;
    private static final int   BIG_PAD                  = 20;

    private static final int DEFAULT_FONT_PAD  = 5;
    private static final int BIG_FONT_PAD      = 10;

    private static final String SCREEN_STATE_SCROLL_POSITION_Y = "scrollPositionY";

    private final TablexiaLabel.TablexiaLabelStyle LABEL_STYLE_18      = new TablexiaLabel.TablexiaLabelStyle(ApplicationFontManager.FontType_NEW.REGULAR_18, DEFAULT_FONT_COLOR);
    private final TablexiaLabel.TablexiaLabelStyle BOLD_LABEL_STYLE_18 = new TablexiaLabel.TablexiaLabelStyle(ApplicationFontManager.FontType_NEW.BOLD_18, DEFAULT_FONT_COLOR);
    private final TablexiaLabel.TablexiaLabelStyle BOLD_LABEL_STYLE_20 = new TablexiaLabel.TablexiaLabelStyle(ApplicationFontManager.FontType_NEW.BOLD_20, DEFAULT_FONT_COLOR);
    private final TablexiaLabel.TablexiaLabelStyle BOLD_LABEL_STYLE_26 = new TablexiaLabel.TablexiaLabelStyle(ApplicationFontManager.FontType_NEW.BOLD_26, DEFAULT_FONT_COLOR);

    private Table                       backgroundTable;
    private Container<TablexiaButton>   contentStack;
    private Table                       content;

    private ScrollPane contentScrollPane;
    private float      contentWidth;
    private float      contentHeight;

    private LabelClickListener labelClickListener;

    @Override
    public boolean hasSoftBackButton() {
        return true;
    }

    @Override
    protected Viewport createViewport() {
        return new ExtendViewport(TablexiaSettings.getWorldSize(), TablexiaSettings.getMinWorldHeight());
    }

    @Override
    protected void screenPaused(Map<String, String> screenState) {
        screenState.put(SCREEN_STATE_SCROLL_POSITION_Y, String.valueOf(contentScrollPane.getScrollPercentY()));
    }

    @Override
    protected void screenLoaded(Map<String, String> screenState) {
        TextureRegionDrawable backgroundTexture = new TextureRegionDrawable(getColorTextureRegion(DEFAULT_BACKGROUND_COLOR));
        backgroundTable = new Table();
        contentStack = new Container<TablexiaButton>();
        backgroundTable.background(backgroundTexture);
        backgroundTable.setFillParent(true);
        prepareScreenLayout();

        if (screenState.containsKey(SCREEN_STATE_SCROLL_POSITION_Y)) {
            contentScrollPane.pack();
            contentScrollPane.setScrollPercentY(Float.valueOf(screenState.get(SCREEN_STATE_SCROLL_POSITION_Y)));
            contentScrollPane.updateVisualScroll();
        }

    }


    /**
     * Prepare screen content layout
     */
    private void prepareScreenLayout() {
        contentWidth = CONTENT_RELATIVE_WIDTH * getViewportWidth();
        contentHeight = CONTENT_RELATIVE_HEIGHT * getViewportHeight();

        content = new Table();
        content.setSize(contentWidth, contentHeight);
        labelClickListener = new LabelClickListener();

        prepareContent();

        contentScrollPane = new ScrollPane(content);
        contentScrollPane.setScrollingDisabled(true, false);
        contentScrollPane.setFillParent(true);

        contentStack.setFillParent(true);
        contentStack.align(Align.bottomLeft);

        getStage().addActor(backgroundTable);
        getStage().addActor(contentScrollPane);
        getStage().addActor(contentStack);

        contentStack.setDebug(TablexiaSettings.getInstance().isShowBoundingBoxes(), true);
    }


    private void prepareContent() {
        Links nicWeb;
        Links tablexiaWeb;
        switch (TablexiaSettings.getInstance().getLocaleDefinition()) {
            default:
                tablexiaWeb = Links.TABLEXIA;
                nicWeb = Links.NIC_CS;
                break;
            case de_DE:
                tablexiaWeb = Links.TABLEXIA_DE;
                nicWeb = Links.NIC_EN;
                break;
            case sk_SK:
                tablexiaWeb = Links.TABLEXIA_SK;
                nicWeb = Links.NIC_CS;
                break;
        }
        addImageToContainer(AboutAssets.LOGO_TABLEXIA, LOGO_TABLEXIA_MAX_HEIGHT, tablexiaWeb);
        content.row();

        addDefaultTextToContainer(AboutAssets.ABOUT_INFO_1);

        //add cz.nic logo

        addImageToContainer(AboutAssets.LOGO_NIC, LOGO_CZ_NIC_MAX_HEIGHT, nicWeb);
        content.row();

        //add text from properties
        addDefaultTextToContainer(AboutAssets.ABOUT_INFO_2);

        addDefaultTextToContainer(AboutAssets.ABOUT_INFO_3);

        addParagraphName(AboutAssets.PARAGRAPH_1);

        addBoldTextToContainer(AboutAssets.PSYCHOLOGICAL_CONCEPT);
        addDefaultTextToContainer(AboutAssets.PSYCHOLOGICAL_CONCEPT_DETAILS);

        addBoldTextToContainer(AboutAssets.DEVELOPERS);
        addDefaultTextToContainer(AboutAssets.DEVELOPERS_DETAILS_1);
        addDefaultTextToContainer(AboutAssets.DEVELOPERS_DETAILS_2);

        addBoldTextToContainer(AboutAssets.GAME_CONCEPT);
        addDefaultTextToContainer(AboutAssets.GAME_CONCEPT_DETAILS);

        addBoldTextToContainer(AboutAssets.ILLUSTRATORS);
        addDefaultTextToContainer(AboutAssets.ILLUSTRATORS_DETAILS);

        addParagraphName(AboutAssets.PARAGRAPH_2);
        addDefaultTextToContainer(AboutAssets.PARAGRAPH_2_DETAILS);

        addParagraphName(AboutAssets.PARAGRAPH_3);
        addDefaultTextToContainer(AboutAssets.PARAGRAPH_3_DETAILS_1_1);

        addLabelLink(Links.TABLEXIA_GIT);

        addDefaultTextToContainer(AboutAssets.PARAGRAPH_3_DETAILS_1_2);
        addDefaultTextToContainer(AboutAssets.PARAGRAPH_3_DETAILS_2);

        content.add(createTableWithLibrariesLinks()).width(contentWidth).center();
        content.row();

        addParagraphName(AboutAssets.PARAGRAPH_4);
        addDefaultTextToContainer(AboutAssets.PARAGRAPH_4_DETAILS);

        addSocial();

        addDefaultTextToContainer(AboutAssets.ABOUT_INFO_4);

        addBigLabelLink(tablexiaWeb);

        content.pad(CONTENT_PAD);
    }


    /**
     * Add default text
     *
     * @param textID
     */
    private void addDefaultTextToContainer(String textID) {
        addTextToContainer(textID, LABEL_STYLE_18, false);
        content.row();
    }

    private void addBoldTextToContainer(String textID) {
        addTextToContainer(textID, BOLD_LABEL_STYLE_18, false);
        content.row();
    }

    private void addParagraphName(String textID) {
        addTextToContainer(textID, BOLD_LABEL_STYLE_26, false);
        content.row();
    }

    private void addBigBoldTextToContainer(String textID) {
        addTextToContainer(textID, BOLD_LABEL_STYLE_20, true);
        content.row();
    }

    /**
     * @param textID
     * @param style
     * @param bigFont
     */
    private void addTextToContainer(String textID, TablexiaLabel.TablexiaLabelStyle style, boolean bigFont) {
        float labelPad = DEFAULT_FONT_PAD;
        if (bigFont) {
            labelPad = BIG_FONT_PAD;
        }
        content.add(createStyledLabel(getText(textID), style)).width(contentWidth - SMALL_PAD).padTop(labelPad).padBottom(labelPad);
    }

    //TODO - Fix fillX
    private Table createTableWithLibrariesLinks() {
        Table linksTable = new Table();
        linksTable.align(Align.center);
        List<Links> links = Links.getGroupLinks(LinkGroups.LIBRARY);
        float rowWidthLeft = contentWidth;
        TablexiaLabel label;
        Table temp = new Table();
        for (Links link : links) {
            label = (createLabelWithLink(link, false));
            if (rowWidthLeft < label.getWidth()) {
                linksTable.add(temp).padLeft(DEFAULT_FONT_PAD);
                temp = new Table();
                rowWidthLeft = contentWidth;
                linksTable.row();
            }
            temp.add(label).padLeft(DEFAULT_FONT_PAD);
            rowWidthLeft = rowWidthLeft - (label.getWidth() + DEFAULT_FONT_PAD);
        }
        if (temp.hasChildren()) {
            linksTable.add(temp);
        }
        return linksTable;
    }


    /**
     * Add image to content container
     *
     * @param path      path to image in assets folder
     * @param maxHeight height limit
     */
    private void addImageToContainer(String path, int maxHeight, Links link) {
        Actor actor = new Image(getScreenTextureRegion(path));
        float actorHeight = actor.getHeight();
        float actorWidth = actor.getWidth();
        float actorSizeRatio = actorWidth / actorHeight;
        if (actorHeight > maxHeight) {
            actorHeight = maxHeight;
            actorWidth = actorSizeRatio * actorHeight;
        }
        if (link != null) {
            actor.setUserObject(link);
            actor.addListener(labelClickListener);
        }
        content.add(actor).width(actorWidth).height(actorHeight).align(Align.center).pad(BIG_PAD);
//        content.row();
    }


    private void addSocial() {
        Table socialTable = new Table();
        String[] paths = {AboutAssets.FACEBOOK,AboutAssets.MAIL, AboutAssets.TWITTER};
        int i = 0;
        List<Links> linksList = Links.getGroupLinks(LinkGroups.SOCIAL);
        for (Links links : linksList) {
            Actor actor = new Image(getScreenTextureRegion(paths[i]));
            float actorHeight = actor.getHeight();
            float actorWidth = actor.getWidth();
            float actorSizeRatio = actorWidth / actorHeight;
            if (actorHeight > SOCIAL_MAX_HEIGHT) {
                actorHeight = SOCIAL_MAX_HEIGHT;
                actorWidth = actorSizeRatio * actorHeight;
            }
            actor.setUserObject(links);
            actor.addListener(labelClickListener);
            socialTable.add(actor).width(actorWidth).height(actorHeight).pad(BIG_PAD);
            i++;
        }
        content.add(socialTable).width(contentWidth).center();
        content.row();
    }


    private void addLabelLink(Links link) {
        content.add(createLabelWithLink(link, false)).pad(BIG_PAD);
        content.row();
    }

    private void addBigLabelLink(Links link) {
        content.add(createLabelWithLink(link, true));
        content.row();
    }

    private TablexiaLabel createLabelWithLink(Links link, boolean bigFont) {
        TablexiaLabel.TablexiaLabelStyle style = new TablexiaLabel.TablexiaLabelStyle(ApplicationFontManager.FontType_NEW.REGULAR_18, DEFAULT_LINK_COLOR);
        if (bigFont) {
            style = new TablexiaLabel.TablexiaLabelStyle(ApplicationFontManager.FontType_NEW.REGULAR_20, DEFAULT_LINK_COLOR);
        }
        TablexiaLabel label = new TablexiaLabel(link.getLinkName(), style);
        label.setUserObject(link);
        label.addListener(labelClickListener);
        return label;
    }


    private TablexiaLabel createStyledLabel(String text, TablexiaLabel.TablexiaLabelStyle labelStyle) {
        TablexiaLabel newLabel = new TablexiaLabel(text, labelStyle);
        newLabel.setAlignment(Align.center);
        newLabel.setWrap(true);

        return newLabel;

    }

    private static class LabelClickListener extends ClickListenerWithSound {
        @Override
        public void onClick(InputEvent event, float x, float y) {
            String url = ((Links) event.getListenerActor().getUserObject()).getLinkAddress();
            Gdx.net.openURI(url);
        }
    }
}