package cz.nic.tablexia.screen.createuser;

import com.badlogic.gdx.Input;
import com.badlogic.gdx.audio.Music;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Pixmap;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.scenes.scene2d.Action;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.InputListener;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.actions.RepeatAction;
import com.badlogic.gdx.scenes.scene2d.ui.Button;
import com.badlogic.gdx.scenes.scene2d.ui.Cell;
import com.badlogic.gdx.scenes.scene2d.ui.HorizontalGroup;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;
import com.badlogic.gdx.scenes.scene2d.ui.Stack;
import com.badlogic.gdx.scenes.scene2d.ui.TextField;
import com.badlogic.gdx.scenes.scene2d.ui.VerticalGroup;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.scenes.scene2d.utils.Drawable;
import com.badlogic.gdx.scenes.scene2d.utils.TextureRegionDrawable;
import com.badlogic.gdx.utils.Json;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;

import cz.nic.tablexia.Tablexia;
import cz.nic.tablexia.TablexiaApplication;
import cz.nic.tablexia.TablexiaSettings;
import cz.nic.tablexia.bus.ApplicationBus;
import cz.nic.tablexia.bus.event.StartFullSynchronizationEvent;
import cz.nic.tablexia.game.ranksystem.UserRankManager;
import cz.nic.tablexia.loader.application.ApplicationAtlasManager;
import cz.nic.tablexia.loader.application.ApplicationFontManager;
import cz.nic.tablexia.menu.user.UserAvatarDefinition;
import cz.nic.tablexia.model.UserDAO;
import cz.nic.tablexia.screen.AbstractTablexiaScreen;
import cz.nic.tablexia.screen.createuser.form.FormActorsLayout;
import cz.nic.tablexia.screen.createuser.form.SignatureInputPane;
import cz.nic.tablexia.screen.createuser.form.SignaturePane;
import cz.nic.tablexia.shared.model.User;
import cz.nic.tablexia.util.Log;
import cz.nic.tablexia.util.Point;
import cz.nic.tablexia.util.ScaleUtil;
import cz.nic.tablexia.util.listener.DragActorListener;
import cz.nic.tablexia.util.ui.ClickListenerWithSound;
import cz.nic.tablexia.util.ui.Switch;
import cz.nic.tablexia.util.ui.TablexiaLabel;
import cz.nic.tablexia.util.ui.TablexiaNoBlendingImage;
import cz.nic.tablexia.util.ui.TextFieldWithPlaceholder;
import cz.nic.tablexia.util.ui.button.StandardTablexiaButton;
import cz.nic.tablexia.util.ui.dialog.TablexiaComponentDialog;
import cz.nic.tablexia.util.ui.dialog.TablexiaComponentDialogFactory;
import cz.nic.tablexia.util.ui.dialog.components.AdaptiveSizeDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.CenterPositionDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.DimmerDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.DividerContentDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.FixedSpaceContentDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.ResizableSpaceContentDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.SpeechDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.TablexiaDialogComponentAdapter;
import cz.nic.tablexia.util.ui.dialog.components.TextContentDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.TouchCloseDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.TwoButtonContentDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.ViewportMaximumSizeComponent;

public class FormScreen extends AbstractTablexiaScreen<Void> {
    //TODO - Get rid of all magic values and strings in the code (Use constants)

    private static final Color                                  TEXT_COLOR                      = Color.BLACK;
    private static final ApplicationFontManager.FontType_NEW    TEXT_14                         = ApplicationFontManager.FontType_NEW.REGULAR_14;
    private static final ApplicationFontManager.FontType_NEW    TEXT_20                         = ApplicationFontManager.FontType_NEW.REGULAR_20;
    private static final ApplicationFontManager.FontType_NEW    TEXT_26                         = ApplicationFontManager.FontType_NEW.REGULAR_26;

    private static final int    DEFAULT_USER_AGE                = 10;
    private static final int    MAX_USER_AGE                    = 99;
    private static final int    GENDER_SWITCH_MALE              = 0;
    private static final int    GENDER_SWITCH_FEMALE            = 2;
    private static final int    GENDER_SWITCH_UNKNOWN           = 1;
	private static final int	MAX_NAME_LENGTH					= 15;

    private static final int    SHAKE_ANIMATION_OFFSET          = 10;
    private static final int    SHAKE_ANIMATION_REPEAT_COUNT    = 6;
    private static final float  SHAKE_ANIMATION_STEP_DURATION   = 0.1f;
    private static final float  SIGNATURE_Y_OFFSET_RATION       = 0.7f;

	private static final int    COMMON_INT_CODE_FOR_ENTER		= 13;
	private static final int    ANDROID_INT_CODE_FOR_ENTER		= 10;

    private static final int    TEXT_FIELD_HINT_DELAY           = 5000;

    protected Actor pen, mugshotFrame, stamp, inkpad, ink;
    protected Image stampPlaceholder, mugshotImage, badge, book;
    protected Actor buttonPlus, buttonMinus;
    protected TablexiaLabel ageValue, genderLabel, ageLabel;
    protected Switch switchGender;
    protected Image  imageMale, imageFemale, signaturePlace, clearName;
    protected SignaturePane signaturePane;
    protected Map<FormValidationEnum, Boolean> validations;
    protected Set<String> alreadyPlayedHints;
    private Group mugshots;
    private Timer timer;
    private boolean timerSet;
    private boolean finalStamp; //set true when form is complete to prevent unwanted multiple checking of form

    protected TextField nameField;

    private int         age;
    private User.Gender gender;

    private TablexiaComponentDialog dialog;

    private String mugshot;
    private String signature;

    private float defaultViewportHeight;

    @Override
    public boolean canBePaused() {
        return false;
    }

    @Override
    protected void screenLoaded(Map<String, String> screenState) {
        defaultViewportHeight = getViewportHeight();
        final Group group = new Group();

        validations = new HashMap<FormValidationEnum, Boolean>();
        alreadyPlayedHints = new HashSet<String>();

        Image background = new TablexiaNoBlendingImage(getScreenTextureRegion(FormScreenAssets.TABLE));
        ScaleUtil.setBackgroundBounds(background);
        group.addActor(background);

        Image coverName;

        group.addActor(book = ScaleUtil.createImageSizePosition(getScreenTextureRegion(FormScreenAssets.BACKGROUND), getStage().getWidth() * FormActorsLayout.BACKGROUND_WIDTH, TablexiaSettings.getMinWorldHeight() * FormActorsLayout.BACKGROUND_HEIGHT, getStage().getWidth() * FormActorsLayout.BACKGROUND_X, 0));
        group.addActor(inkpad = ScaleUtil.createImageWidthPosition(getScreenTextureRegion(FormScreenAssets.INKPAD), getStage().getWidth() * FormActorsLayout.INK_PAD_WIDTH, getStage().getWidth() * FormActorsLayout.INK_PAD_X, getStage().getWidth() * FormActorsLayout.INK_PAD_Y));
        group.addActor(mugshotImage = ScaleUtil.createImageWidthPosition(getApplicationTextureRegion(ApplicationAtlasManager.USERMENU_MENUITEM_AVATAR_0), getStage().getWidth() * FormActorsLayout.MUGSHOT_IMAGE_WIDTH, getStage().getWidth() * FormActorsLayout.MUGSHOT_IMAGE_X, getStage().getWidth() * FormActorsLayout.MUGSHOT_IMAGE_Y)); // size for all mugshots
        group.addActor(mugshotFrame = ScaleUtil.createImageWidthPosition(getScreenTextureRegion(FormScreenAssets.MUGSHOT_FRAME), getStage().getWidth() * FormActorsLayout.MUGSHOT_FRAME_WIDTH, getStage().getWidth() * FormActorsLayout.MUGSHOT_FRAME_X, getStage().getWidth() * FormActorsLayout.MUGSHOT_FRAME_Y));
        group.addActor(coverName = ScaleUtil.createImageWidthPosition(getScreenTextureRegion(FormScreenAssets.TEXTFIELD_COVERNAME), getStage().getWidth() * FormActorsLayout.COVER_NAME_WIDTH, getStage().getWidth() * FormActorsLayout.COVER_NAME_X, getStage().getWidth() * FormActorsLayout.COVER_NAME_Y));
        group.addActor(stampPlaceholder = ScaleUtil.createImageWidthPosition(getScreenTextureRegion(FormScreenAssets.STAMP_PLACEHOLDER), getStage().getWidth() * FormActorsLayout.STAMP_PLACEHOLDER_WIDTH, getStage().getWidth() * FormActorsLayout.STAMP_PLACEHOLDER_X, getStage().getWidth() * FormActorsLayout.STAMP_PLACEHOLDER_Y));
        group.addActor(signaturePlace = ScaleUtil.createImageWidthPosition(getScreenTextureRegion(FormScreenAssets.TEXTFIELD_SIGNATURE), getStage().getWidth() * FormActorsLayout.SIGNATURE_PLACE_WIDTH, getStage().getWidth() * FormActorsLayout.SIGNATURE_PLACE_X, getStage().getWidth() * FormActorsLayout.SIGNATURE_PLACE_Y));
        group.addActor(ink = ScaleUtil.createImageWidthPosition(getScreenTextureRegion(FormScreenAssets.INK), getStage().getWidth() * FormActorsLayout.INK_WIDTH, getStage().getWidth() * FormActorsLayout.INK_X, getStage().getHeight() / 2));

        // button for clearing user cover name
        clearName = new Image(getScreenTextureRegion(FormScreenAssets.CLEAR));
        clearName.setPosition(coverName.getX() + coverName.getWidth() + FormActorsLayout.CLEAR_NAME_BUTTON_PADDING, coverName.getY());
        clearName.setSize(clearName.getWidth() / 2, clearName.getHeight() / 2);
        clearName.addListener(new ClickListener() {
            @Override
            public boolean touchDown(InputEvent event, float x, float y, int pointer, int button) {
                nameField.setText("");
                validate();
                return false;
            }
        });
        group.addActor(clearName);

        // TODO pens and stamps variant
        mugshotFrame.addListener(new ClickListener() {
            @Override
            public void clicked(InputEvent event, float x, float y) {
                showMugshotDialog();
                clearValidations();
            }
        });

        this.age = DEFAULT_USER_AGE;
        TablexiaLabel.TablexiaLabelStyle labelStyleFontSize16 = new TablexiaLabel.TablexiaLabelStyle(TEXT_14, TEXT_COLOR);
        TablexiaLabel.TablexiaLabelStyle labelStyleFontSize20 = new TablexiaLabel.TablexiaLabelStyle(TEXT_20, TEXT_COLOR);

        ageLabel = new TablexiaLabel(getText(FormScreenAssets.TEXT_FORM_AGE), labelStyleFontSize16);
        ageValue = new TablexiaLabel(String.valueOf(this.age), labelStyleFontSize20);
        genderLabel = new TablexiaLabel(getText(FormScreenAssets.TEXT_FORM_GENDER), labelStyleFontSize16);

        nameField = new TextFieldWithPlaceholder("", new TextFieldWithPlaceholder.TablexiaTextFieldStyle(ApplicationFontManager.FontType_NEW.REGULAR_18, TEXT_COLOR, createCursorForTextField(), null, null), getText("createuser_form_username"));
        nameField.setSize(getStage().getWidth() * FormActorsLayout.NAME_FIELD_WIDTH, getStage().getWidth() * FormActorsLayout.NAME_FIELD_HEIGHT);
        nameField.setPosition(getStage().getWidth() * FormActorsLayout.NAME_FIELD_X, getStage().getWidth() * FormActorsLayout.NAME_FIELD_Y);
        nameField.setFocusTraversal(false);
		nameField.setMaxLength(MAX_NAME_LENGTH);

        timerSet = false;

        nameField.addListener(new InputListener() {
            @Override
            public boolean keyTyped(InputEvent event, char character) {

                if(timerSet) timer.cancel();

                // have to check also (int)character because of some bug in iOS keyboard. It returns code Input.Keys.ENTER for every key after "done" button pressed
                if ((event.getKeyCode() == Input.Keys.ENTER && ((int)character == ANDROID_INT_CODE_FOR_ENTER || (int)character == COMMON_INT_CODE_FOR_ENTER)) || event.getKeyCode() == Input.Keys.TAB) {
                    validate();
                } else {
                    timer = new Timer();
                    timer.schedule(new TimerTask() {
                        @Override
                        public void run() {
                            validate();
                            timerSet = false;
                        }
                    }, TEXT_FIELD_HINT_DELAY);
                    timerSet = true;
                }
                return false;
            }
        });

        ageLabel.setPosition(getStage().getWidth() * FormActorsLayout.AGE_LABEL_X, getStage().getWidth() * FormActorsLayout.AGE_LABEL_Y);
        ageValue.setPosition(getStage().getWidth() * FormActorsLayout.AGE_VALUE_X, getStage().getWidth() * FormActorsLayout.AGE_VALUE_Y);
        genderLabel.setPosition(getStage().getWidth() * FormActorsLayout.GENDER_LABEL_X, getStage().getWidth() * FormActorsLayout.GENDER_LABEL_Y);

        group.addActor(nameField);
        group.addActor(ageValue);
        group.addActor(ageLabel);
        group.addActor(genderLabel);
        group.addActor(imageFemale = ScaleUtil.createImageWidthPosition(getScreenTextureRegion(FormScreenAssets.SEX_FEMALE), getStage().getWidth() * FormActorsLayout.IMAGE_FEMALE_WIDTH, getStage().getWidth() * FormActorsLayout.IMAGE_FEMALE_X, getStage().getWidth() * FormActorsLayout.IMAGE_FEMALE_Y));
        group.addActor(imageMale = ScaleUtil.createImageWidthPosition(getScreenTextureRegion(FormScreenAssets.SEX_MALE), getStage().getWidth() * FormActorsLayout.IMAGE_MALE_WIDTH, getStage().getWidth() * FormActorsLayout.IMAGE_MALE_X, getStage().getWidth() * FormActorsLayout.IMAGE_MALE_Y));
        group.addActor(prepareBadge());
        group.addActor(buttonMinus = new Button(new TextureRegionDrawable(getScreenTextureRegion(FormScreenAssets.BUTTON_MINUS)), new TextureRegionDrawable(getScreenTextureRegion(FormScreenAssets.BUTTON_MINUS_PRESSED))));
        group.addActor(buttonPlus = new Button(new TextureRegionDrawable(getScreenTextureRegion(FormScreenAssets.BUTTON_PLUS)), new TextureRegionDrawable(getScreenTextureRegion(FormScreenAssets.BUTTON_PLUS_PRESSED))));
        buttonMinus.setBounds(getStage().getWidth() * FormActorsLayout.BUTTON_MINUS_X, getStage().getWidth() * FormActorsLayout.BUTTON_MINUS_Y, getStage().getWidth() * FormActorsLayout.BUTTON_MINUS_WIDTH, getStage().getWidth() * FormActorsLayout.BUTTON_MINUS_HEIGHT);
        buttonPlus.setBounds(getStage().getWidth() * FormActorsLayout.BUTTON_PLUS_X, getStage().getWidth() * FormActorsLayout.BUTTON_PLUS_Y, getStage().getWidth() * FormActorsLayout.BUTTON_PLUS_WIDTH, getStage().getWidth() * FormActorsLayout.BUTTON_PLUS_HEIGHT);
        buttonPlus.addListener(new ClickListenerWithSound() {
            @Override
            public void onClick(InputEvent event, float x, float y) {
                setAge(Math.min(FormScreen.this.age + 1, MAX_USER_AGE));
				setValid(FormValidationEnum.AGE, true);
                validate();
            }
        });
        buttonMinus.addListener(new ClickListenerWithSound() {
            @Override
            public void onClick(InputEvent event, float x, float y) {
                setAge(Math.max(FormScreen.this.age - 1, 1));
				setValid(FormValidationEnum.AGE, true);
                validate();
            }
        });

        // TODO images as parameters for Switch's constructor
        switchGender = new Switch(getScreenTextureRegion(FormScreenAssets.SWITCH_BACKGROUND), getScreenTextureRegion(FormScreenAssets.SWITCH_LEFT), getScreenTextureRegion(FormScreenAssets.SWITCH_CENTER), getScreenTextureRegion(FormScreenAssets.SWITCH_RIGHT));
        switchGender.setPosition(getStage().getWidth() * FormActorsLayout.SWITCH_GENDER_X, getStage().getWidth() * FormActorsLayout.SWITCH_GENDER_Y);
        switchGender.setScale(FormActorsLayout.SWITCH_GENDER_SCALE);
        switchGender.setSwitchSelectedListener(new Switch.DragSwitchListener.SwitchSelectedListener() {
            @Override
            public void stepSelected(int step) {
                if (step == GENDER_SWITCH_UNKNOWN) {
                    FormScreen.this.gender = null;
                } else {
                    FormScreen.this.gender = (step == GENDER_SWITCH_FEMALE ? User.Gender.FEMALE : User.Gender.MALE);
                    validate();
                }
            }
        });
        switchGender.switchToStep(GENDER_SWITCH_UNKNOWN);
        switchGender.setDisabledStep(GENDER_SWITCH_UNKNOWN);
        group.addActor(switchGender);
        imageFemale.addListener(new ClickListenerWithSound() {
            @Override
            public void onClick(InputEvent event, float x, float y) {
                switchGender.switchToStep(GENDER_SWITCH_FEMALE);
                validate();
            }
        });

        imageMale.addListener(new ClickListenerWithSound() {
            @Override
            public void onClick(InputEvent event, float x, float y) {
                switchGender.switchToStep(GENDER_SWITCH_MALE);
                validate();
            }
        });

        // signature pen needs to be below pen and stamp
        signaturePane = new SignaturePane();
        group.addActor(signaturePane);
        signaturePane.setBounds(getSceneWidth() * FormActorsLayout.SIGNATURE_PANE_X, getSceneInnerHeight() * FormActorsLayout.SIGNATURE_PANE_Y, getSceneWidth() * FormActorsLayout.SIGNATURE_PANE_WIDTH, getSceneInnerHeight() * FormActorsLayout.SIGNATURE_PANE_HEIGHT);

        signaturePane.setDebug(TablexiaSettings.getInstance().isShowBoundingBoxes());

        // pencil and stamp layers
        group.addActor(pen = ScaleUtil.createImageWidthPosition(getScreenTextureRegion(FormScreenAssets.PENCIL_DROPPED), getStage().getWidth() * FormActorsLayout.PEN_WIDTH, getStage().getWidth() * FormActorsLayout.PEN_X, getStage().getWidth() * FormActorsLayout.PEN_Y));
        group.addActor(stamp = ScaleUtil.createImageWidthPosition(getScreenTextureRegion(FormScreenAssets.STAMP_LEFT_DROPPED), getStage().getWidth() * FormActorsLayout.STAMP_WIDTH, getStage().getWidth() * FormActorsLayout.STAMP_X, getStage().getWidth() * FormActorsLayout.STAMP_Y));
        pen.addListener(new DragActorListener(pen, true, new DragActorListener.DragListenerAdapter() {
            @Override
            public void onDropped(float x, float y) {
                if ((x > FormActorsLayout.PEN_DROP_AREA_LEFT_X && x < FormActorsLayout.PEN_DROP_AREA_RIGHT_X) && (y > FormActorsLayout.PEN_DROP_AREA_BOTTOM_Y && y < FormActorsLayout.PEN_DROP_AREA_TOP_Y)) {
                    showSubscribeDialog();
                }
            }

            @Override
            public void onStarted(float x, float y) {
                clearValidations();
                pen.clearActions();
            }

            @Override
            public void onMoveBackStart(float x, float y) {
                validate();
            }
        }));

        stamp.addListener(new DragActorListener(stamp, true, new DragActorListener.DragListenerAdapter() {
            @Override
            public void onDropped(float x, float y) {
                if ((x > FormActorsLayout.STAMP_DROP_AREA_LEFT_X && x < FormActorsLayout.STAMP_DROP_AREA_RIGHT_X) && (y > FormActorsLayout.STAMP_DROP_AREA_BOTTOM_Y && y < FormActorsLayout.STAMP_DROP_AREA_TOP_Y)) {
                    finalStamp = true;
                    if (!validate()) {
                        Log.debug("Validator", "Form is not valid");
                        group.setTouchable(Touchable.enabled);
                        group.addListener(new InputListener() {
                            @Override
                            public boolean touchDown(InputEvent event, float x, float y, int pointer, int button) {
                                group.removeListener(this);
                                group.setTouchable(Touchable.childrenOnly);
                                return super.touchDown(event, x, y, pointer, button);
                            }
                        });
                        return;
                    }
					clearValidations();
					setValid(FormValidationEnum.STAMP, true);
                    stampIt();
                    User user = UserDAO.createUser(nameField.getText().trim(), FormScreen.this.age, FormScreen.this.gender, mugshot, signature);
                    TablexiaSettings.getInstance().changeUser(user);
                    ApplicationBus.getInstance().post(new Tablexia.ChangeScreenEvent(TablexiaSettings.INITIAL_SCREEN, TablexiaApplication.ScreenTransaction.FADE)).asynchronously();
                    ApplicationBus.getInstance().publishAsync(new StartFullSynchronizationEvent(user));
                }
            }

            @Override
            public void onStarted(float x, float y) {
                clearValidations();
                clearStampUseMeAction();
            }

            @Override
            public void onMoveBackStart(float x, float y) {
                validate();
            }
        }));

        getStage().addActor(group);

        validate();
    }

    protected Actor prepareBadge() {
        badge = ScaleUtil.createImageWidthPosition(ApplicationAtlasManager.getInstance().getTextureRegion(UserRankManager.UserRank.getInitialRank().getBadgeImageKey()), getStage().getWidth() * FormActorsLayout.BADGE_WIDTH, getStage().getWidth() * FormActorsLayout.BADGE_X, getStage().getWidth() * FormActorsLayout.BADGE_Y);
        return badge;
    }

    private boolean validate() {

        if (validationEnabled() == false) {
            return true;
        }

        clearValidations();

		setValid(FormValidationEnum.MUGSHOT, mugshot != null);
        if (isValid(FormValidationEnum.MUGSHOT) == false) {
            displayHintDialog(FormValidationEnum.MUGSHOT, mugshotImage);
            playValidationSound(FormScreenAssets.VALIDATION_MUGSHOT_SOUND);
            return false;
        }

		setValid(FormValidationEnum.NAME, nameField.getText() != null && !nameField.getText().isEmpty() && nameField.getText().trim().length() > 0);
        if (isValid(FormValidationEnum.NAME) == false) {
            displayHintDialog(FormValidationEnum.NAME, nameField);
            playValidationSound(FormScreenAssets.VALIDATION_USERNAME_SOUND);
            return false;
        }

		setValid(FormValidationEnum.NAME, !UserDAO.userNameExists(nameField.getText().trim()));
        if (isValid(FormValidationEnum.NAME) == false && !finalStamp) {
            displayHintDialog(FormValidationEnum.NAME_EXISTS, nameField, true);
            return false;
        }

        if (isValid(FormValidationEnum.AGE) == false) {
            displayHintDialog(FormValidationEnum.AGE, ageValue);
            playValidationSound(FormScreenAssets.VALIDATION_AGE_SOUND);
            return false;
        }

		setValid(FormValidationEnum.GENDER, gender != null);
        if (isValid(FormValidationEnum.GENDER) == false) {
            displayHintDialog(FormValidationEnum.GENDER, switchGender);
            playValidationSound(FormScreenAssets.VALIDATION_GENDER_SOUND);
            return false;
        }

		setValid(FormValidationEnum.SIGNATURE, signature != null);
        if (isValid(FormValidationEnum.SIGNATURE) == false) {
            displayHintDialog(FormValidationEnum.SIGNATURE, signaturePlace);
            playValidationSound(FormScreenAssets.VALIDATION_SIGNATURE_SOUND);
            pen.addAction(Actions.forever(Actions.sequence(Actions.repeat(SHAKE_ANIMATION_REPEAT_COUNT, Actions.sequence(Actions.moveBy(SHAKE_ANIMATION_OFFSET, 0, SHAKE_ANIMATION_STEP_DURATION), Actions.moveBy(-SHAKE_ANIMATION_OFFSET, 0, SHAKE_ANIMATION_STEP_DURATION))), Actions.delay(1))));
            return false;
        }

       if (isValid(FormValidationEnum.STAMP) == false && !finalStamp) {
            displayHintDialog(FormValidationEnum.STAMP, stampPlaceholder);
            playValidationSound(FormScreenAssets.VALIDATION_STAMP_SOUND);
            stamp.addAction(Actions.forever(Actions.sequence(Actions.repeat(SHAKE_ANIMATION_REPEAT_COUNT, Actions.sequence(Actions.moveBy(SHAKE_ANIMATION_OFFSET, 0, SHAKE_ANIMATION_STEP_DURATION), Actions.moveBy(-SHAKE_ANIMATION_OFFSET, 0, SHAKE_ANIMATION_STEP_DURATION))), Actions.delay(1))));
        }

        return true;
    }

	private void setValid(FormValidationEnum key, boolean valid) {
		validations.put(key, valid);
		nameField.setTouchable(canEdit(FormValidationEnum.NAME) ? Touchable.enabled : Touchable.disabled);
		switchGender.setTouchable(canEdit(FormValidationEnum.GENDER) ? Touchable.enabled : Touchable.disabled);
		imageFemale.setTouchable(canEdit(FormValidationEnum.GENDER) ? Touchable.enabled : Touchable.disabled);
		imageMale.setTouchable(canEdit(FormValidationEnum.GENDER) ? Touchable.enabled : Touchable.disabled);
		buttonPlus.setTouchable(canEdit(FormValidationEnum.AGE) ? Touchable.enabled : Touchable.disabled);
		buttonMinus.setTouchable(canEdit(FormValidationEnum.AGE) ? Touchable.enabled : Touchable.disabled);
		pen.setTouchable(canEdit(FormValidationEnum.SIGNATURE) ? Touchable.enabled : Touchable.disabled);
		stamp.setTouchable(canEdit(FormValidationEnum.STAMP) ? Touchable.enabled : Touchable.disabled);
	}

	private boolean canEdit(FormValidationEnum key) {
		List<FormValidationEnum> shouldBeValid = key.getPredecessors();
		if (shouldBeValid != null) {
			for (FormValidationEnum validation : shouldBeValid) {
				if (!(validations.containsKey(validation) && validations.get(validation))) {
					return false;
				}
			}
		}
		return true;
	}

    private boolean isValid(FormValidationEnum key) {
        return (validations.containsKey(key) && validations.get(key));
    }

    private void clearValidations() {
        clearPenUseMeAction();
        clearStampUseMeAction();

        if (dialog != null)
            dialog.hide();
    }

    private void clearPenUseMeAction() {
        for (Action action : pen.getActions()) {
            if (action instanceof RepeatAction) {
                pen.removeAction(action);
            }
        }
    }

    private void clearStampUseMeAction() {
        for (Action action : stamp.getActions()) {
            if (action instanceof RepeatAction) {
                stamp.removeAction(action);
            }
        }
    }

    protected void stampIt() {
        Drawable d = new TextureRegionDrawable(new TextureRegion(getScreenTextureRegion(FormScreenAssets.STAMP)));
        stampPlaceholder.setDrawable(d);
    }

    private void showMugshotDialog() {
        mugshots = new Group();

        // grey translucent overlay
        Image overlay = new Image(ApplicationAtlasManager.getInstance().getColorTexture(ApplicationAtlasManager.COLOR_OVERLAY));
        mugshots.addActor(ScaleUtil.setBackgroundBounds(overlay));
        overlay.addListener(new InputListener() {
            @Override
            public boolean touchDown(InputEvent event, float x, float y, int pointer, int button) {
                event.stop();
                mugshots.remove();
                return false;
            }
        });


        // avatar pictures
        HorizontalGroup hg = new HorizontalGroup();
        for (int i = 0; i < TablexiaSettings.AVATAR_COUNT; i++) {
            int ordinal = UserAvatarDefinition.getOrdinalByOrder(i + 1);
            final Actor avatar = prepareAvatarPicture(ordinal);
            avatar.setName(String.valueOf(ordinal));
            avatar.addListener(new ClickListener() {
                @Override
                public void clicked(InputEvent event, float x, float y) {
                    event.stop();
                    mugshots.remove();
                    Drawable d = new TextureRegionDrawable(new TextureRegion(getApplicationTextureRegion(UserAvatarDefinition.getAvatar(avatar.getName()))));
                    mugshotImage.setDrawable(d);
                    mugshot = avatar.getName();
                    validate();
                }
            });
            hg.addActor(avatar);
            hg.setHeight(avatar.getHeight());
        }


        ScrollPane sp = new ScrollPane(hg);
        sp.setScrollingDisabled(false, true);
        sp.setSize(getSceneWidth(), hg.getHeight());
        sp.setPosition(0, FormActorsLayout.MUGSHOTS_SCROLL_PANE_Y);
        mugshots.addActor(sp);
        getStage().addActor(mugshots);
    }

    private Actor prepareAvatarPicture(int num) {
        TextureRegion textureRegion = getApplicationTextureRegion(UserAvatarDefinition.getAvatar(num));
        float mugshotWidth = ScaleUtil.getWidth(textureRegion.getRegionWidth(), textureRegion.getRegionHeight(), FormActorsLayout.MUGSHOT_HEIGHT);

        Image avatar = ScaleUtil.createImageToHeight(getApplicationTextureRegion(UserAvatarDefinition.getAvatar(num)), FormActorsLayout.MUGSHOT_HEIGHT);
        avatar.getDrawable().setMinHeight(FormActorsLayout.MUGSHOT_HEIGHT);
        avatar.getDrawable().setMinWidth(mugshotWidth);
        Image film = ScaleUtil.createImageToHeight(getScreenTextureRegion(FormScreenAssets.FILM_INSIDE), FormActorsLayout.MUGSHOT_HEIGHT);
        film.getDrawable().setMinHeight(FormActorsLayout.MUGSHOT_HEIGHT);
        film.getDrawable().setMinWidth(mugshotWidth);

        VerticalGroup vg = new VerticalGroup();
        Image frameTop = ScaleUtil.createImageToWidth(getScreenTextureRegion(FormScreenAssets.FILM_OUTERLINE), mugshotWidth);
        frameTop.getDrawable().setMinWidth(mugshotWidth);
        frameTop.getDrawable().setMinHeight(frameTop.getHeight());
        Image frameBottom = ScaleUtil.createImageToWidth(getScreenTextureRegion(FormScreenAssets.FILM_OUTERLINE), mugshotWidth);
        frameBottom.getDrawable().setMinWidth(mugshotWidth);
        frameBottom.getDrawable().setMinHeight(frameTop.getHeight());

        Stack stack = new Stack();
        stack.setSize(mugshotWidth, FormActorsLayout.MUGSHOT_HEIGHT);
        stack.addActor(avatar);
        stack.addActor(film);

        vg.addActor(frameTop);
        vg.addActor(stack);
        vg.addActor(frameBottom);

        vg.setHeight(FormActorsLayout.MUGSHOT_HEIGHT + frameBottom.getHeight() + frameTop.getHeight());
        vg.setWidth(mugshotWidth);

        return vg;
    }

    public void setAge(int age) {
        ageValue.setText(String.valueOf(this.age = age));
    }

    private void showSubscribeDialog() {
        ArrayList<TablexiaDialogComponentAdapter> adapters = new ArrayList<TablexiaDialogComponentAdapter>();

        final SignatureInputPane signatureInputPane = new SignatureInputPane();

        adapters.add(new AdaptiveSizeDialogComponent());
        adapters.add(new ViewportMaximumSizeComponent());
        adapters.add(new CenterPositionDialogComponent());
        adapters.add(new DimmerDialogComponent(0.7f));

        //Signature title and spacers...
        adapters.add(new FixedSpaceContentDialogComponent());
        adapters.add(new TextContentDialogComponent(getText(FormScreenAssets.TEXT_FORM_SIGNATURE), TEXT_26));
        adapters.add(new FixedSpaceContentDialogComponent());
        adapters.add(new DividerContentDialogComponent());
        adapters.add(new FixedSpaceContentDialogComponent());

        //Anonymous signature component
        adapters.add(new TablexiaDialogComponentAdapter() {
            //TODO 4 Desktop - Override sizeChanged?? Resize SignaturePane when resizing the window
            @Override
            public void prepareContent(Cell content) {
                content.getTable().row();
                content.setActor(signatureInputPane)
                        .expand()
                        .fill();
            }
        });

        //Separator between the signarute and buttons components
        adapters.add(new FixedSpaceContentDialogComponent());

        //Two button component (Cancel and Sign button)
        adapters.add(new TwoButtonContentDialogComponent(
                getText(FormScreenAssets.TEXT_FORM_CANCEL),
                getText(FormScreenAssets.TEXT_FORM_SIGN),
                StandardTablexiaButton.TablexiaButtonType.RED,
                StandardTablexiaButton.TablexiaButtonType.GREEN,
                new ClickListener() {
                    @Override
                    public void clicked(InputEvent event, float x, float y) {
                        cancel();
                    }
                },
                new ClickListener() {
                    @Override
                    public void clicked(InputEvent event, float x, float y) {
                        cancel();
                        new Thread(new Runnable() {
                            @Override
                            public void run() {
                                Json json = new Json();
                                signature = json.toJson(signatureInputPane.getSignature());
                                signaturePane.setSignature(signatureInputPane.getSignature());
                                FormScreen.this.validate();
                            }
                        }).start();
                    }
                }
        ));

        //Show dialog
        TablexiaComponentDialogFactory.getInstance().createDialog(adapters.toArray(new TablexiaDialogComponentAdapter[]{})).show(getSceneWidth(), getSceneInnerHeight());;
        clearValidations();
    }

    protected boolean validationEnabled() {
        return true;
    }

    @Override
    protected void screenResized(int width, int height) {
        //TODO 4 Desktop - Resize SignaturePane when resizing the window

        if (signaturePane != null) {
            signaturePane.setPointOffsetY((defaultViewportHeight / 2 - getViewportHeight() / 2) * (getSceneInnerHeight() * SIGNATURE_Y_OFFSET_RATION / getViewportHeight()));
        }
    }

    private Drawable createCursorForTextField() {
        Pixmap pixmap = new Pixmap(1, TEXT_20.getSize(), Pixmap.Format.RGBA8888);
        pixmap.setColor(Color.WHITE);
        pixmap.drawLine(0, 0, 0, TEXT_20.getSize());
        Texture texture = new Texture(pixmap);
        pixmap.dispose();
        Image img = new Image(texture);
        return img.getDrawable();
    }

	private void displayHintDialog(final FormValidationEnum validation, Actor relatedActor) {
		displayHintDialog(validation, relatedActor, false);
	}

	private void displayHintDialog(final FormValidationEnum validation, Actor relatedActor, boolean biggerDialog) {
        ArrayList<TablexiaDialogComponentAdapter> components = new ArrayList<TablexiaDialogComponentAdapter>();
        components.add(new AdaptiveSizeDialogComponent());
        components.add(new SpeechDialogComponent(
                new Point(relatedActor.getX() + relatedActor.getWidth()/2, relatedActor.getY() + relatedActor.getHeight()/2),
                new Vector2(16, 4),
                SpeechDialogComponent.ArrowType.BEND_ARROW));

        components.add(new FixedSpaceContentDialogComponent());
        components.add(new ResizableSpaceContentDialogComponent());

        components.add(new TextContentDialogComponent(getText(validation.getHintText()), true, true));

        components.add(new ResizableSpaceContentDialogComponent());
        components.add(new FixedSpaceContentDialogComponent());

		components.add(new TouchCloseDialogComponent() {
			@Override
			public void dialogTouched(float x, float y) {
				if (validation == FormValidationEnum.AGE) {
					setValid(validation, true);
					validate();
				}
			}
		});

        dialog = TablexiaComponentDialogFactory.getInstance().createDialog(
                getStage(),
                TablexiaComponentDialog.TablexiaDialogType.BUBBLE_SQUARE,
                components.toArray(new TablexiaDialogComponentAdapter[]{}));
        dialog.show(biggerDialog ? FormActorsLayout.HINT_DIALOG_BIGGER_WIDTH : FormActorsLayout.HINT_DIALOG_WIDTH, biggerDialog ? FormActorsLayout.HINT_DIALOG_BIGGER_HEIGHT : FormActorsLayout.HINT_DIALOG_HEIGHT);
    }

    private void playValidationSound(String sound) {
        if (alreadyPlayedHints.contains(sound)) {
            return;
        }

        final Music mu = getMusic(sound);
        mu.play();
        mu.setOnCompletionListener(new Music.OnCompletionListener() {
            @Override
            public void onCompletion(Music music) {
                mu.dispose();
            }
        });
        alreadyPlayedHints.add(sound);
    }

    @Override
    public void backButtonPressed() {
            if(mugshots != null) {
                mugshots.remove();
                mugshots = null;
            } else {
                ApplicationBus.getInstance().post(new Tablexia.ChangeScreenEvent(TablexiaSettings.LOADER_SCREEN, TablexiaApplication.ScreenTransaction.FADE)).asynchronously();
            }
    }

	private enum FormValidationEnum {
		MUGSHOT(null, "validation_mugshot"),
		NAME(new ArrayList<FormValidationEnum>(Arrays.asList(MUGSHOT)), "validation_username"),
		NAME_EXISTS(new ArrayList<FormValidationEnum>(Arrays.asList(MUGSHOT)), "validation_username_exists"),
		AGE(new ArrayList<FormValidationEnum>(Arrays.asList(MUGSHOT, NAME)), "validation_age"),
		GENDER(new ArrayList<FormValidationEnum>(Arrays.asList(MUGSHOT, NAME, AGE)), "validation_gender"),
		SIGNATURE(new ArrayList<FormValidationEnum>(Arrays.asList(MUGSHOT, NAME, AGE, GENDER)), "validation_signature"),
		STAMP(new ArrayList<FormValidationEnum>(Arrays.asList(MUGSHOT, NAME, AGE, GENDER, SIGNATURE)), "validation_stamp");

		private List<FormValidationEnum> predecessors;
		private String hintText;

		FormValidationEnum(List<FormValidationEnum> predecessors, String text) {
			this.predecessors = predecessors;
			hintText = text;
		}

		public List<FormValidationEnum> getPredecessors() {
			return predecessors;
		}

		public String getHintText() {
			return hintText;
		}
	}
}