package cz.nic.tablexia.screen.createuser;

import com.badlogic.gdx.Input;
import com.badlogic.gdx.audio.Music;
import com.badlogic.gdx.audio.Sound;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Rectangle;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.scenes.scene2d.Action;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Event;
import com.badlogic.gdx.scenes.scene2d.EventListener;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.InputListener;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.actions.RepeatAction;
import com.badlogic.gdx.scenes.scene2d.actions.RunnableAction;
import com.badlogic.gdx.scenes.scene2d.actions.SequenceAction;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;
import com.badlogic.gdx.scenes.scene2d.ui.Stack;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.utils.Scaling;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import cz.nic.tablexia.Tablexia;
import cz.nic.tablexia.TablexiaApplication;
import cz.nic.tablexia.TablexiaSettings;
import cz.nic.tablexia.bus.ApplicationBus;
import cz.nic.tablexia.loader.application.ApplicationAtlasManager;
import cz.nic.tablexia.screen.AbstractTablexiaScreen;
import cz.nic.tablexia.screen.createuser.panorama.PanoramaActorsLayout;
import cz.nic.tablexia.util.Point;
import cz.nic.tablexia.util.ScaleUtil;
import cz.nic.tablexia.util.Utility;
import cz.nic.tablexia.util.ui.ComponentScaleUtil;
import cz.nic.tablexia.util.ui.ScrollPaneWithBorders;
import cz.nic.tablexia.util.ui.TablexiaNoBlendingImage;
import cz.nic.tablexia.util.ui.dialog.TablexiaComponentDialog;
import cz.nic.tablexia.util.ui.dialog.TablexiaComponentDialogFactory;
import cz.nic.tablexia.util.ui.dialog.components.AdaptiveSizeDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.CenterPositionDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.DimmerDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.FixedSpaceContentDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.ImageContentDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.ResizableSpaceContentDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.SpeechDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.TablexiaDialogComponentAdapter;
import cz.nic.tablexia.util.ui.dialog.components.TextContentDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.TouchCloseDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.ViewportMaximumSizeComponent;

/**
 * Screen showing street animation and office before user create form
 */
public class PanoramaScreen extends AbstractTablexiaScreen<int[][]> {

    private static Vector2 NEWSPAPER_TITLE_POSITION = new Vector2(0.26f, 0.197f);
    private static Vector2 SIGN_POSITION            = new Vector2(0.30f, 0.285f);
    private static Vector2 PLATE_TITLE_POSITION     = new Vector2(0.368f, 0.638f);
    private static Vector2 DOOR_SIGN_POSITION       = new Vector2(0.575f, 0f);

    public static final String GFX_PATH = "gfx/";
    public static final String SFX_PATH = "sfx/";
    public static final String MFX_PATH = "mfx/";

    public static final String BALCONY             = GFX_PATH + "balcony";
    public static final String BALCONY_WINTER      = GFX_PATH + "balcony_winter";
    public static final String BALCONY_NEWS_TITLE  = GFX_PATH + "balcony_news_title";
    public static final String BALCONY_BLUR        = GFX_PATH + "balcony_blur";
    public static final String BALCONY_BLUR_WINTER = GFX_PATH + "balcony_blur_winter";

    public static final String SWIPE_OVERSCROLL_RIGHT      = GFX_PATH + "swipe/overscroll_right";
    public static final Color  SWIPE_OVERSCROLL_LEFT_COLOR = new Color(118/255f, 133/255f, 200/255f, 1);

    private static final String NEWSPAPER_DETAIL = "newspaper/detail0";

    public static final String SFX_NEWSPAPER_SPINNER = SFX_PATH + "noviny_prilet.mp3";
    public static final String SFX_NEWSPAPER_OPEN    = SFX_PATH + "noviny_rozbaleni.mp3";
    public static final String SFX_KNOCK_1           = SFX_PATH + "tuk_1.mp3";
    public static final String SFX_KNOCK_2           = SFX_PATH + "tuk_2.mp3";
    public static final String SFX_KNOCK_3           = SFX_PATH + "tuk_3.mp3";

    public static final String MUSIC_5 = MFX_PATH + "newspaper/5.mp3";
    public static final String MUSIC_1 = MFX_PATH + "newspaper/1.mp3";
    public static final String MUSIC_2 = MFX_PATH + "newspaper/2.mp3";
    public static final String MUSIC_3 = MFX_PATH + "newspaper/3.mp3";
    public static final String MUSIC_4 = MFX_PATH + "newspaper/4.mp3";

    public static final float    AMBIENT_SOUND_VOLUME = 0.5f;

    private static final String  AMBIENT_RESTAURANT  = MFX_PATH + "ambient/restaurant.mp3";
    private static final String  AMBIENT_STREET      = MFX_PATH + "ambient/swipe_street.mp3";

    private static final int     NUMBER_OF_NEWSPAPER_OPEN_FRAMES        = 5;
    private static final int     NUMBER_OF_OFFICE_KNOCKS                = 3;
    private static final int     NUMBER_OF_OFFICE_DIALOGS               = 4;

    private static final float   OVERSCROLL_DISTANCE                    = 50;
    private static final float   OVERSCROLL_MIN_SPEED                   = 30;
    private static final float   OVERSCROLL_MAX_SPEED                   = 200;

    private static final Vector2 OFFICE_DIALOG_OFFSET                   = new Vector2(24, 12);
    private static final int     OFFICE_DIALOG_TEXT_PADDING_LEFT        = 4;
    private static final int     OFFICE_DIALOG_TEXT_PADDING_RIGHT       = 24;

	private static final int     NEWSPAPERS_SPEECH_DIALOG_OFFSET		= 4;
	private static final float   NEWSPAPERS_SPEECH_DIALOG_OFFSET_MULTI	= 0.6f;
    private static final float   NEWSPAPER_DETAIL_DIALOG_DIMMER         = 0.6f;

    private static final float   NEWS_DIALOG_MAX_WIDTH                  = 0.85f;
    private static final float   NEWS_DIALOG_MAX_HEIGHT                 = 0.85f;

    private TextureRegion clickmap;

    private Group panel;
    private ScrollPaneWithBorders scrollpane;
    private Music ambient;

    private Image leftFinger, rightFinger, hint;
    private Stack newspaperStages;
    private boolean newspapersOpened = false;

    private Image hintFinger;
    private float lastScroll = Float.NEGATIVE_INFINITY;
    private Action lastHintFingerAction;
    private TablexiaComponentDialog newsDetailDialog;
    private TablexiaComponentDialog newsSpeechDialog;

    @Override
    public boolean canBePaused() {
        return false;
    }

    @Override
    protected int[][] prepareScreenData(Map<String, String> screenState) {
        clickmap = getScreenTextureRegion(GFX_PATH + "newspaper/clickmap");
        if (!clickmap.getTexture().getTextureData().isPrepared()) {
            clickmap.getTexture().getTextureData().prepare();
        }
        return Utility.createColorMap(clickmap);
    }

    @Override
    protected void screenLoaded(Map<String, String> screenState) {
        prepareBalcony();
    }

    @Override
    protected void prepareScreenSoundAssetNames(List<String> soundsFileNames) {
        soundsFileNames.add(SFX_NEWSPAPER_SPINNER);
        soundsFileNames.add(SFX_NEWSPAPER_OPEN);
        soundsFileNames.add(SFX_KNOCK_1);
        soundsFileNames.add(SFX_KNOCK_2);
        soundsFileNames.add(SFX_KNOCK_3);
    }

    /**
     * Create balcony view waiting for click
     * Shows winter image based on current time
     */
    private void prepareBalcony() {
        boolean isWinter = TablexiaSettings.getInstance().isWinterMode();
        ambient = getMusic(AMBIENT_RESTAURANT);
        ambient.setLooping(true);
        ambient.setVolume(AMBIENT_SOUND_VOLUME);
        ambient.play();

        final Image balcony = new TablexiaNoBlendingImage(getScreenTextureRegion(isWinter ? BALCONY_WINTER : BALCONY));
        setActorToFullScene(balcony);
        getStage().addActor(balcony);

        Image balconyNewsTitle = ScaleUtil.createImageWidthPosition(getScreenTextureRegion(BALCONY_NEWS_TITLE),
                balcony.getWidth() * PanoramaActorsLayout.NEWSPAPERS_TITLE_WIDTH,
                NEWSPAPER_TITLE_POSITION.x * balcony.getWidth(),
                NEWSPAPER_TITLE_POSITION.y * balcony.getHeight());
        balconyNewsTitle.setTouchable(Touchable.disabled);
        getStage().addActor(balconyNewsTitle);

        final Image balconyBlur = new TablexiaNoBlendingImage(getScreenTextureRegion(isWinter ? BALCONY_BLUR_WINTER : BALCONY_BLUR));
        setActorToFullScene(balconyBlur);
        balconyBlur.setVisible(false);
        getStage().addActor(balconyBlur);

        final Rectangle newspaperBounds = new Rectangle(getStage().getWidth() * PanoramaActorsLayout.NEWSPAPER_X, getStage().getWidth() * PanoramaActorsLayout.NEWSPAPER_Y, getStage().getWidth() * PanoramaActorsLayout.NEWSPAPER_WIDTH, getStage().getWidth() * PanoramaActorsLayout.NEWSPAPER_HEIGHT);

        final Image pointer = new Image(getScreenTextureRegion(GFX_PATH + "newspaper/arrow_pointer"));
        pointer.setPosition(newspaperBounds.getX() - pointer.getWidth() / 2 - PanoramaActorsLayout.NEWSPAPER_POINTER_ANIMATION_OFFSET, newspaperBounds.getHeight() - pointer.getHeight() / 2);
        pointer.addAction(Actions.repeat(RepeatAction.FOREVER, Actions.sequence(Actions.moveBy(PanoramaActorsLayout.NEWSPAPER_POINTER_ANIMATION_OFFSET, 0, PanoramaActorsLayout.NEWSPAPER_POINTER_SPEED), Actions.moveBy(-PanoramaActorsLayout.NEWSPAPER_POINTER_ANIMATION_OFFSET, 0, PanoramaActorsLayout.NEWSPAPER_POINTER_SPEED))));
        pointer.setTouchable(Touchable.disabled);
        getStage().addActor(pointer);

        balcony.addListener(new ClickListener() {
            @Override
            public void clicked(InputEvent event, float x, float y) {
                if (newspaperBounds.contains(x, y + balcony.getY())) {
                    ambient.dispose();
                    balconyBlur.addAction(Actions.fadeIn(PanoramaActorsLayout.BALCONY_FADE_IN_DURATION));
                    balconyBlur.setVisible(true);
                    pointer.remove();

                    final Sound spinSound = getSound(SFX_NEWSPAPER_SPINNER);
                    spinSound.play();

                    Image spin = new Image(getScreenTextureRegion(GFX_PATH + "newspaper/spinner"));
                    spin.setOrigin(spin.getWidth() / 2, spin.getHeight() / 2);
                    spin.setPosition(getStage().getWidth() / 2 - spin.getWidth() / 2, getSceneInnerHeight() / 2 - spin.getHeight() / 2);
                    spin.setScale(PanoramaActorsLayout.SPIN_SCALE);
                    spin.addAction(Actions.sequence(Actions.parallel(Actions.rotateBy(PanoramaActorsLayout.SPIN_ROTATION, PanoramaActorsLayout.SPIN_DURATION), Actions.scaleTo(PanoramaActorsLayout.SPIN_SCALE_TO_X, PanoramaActorsLayout.SPIN_SCALE_TO_Y, PanoramaActorsLayout.SPIN_SCALE_TO_DURATION)), Actions.parallel(Actions.sequence(Actions.fadeOut(PanoramaActorsLayout.SPIN_FADE_OUT_DURATION), Actions.hide()), Actions.run(new Runnable() {
                        @Override
                        public void run() {
                            prepareOpenAction();
                            spinSound.stop();
                        }
                    }))));
                    getStage().addActor(spin);
                }
            }
        });
    }

    /**
     * Prepare closed newspaper with hint to open
     */
    private void prepareOpenAction() {
        newspaperStages = new Stack();
        ScaleUtil.setFullInnerScene(newspaperStages, getStage());
        newspaperStages.setY(getViewportBottomY());

        final Image[] stages = new Image[NUMBER_OF_NEWSPAPER_OPEN_FRAMES];
        for (int i = 1; i <= NUMBER_OF_NEWSPAPER_OPEN_FRAMES; i++) {
            Image stage = new Image(getScreenTextureRegion(GFX_PATH + "newspaper/open" + i));
            newspaperStages.addActor(stage);
            stages[i - 1] = stage;
            stage.setVisible(false);
        }
        stages[0].setVisible(true);
        getStage().addActor(newspaperStages);

        leftFinger = ScaleUtil.createImageWidthPosition(getScreenTextureRegion(GFX_PATH + "newspaper/left_finger"), getStage().getWidth() * PanoramaActorsLayout.LEFT_FINGER_WIDTH, getStage().getWidth() * PanoramaActorsLayout.LEFT_FINGER_X - getStage().getWidth() * PanoramaActorsLayout.LEFT_FINGER_WIDTH / 2, getViewportBottomY());
        rightFinger = ScaleUtil.createImageWidthPosition(getScreenTextureRegion(GFX_PATH + "newspaper/right_finger"), getStage().getWidth() * PanoramaActorsLayout.RIGHT_FINGER_WIDTH, getStage().getWidth() * PanoramaActorsLayout.RIGHT_FINGER_X - getStage().getWidth() * PanoramaActorsLayout.RIGHT_FINGER_WIDTH / 2, getViewportBottomY());
        hint = ScaleUtil.createImageWidthPosition(getScreenTextureRegion(GFX_PATH + "newspaper/stretchhint"), getStage().getWidth() * PanoramaActorsLayout.HINT_WIDTH, getStage().getWidth() / 2 - getStage().getWidth() * PanoramaActorsLayout.HINT_WIDTH / 2, getViewportBottomY() + leftFinger.getHeight() / 2);

        prepareFingersSweepActions();

        getStage().addActor(hint);
        getStage().addActor(leftFinger);
        getStage().addActor(rightFinger);

        getStage().addListener(new InputListener() {
            private boolean dragging, mouse;
            private Sound sound;
            private float x1start, x2start, x1, x2;

            @Override
            public boolean touchDown(InputEvent event, float x, float y, int pointer, int button) {
                super.touchDown(event, x, y, pointer, button);
                // desktop mouse button workaround
                mouse = (button == Input.Buttons.RIGHT);

                if (pointer == 0) {
                    x1start = x1 = x;
                }
                if (pointer == 1) {
                    x2start = x2 = x;
                    dragging = true;
                }
                return true;
            }

            private void showStage(int i) {
                for (Image image : stages) {
                    image.setVisible(false);
                }
                stages[i].setVisible(true);
            }

            @Override
            public void touchDragged(InputEvent event, float x, float y, int pointer) {
                super.touchDragged(event, x, y, pointer);
                if (pointer == 0) {
                    x1 = x;
                } else if (pointer == 1) {
                    x2 = x;
                }

                if (getDiff() > getStage().getWidth() * PanoramaActorsLayout.NEWSPAPERS_OPEN_STAGE_5) {
                    if (!newspapersOpened) {
                        onScaled();
                        newspapersOpened = true;
                    }
                } else if (getDiff() > getStage().getWidth() * PanoramaActorsLayout.NEWSPAPERS_OPEN_STAGE_4) {
                    showStage(4);
                } else if (getDiff() > getStage().getWidth() * PanoramaActorsLayout.NEWSPAPERS_OPEN_STAGE_3) {
                    showStage(3);
                } else if (getDiff() > getStage().getWidth() * PanoramaActorsLayout.NEWSPAPERS_OPEN_STAGE_2) {
                    showStage(2);
                } else if (getDiff() > getStage().getWidth() * PanoramaActorsLayout.NEWSPAPERS_OPEN_STAGE_1) {
                    showStage(1);
                    if (sound == null) {
                        sound = getSound(SFX_NEWSPAPER_OPEN);
                        sound.play();
                    }
                } else {
                    stopSound();
                    showStage(0);
                }
            }

            private void stopSound() {
                if (sound != null) {
                    sound.stop();
                    sound = null;
                }
            }

            @Override
            public void touchUp(InputEvent event, float x, float y, int pointer, int button) {
                super.touchUp(event, x, y, pointer, button);
                showStage(0);
                stopSound();
                if (pointer == 1) {
                    dragging = false;
                }
            }

            private float getDiff() {
                if (mouse) {
                    return Math.abs(x1start - x1);
                } else if (dragging) {
                    return Math.abs(Math.abs(x1 - x2) - Math.abs(x1start - x2start));
                }
                return 0;
            }

            public void onScaled() {
                sound = null;
                if (hint != null) hint.remove();
                if (leftFinger != null) leftFinger.remove();
                if (rightFinger != null) rightFinger.remove();
                if (newspaperStages != null) newspaperStages.remove();
                //cleanup
                hint = null;
                leftFinger = null;
                rightFinger = null;
                newspaperStages = null;
                prepareOpenedNewspaper();
            }
        });
    }

    private void prepareFingersSweepActions() {
        leftFinger.clearActions();
        rightFinger.clearActions();

        leftFinger.addAction(Actions.sequence(
                Actions.fadeIn(PanoramaActorsLayout.FINGER_FADE_IN)
                , Actions.forever(Actions.sequence(
                        Actions.moveTo(getStage().getWidth() * PanoramaActorsLayout.LEFT_FINGER_MOVE_TO - leftFinger.getWidth() / 2, leftFinger.getY(), PanoramaActorsLayout.FINGER_MOVE_DURATION)
                        , Actions.scaleTo(PanoramaActorsLayout.FINGER_SCALE_TO, PanoramaActorsLayout.FINGER_SCALE_TO, PanoramaActorsLayout.FINGER_SCALE_DURATION)
                        , Actions.moveTo(-leftFinger.getWidth(), leftFinger.getY(), PanoramaActorsLayout.FINGER_MOVE_DURATION)
                        , Actions.scaleTo(PanoramaActorsLayout.FINGER_ORIGINAL_SCALE_TO, PanoramaActorsLayout.FINGER_ORIGINAL_SCALE_TO, PanoramaActorsLayout.FINGER_ORIGINAL_SCALE_DURATION)))));

        rightFinger.addAction(Actions.sequence(
                Actions.fadeIn(PanoramaActorsLayout.FINGER_FADE_IN)
                , Actions.forever(Actions.sequence(
                        Actions.moveTo(getStage().getWidth() * PanoramaActorsLayout.RIGHT_FINGER_MOVE_TO - rightFinger.getWidth() / 2, rightFinger.getY(), PanoramaActorsLayout.FINGER_MOVE_DURATION)
                        , Actions.scaleTo(PanoramaActorsLayout.FINGER_SCALE_TO, PanoramaActorsLayout.FINGER_SCALE_TO, PanoramaActorsLayout.FINGER_SCALE_DURATION)
                        , Actions.moveTo(getStage().getWidth() + rightFinger.getWidth(), rightFinger.getY(), PanoramaActorsLayout.FINGER_MOVE_DURATION)
                        , Actions.scaleTo(PanoramaActorsLayout.FINGER_ORIGINAL_SCALE_TO, PanoramaActorsLayout.FINGER_ORIGINAL_SCALE_TO, PanoramaActorsLayout.FINGER_ORIGINAL_SCALE_DURATION)))));
    }

    /**
     * Prepare opened newspaper with topic details and playback of comments
     */
    private void prepareOpenedNewspaper() {
        Image opened = new Image(getScreenTextureRegion(GFX_PATH + "newspaper/opened"));
        ScaleUtil.setFullInnerScene(opened, getStage());
        getStage().addActor(opened);

        opened.addListener(new ClickListener() {
            private final Color DETECTIVE_COLOR = Color.BLACK;
            private final Color SWIMSUIT_COLOR = Color.YELLOW;
            private final Color MOTOBIKE_COLOR = Color.BLUE;
            private final Color BEARD_COLOR = Color.RED;
            private final Color BALOON_COLOR = Color.GREEN;

            @Override
            public void clicked(InputEvent event, float x, float y) {
                super.clicked(event, x, y);

                if (newsDetailDialog != null && newsDetailDialog.isVisible()) {
                    newsDetailDialog.hide();
                }

                Color color = getTouchedColor(x, y);
                final int dialogIndex;

				final String text, music;

                if (color.equals(DETECTIVE_COLOR)) {
                    dialogIndex = 4;
                    music = MUSIC_5;
                } else if (color.equals(SWIMSUIT_COLOR)) {
                    dialogIndex = 3;
                    music = MUSIC_1;
                } else if (color.equals(MOTOBIKE_COLOR)) {
                    dialogIndex = 2;
                    music = MUSIC_2;
                } else if (color.equals(BEARD_COLOR)) {
                    dialogIndex = 0;
                    music = MUSIC_3;
                } else if (color.equals(BALOON_COLOR)) {
                    dialogIndex = 1;
                    music = MUSIC_4;
                } else {
                    return;
                }


				showNewsDetailDialog(dialogIndex);

				playMusic(music);
			}

			private Color getTouchedColor(float x, float y) {
				int clickX = (int) (x / TablexiaSettings.getSceneWidth(getStage()) * clickmap.getRegionWidth());
				int clickY = clickmap.getRegionHeight() - (int) (y / TablexiaSettings.getSceneInnerHeight(getStage()) * clickmap.getRegionHeight());

                if( clickX < 0 || clickY < 0 || clickX > getData().length || clickY > getData()[0].length)
                    return Color.WHITE;
                else
				    return new Color(getData()[clickX][clickY]);
			}
		});
    }

    public boolean clickedOnDialog(float x, float y) {
        return x > newsDetailDialog.getOutterLeftX() &&
               x < newsDetailDialog.getOutterRightX() &&
               y > newsDetailDialog.getOutterBottomY() &&
               y < newsDetailDialog.getOutterTopY();
    }

    public void showNewsDetailDialog(final int dialogIndex) {
        final ArrayList<TablexiaDialogComponentAdapter> components = new ArrayList<TablexiaDialogComponentAdapter>();
        components.add(new AdaptiveSizeDialogComponent());
        components.add(new ViewportMaximumSizeComponent());
        components.add(new DimmerDialogComponent(NEWSPAPER_DETAIL_DIALOG_DIMMER));
        components.add(new TouchCloseDialogComponent(new TouchCloseDialogComponent.TouchListener() {
            @Override
            public boolean dialogTouched(float x, float y, TablexiaComponentDialog dialog) {
                if (dialogIndex == 4 && clickedOnDialog(x, y)) {
					moveToStreet();
                }
                return true;
            }
        }));

        components.add(new FixedSpaceContentDialogComponent());
        components.add(new ImageContentDialogComponent(new Image(getScreenTextureRegion(GFX_PATH + NEWSPAPER_DETAIL + (dialogIndex + 1))), Scaling.fill, 1f, 1f));
        components.add(new FixedSpaceContentDialogComponent());
        components.add(new CenterPositionDialogComponent());

        this.newsDetailDialog = TablexiaComponentDialogFactory.getInstance().createDialog(
                TablexiaComponentDialog.TablexiaDialogType.DIALOG_RECTANGLE,
                components.toArray(new TablexiaDialogComponentAdapter[]{})
        );

		final int origNewsDetailWidth  = getScreenTextureRegion(GFX_PATH + NEWSPAPER_DETAIL + (dialogIndex + 1)).getRegionWidth();
		final int origNewsDetailHeight = getScreenTextureRegion(GFX_PATH + NEWSPAPER_DETAIL + (dialogIndex + 1)).getRegionHeight();

		this.newsDetailDialog.show(
				origNewsDetailWidth,
				origNewsDetailHeight
		);

		this.newsDetailDialog.setComponentsSizeComputedCallback(new Runnable() {
			@Override
			public void run() {
				if (newsDetailDialog.getOutterHeight() > getSceneInnerHeight() * NEWS_DIALOG_MAX_HEIGHT) {
					int currNewsDetailHeight;
					int currNewsDetailWidth;

					currNewsDetailHeight = (int) (getSceneInnerHeight() * NEWS_DIALOG_MAX_HEIGHT);
					currNewsDetailWidth = (int) (currNewsDetailHeight * ((float)(origNewsDetailWidth) / origNewsDetailHeight));

					if(currNewsDetailWidth > getSceneWidth() * NEWS_DIALOG_MAX_WIDTH) {
						currNewsDetailWidth = (int) (getSceneWidth() * NEWS_DIALOG_MAX_WIDTH);
						currNewsDetailHeight = (int) (currNewsDetailWidth * ((float)(origNewsDetailHeight) / origNewsDetailWidth));
					}

					newsDetailDialog.setSize(currNewsDetailWidth, currNewsDetailHeight);

					if (dialogIndex == 4) {
						showContinueTextDialog(getText("newuseranim_topic_" + (dialogIndex + 1)));
					} else {
						showTextDialog(getText("newuseranim_topic_" + (dialogIndex + 1)));
					}
				}
			}
		});
    }

	private void moveToStreet() {
		for (Actor a : getStage().getActors()) {
			a.setTouchable(Touchable.disabled);
			a.addAction(Actions.sequence(Actions.moveBy(-getStage().getWidth(), 0, PanoramaActorsLayout.NEWSPAPERS_STREET_TRANSITION_SPEED), Actions.removeActor()));
		}
		Actor street = prepareSwipeStreet();
		getStage().addActor(street);
		street.setX(getStage().getWidth());
		street.addAction(Actions.moveBy(-getStage().getWidth(), 0, PanoramaActorsLayout.NEWSPAPERS_STREET_TRANSITION_SPEED));
	}

	private void showTextDialog(String text) {
		ArrayList<TablexiaDialogComponentAdapter> components = new ArrayList<TablexiaDialogComponentAdapter>();

		components.add(new TouchCloseDialogComponent());
		components.add(new AdaptiveSizeDialogComponent());
		components.add(
			new SpeechDialogComponent(
				new Point(
					//4 is just a little offset to make sure that dialog appears to the top and right...
					newsDetailDialog.getInnerLeftX() + newsDetailDialog.getInnerWidth() / 2 + NEWSPAPERS_SPEECH_DIALOG_OFFSET,
					newsDetailDialog.getInnerHeight() / 2 + NEWSPAPERS_SPEECH_DIALOG_OFFSET
				),
				new Vector2(newsDetailDialog.getInnerWidth() / 2f * NEWSPAPERS_SPEECH_DIALOG_OFFSET_MULTI, newsDetailDialog.getInnerHeight() / 2f * NEWSPAPERS_SPEECH_DIALOG_OFFSET_MULTI),
				SpeechDialogComponent.ArrowType.BEND_ARROW
			)
		);

		components.add(new FixedSpaceContentDialogComponent());
		components.add(new ResizableSpaceContentDialogComponent());

		TextContentDialogComponent textComponent = new TextContentDialogComponent(text, true, true);
		textComponent.setPadding(4, 4);
		components.add(textComponent);

		components.add(new ResizableSpaceContentDialogComponent());
		components.add(new FixedSpaceContentDialogComponent());

		newsSpeechDialog = TablexiaComponentDialogFactory.getInstance().createDialog(
			TablexiaComponentDialog.TablexiaDialogType.BUBBLE_SQUARE,
			components.toArray(new TablexiaDialogComponentAdapter[]{})
		);
		newsSpeechDialog.show(PanoramaActorsLayout.NEWSPAPERS_DIALOG_WIDTH, PanoramaActorsLayout.NEWSPAPERS_BUBBLE_DIALOG_HEIGHT);
	}

	private void showContinueTextDialog(String text) {
		ArrayList<TablexiaDialogComponentAdapter> components = new ArrayList<TablexiaDialogComponentAdapter>();

		components.add(new TouchCloseDialogComponent(new TouchCloseDialogComponent.TouchListener() {
			@Override
			public boolean dialogTouched(float x, float y, TablexiaComponentDialog dialog) {
				if (x > newsSpeechDialog.getOutterLeftX() &&
						x < newsSpeechDialog.getOutterRightX() &&
						y > newsSpeechDialog.getOutterBottomY() &&
						y < newsSpeechDialog.getOutterTopY()) {
					moveToStreet();
				}
				return true;
			}
		}));
		components.add(new AdaptiveSizeDialogComponent());
		components.add(new SpeechDialogComponent(
			new Point(
				//4 is just a little offset to make sure that dialog appears to the top and right...
				newsDetailDialog.getInnerLeftX() + newsDetailDialog.getInnerWidth() / 2 + NEWSPAPERS_SPEECH_DIALOG_OFFSET,
				newsDetailDialog.getInnerHeight() / 2 + NEWSPAPERS_SPEECH_DIALOG_OFFSET
			),
			new Vector2(newsDetailDialog.getInnerWidth() / 2f * NEWSPAPERS_SPEECH_DIALOG_OFFSET_MULTI, getSceneInnerHeight() / 2f * NEWSPAPERS_SPEECH_DIALOG_OFFSET_MULTI),
			SpeechDialogComponent.ArrowType.CLASSIC_SHADOW));

		TablexiaDialogComponentAdapter tdca = new TablexiaDialogComponentAdapter() {
			private static final float ADAPTIVE_SCALE = 1.7f;

			TextureRegion buttonTexture = ApplicationAtlasManager.getInstance().getTextureRegion(ApplicationAtlasManager.DIALOG_CLASSIC_BUTTON_SHADOW);
			float currentWidth = buttonTexture.getRegionWidth();
			float currentHeight = buttonTexture.getRegionHeight();

			@Override
			public void drawAdditions(Batch batch, float parentAlpha) {
				batch.draw(
					buttonTexture,
					getDialog().getInnerRightX() - currentWidth / 2,
					getDialog().getInnerBottomY() + getDialog().getInnerHeight() - currentHeight,
					currentWidth,
					currentHeight
				);
			}

			@Override
			public void adaptiveSizeThresholdChanged() {
				currentWidth = buttonTexture.getRegionWidth();
				currentHeight = buttonTexture.getRegionHeight();

				if (ComponentScaleUtil.isUnderThreshold()) {
					currentWidth  *= ADAPTIVE_SCALE;
					currentHeight *= ADAPTIVE_SCALE;
				}
			}
		};
		components.add(tdca);

		components.add(new FixedSpaceContentDialogComponent());
		components.add(new ResizableSpaceContentDialogComponent());

		TextContentDialogComponent textComponent = new TextContentDialogComponent(text, true, true);
		textComponent.setPadding(OFFICE_DIALOG_TEXT_PADDING_LEFT, OFFICE_DIALOG_TEXT_PADDING_RIGHT);
		components.add(textComponent);

		components.add(new ResizableSpaceContentDialogComponent());
		components.add(new FixedSpaceContentDialogComponent());

		newsSpeechDialog = TablexiaComponentDialogFactory.getInstance().createDialog(
				TablexiaComponentDialog.TablexiaDialogType.DIALOG_SQUARE_SHADOW,
				components.toArray(new TablexiaDialogComponentAdapter[]{}));
		newsSpeechDialog.show(PanoramaActorsLayout.NEWSPAPERS_DIALOG_WIDTH, PanoramaActorsLayout.NEWSPAPERS_BUBBLE_DIALOG_HEIGHT);
	}

    private Music playingMusic;

    private void playMusic(String key) {
        stopMusic();
        playingMusic = getMusic(key);
        playingMusic.play();
    }

    private void stopMusic() {
        if (playingMusic != null && playingMusic.isPlaying()) {
            playingMusic.stop();
            playingMusic.dispose();
        }
    }


    /**
     * Prepare street view with swipe and door knock
     */
    private Actor prepareSwipeStreet() {
        ambient = getMusic(AMBIENT_STREET);
        ambient.setVolume(AMBIENT_SOUND_VOLUME);
        ambient.setLooping(true);
        ambient.play();

        panel = new Group();

        ScaleUtil.setFullScreen(panel, getStage());
        panel.setY(getViewportBottomY());
        panel.addActor(prepareStreetGroup());
        panel.setCullingArea(new Rectangle(panel.getX(), panel.getY(), panel.getWidth(), panel.getHeight()));
        prepareSwipeStreetFingerHint();

        return panel;
    }

    /**
     * Prepare detective office with dialogs and playing voice
     */
    private Actor prepareOffice() {
        ambient.dispose();
        Group group = new Group();
        Image background = new TablexiaNoBlendingImage(getScreenTextureRegion(GFX_PATH + "detectiveoffice"));
        setActorToFullScene(background);
        group.addActor(background);
        ScaleUtil.setFullScene(group, getStage());
        return group;
    }

    private void showDialog(final int num) {
        String detectiveText = getText("createuser_detective_" + num);

        ArrayList<TablexiaDialogComponentAdapter> components = new ArrayList<TablexiaDialogComponentAdapter>();

        components.add(new TouchCloseDialogComponent(new TouchCloseDialogComponent.TouchListener() {
            @Override
            public boolean dialogTouched(float x, float y, TablexiaComponentDialog dialog) {
                stopMusic();

                if (num == NUMBER_OF_OFFICE_DIALOGS) {
                    ApplicationBus.getInstance().publishAsync(new Tablexia.ChangeScreenEvent(FormScreen.class, TablexiaApplication.ScreenTransaction.FADE));
                } else {
                    showDialog(num + 1);
                }
                return true;
            }
        }));
        components.add(new AdaptiveSizeDialogComponent());
        components.add(new SpeechDialogComponent(
                new Point(getStage().getWidth() * PanoramaActorsLayout.OFFICE_DIALOG_X, getStage().getWidth() * PanoramaActorsLayout.OFFICE_DIALOG_Y),
                new Vector2(OFFICE_DIALOG_OFFSET),
                SpeechDialogComponent.ArrowType.CLASSIC_SHADOW));

        TablexiaDialogComponentAdapter tdca = new TablexiaDialogComponentAdapter() {
            private static final float ADAPTIVE_SCALE = 1.7f;

            TextureRegion buttonTexture = ApplicationAtlasManager.getInstance().getTextureRegion(ApplicationAtlasManager.DIALOG_CLASSIC_BUTTON_SHADOW);
            float currentWidth = buttonTexture.getRegionWidth();
            float currentHeight = buttonTexture.getRegionHeight();

            @Override
            public void drawAdditions(Batch batch, float parentAlpha) {
                batch.draw(
                    buttonTexture,
                    getDialog().getInnerRightX() - currentWidth/2,
                    getDialog().getInnerBottomY() + getDialog().getInnerHeight() - currentHeight,
                    currentWidth,
                    currentHeight
                );
            }

            @Override
            public void adaptiveSizeThresholdChanged() {
                currentWidth = buttonTexture.getRegionWidth();
                currentHeight = buttonTexture.getRegionHeight();

                if(ComponentScaleUtil.isUnderThreshold()) {
                    currentWidth  *= ADAPTIVE_SCALE;
                    currentHeight *= ADAPTIVE_SCALE;
                }
            }
        };
        components.add(tdca);

        components.add(new FixedSpaceContentDialogComponent());
        components.add(new ResizableSpaceContentDialogComponent());

        TextContentDialogComponent textComponent = new TextContentDialogComponent(detectiveText, true, true);
        textComponent.setPadding(OFFICE_DIALOG_TEXT_PADDING_LEFT, OFFICE_DIALOG_TEXT_PADDING_RIGHT);
        components.add(textComponent);

        components.add(new ResizableSpaceContentDialogComponent());
        components.add(new FixedSpaceContentDialogComponent());

        TablexiaComponentDialogFactory.getInstance().createDialog(
                getStage(),
                TablexiaComponentDialog.TablexiaDialogType.DIALOG_SQUARE_SHADOW,
                components.toArray(new TablexiaDialogComponentAdapter[]{})).show(getSceneWidth() * PanoramaActorsLayout.OFFICE_DIALOG_WIDTH, getSceneWidth() * PanoramaActorsLayout.OFFICE_DIALOG_HEIGHT, false);
        playMusic(MFX_PATH + "detective/" + num + ".mp3");
    }

    private Group prepareStreetGroup() {
        Table group = new Table();
        group.setBounds(getSceneLeftX(), getSceneOuterBottomY(), getSceneWidth(), getSceneOuterHeight());

        TablexiaNoBlendingImage tile00 = new TablexiaNoBlendingImage(getScreenTextureRegion(GFX_PATH + "swipe/tile0"));
        TablexiaNoBlendingImage tile01 = new TablexiaNoBlendingImage(getScreenTextureRegion(GFX_PATH + "swipe/tile1"));

        tile00.setSize((int) (panel.getHeight() * (tile00.getWidth() / tile00.getHeight())), panel.getHeight());
        tile01.setSize((int) (panel.getHeight() * (tile01.getWidth() / tile01.getHeight())), panel.getHeight());

        group.add(tile00).height(tile00.getHeight()).width(tile00.getWidth());
        group.add(tile01).height(tile01.getHeight()).width(tile01.getWidth());
        //tile2
        Group tile2 = new Group();
        TablexiaNoBlendingImage tile02 = new TablexiaNoBlendingImage(getScreenTextureRegion(GFX_PATH + "swipe/tile2"));
        tile02.setSize((int) (panel.getHeight() * (tile02.getWidth() / tile02.getHeight())), panel.getHeight());
        tile2.addActor(tile02);
        Image tile02_title = ScaleUtil.createImageWidthPosition(
                getScreenTextureRegion(GFX_PATH + "swipe/tile2_title"), 
                tile02.getWidth() * PanoramaActorsLayout.STREET_TILE2_TITLE_IMAGE_WIDTH,
                SIGN_POSITION.x*tile02.getWidth(),
                SIGN_POSITION.y*tile02.getHeight());

        tile2.addActor(tile02_title);
        group.add(tile2).width(tile02.getWidth()).height(tile02.getHeight());
        //tile3
        final TablexiaNoBlendingImage tile3Background = new TablexiaNoBlendingImage(getScreenTextureRegion(GFX_PATH + "swipe/tile3"));
        tile3Background.setSize((int) (panel.getHeight() * (tile3Background.getWidth() / tile3Background.getHeight())), panel.getHeight());
        final Image tile3DoorSign = ScaleUtil.createImageHeightPosition(getScreenTextureRegion(GFX_PATH + "swipe/tile3_door_sign"),
                tile3Background.getHeight() * PanoramaActorsLayout.STREET_TILE3_DOOR_SIGN_IMAGE_HEIGHT,
                DOOR_SIGN_POSITION.x * tile3Background.getWidth(),
                DOOR_SIGN_POSITION.y * tile3Background.getHeight());
        Image tile3PlateTitle = ScaleUtil.createImageWidthPosition(
                getScreenTextureRegion(GFX_PATH + "swipe/tile3_plate_title"),
                tile3Background.getWidth() * PanoramaActorsLayout.STREET_TILE3_PLATE_TITLE_IMAGE_WIDTH,
                PLATE_TITLE_POSITION.x * tile3Background.getWidth(),
                PLATE_TITLE_POSITION.y * tile3Background.getHeight());
        //Detective door group
        final Group detectiveDoor = new Group();

        tile3DoorSign.addListener(new ClickListener() {
            private int knockCount = 0;

            @Override
            public void clicked(InputEvent event, float x, float y) {
                super.clicked(event, x, y);
                if (knockCount < NUMBER_OF_OFFICE_KNOCKS) {
                    knockCount++;
                    Image knock = ScaleUtil.createImageWidthPosition(getScreenTextureRegion(GFX_PATH + "swipe/knock"), getStage().getWidth() * PanoramaActorsLayout.STREET_KNOCK_IMAGE_WIDTH, tile3DoorSign.getX() + x, tile3DoorSign.getY() + y);
                    detectiveDoor.addActor(knock);

                    RunnableAction moveToOffice = new RunnableAction();
                    moveToOffice.setRunnable(new Runnable() {
                        @Override
                        public void run() {
                            Actor office = prepareOffice();
                            office.setPosition(getStage().getWidth(), 0);
                            getStage().addActor(office);

                            panel.addAction(Actions.sequence(Actions.moveBy(-getStage().getWidth(), 0, PanoramaActorsLayout.STREET_OFFICE_TRANSITION_DURATION), Actions.removeActor()));
                            final RunnableAction showDialog = new RunnableAction();
                            showDialog.setRunnable(new Runnable() {
                                @Override
                                public void run() {
                                    showDialog(1);
                                }
                            });
                            office.addAction(Actions.sequence(Actions.moveTo(0, 0, PanoramaActorsLayout.STREET_OFFICE_TRANSITION_DURATION), Actions.delay(PanoramaActorsLayout.OFFICE_FIRST_DIALOG_DELAY), showDialog));
                        }
                    });

                    knock.setTouchable(Touchable.disabled);
                    getSound("sfx/tuk_" + knockCount + ".mp3").play();
                    SequenceAction knockActions;
                    if (knockCount != NUMBER_OF_OFFICE_KNOCKS) {
                        knockActions = Actions.sequence(Actions.delay(PanoramaActorsLayout.KNOCK_ACTION_DELAY), Actions.fadeOut(PanoramaActorsLayout.KNOCK_FADE_OUT), Actions.removeActor());
                    } else {
                        knockActions = Actions.sequence(Actions.delay(PanoramaActorsLayout.KNOCK_ACTION_DELAY), Actions.fadeOut(PanoramaActorsLayout.KNOCK_FADE_OUT), moveToOffice, Actions.removeActor());
                    }
                    knock.addAction(knockActions);
                }
            }
        });

        detectiveDoor.addActor(tile3Background);
        detectiveDoor.addActor(tile3DoorSign);
        detectiveDoor.addActor(tile3PlateTitle);
        detectiveDoor.setSize(tile3Background.getWidth(), tile3Background.getHeight());
        group.add(detectiveDoor);

        TablexiaNoBlendingImage leftBorder = new TablexiaNoBlendingImage(ApplicationAtlasManager.getInstance().getColorTexture(SWIPE_OVERSCROLL_LEFT_COLOR));
        leftBorder.setSize(OVERSCROLL_DISTANCE, getSceneInnerHeight());

        TablexiaNoBlendingImage rightBorder = new TablexiaNoBlendingImage(getScreenTextureRegion(SWIPE_OVERSCROLL_RIGHT));
        rightBorder.setHeight(getSceneInnerHeight());

        scrollpane = new ScrollPaneWithBorders(group, leftBorder, rightBorder);
        scrollpane.setFillParent(true);
        scrollpane.setScrollingDisabled(false, true);
        scrollpane.setupOverscroll(OVERSCROLL_DISTANCE, OVERSCROLL_MIN_SPEED, OVERSCROLL_MAX_SPEED);
        return scrollpane;
    }

    private void prepareSwipeStreetFingerHint() {
        hintFinger = ScaleUtil.createImageWidthPosition(getScreenTextureRegion(GFX_PATH + "newspaper/left_finger"), getStage().getWidth() * PanoramaActorsLayout.STREET_HINT_FINGER_WIDTH, getStage().getWidth() * PanoramaActorsLayout.STREET_HINT_FINGER_X - getStage().getWidth() * PanoramaActorsLayout.STREET_HINT_FINGER_WIDTH / 2, 0);
        hintFinger.setVisible(false);
        hintFinger.setTouchable(Touchable.disabled);
        panel.addActor(hintFinger);
        scrollpane.addListener(new EventListener() { // animate finger after 3s of inactivity and only if scrolled to less than 0.7 of maxX

            @Override
            public boolean handle(Event event) {
                return handleHintFinger();
            }
        });

        handleHintFinger();
    }

    private boolean handleHintFinger() {
        if (scrollpane.getScrollX() != lastScroll) {
            final Action oldAction = lastHintFingerAction;
            hintFinger.addAction(Actions.sequence(Actions.fadeOut(PanoramaActorsLayout.FINGER_FADE_OUT), Actions.run(new Runnable() {
                @Override
                public void run() {
                    hintFinger.removeAction(oldAction);
                    hintFinger.setScale(PanoramaActorsLayout.FINGER_ORIGINAL_SCALE_TO);
                    hintFinger.setPosition(getStage().getWidth() * PanoramaActorsLayout.STREET_HINT_FINGER_X - getStage().getWidth() * PanoramaActorsLayout.STREET_HINT_FINGER_WIDTH / 2, 0);
                }
            })));
            //ScrollPanes getScrollPercentX method returns NaN if scroll pane wasn't touched yet...
            float percentScrollX = Float.isNaN(scrollpane.getScrollPercentX()) ? 0 : scrollpane.getScrollPercentX();
            if (percentScrollX < PanoramaActorsLayout.STREET_HINT_END) {
                hintFinger.addAction(lastHintFingerAction = Actions.sequence(
                        Actions.delay(PanoramaActorsLayout.STREET_HINT_DELAY)
                        , Actions.alpha(0)
                        , Actions.visible(true)
                        , Actions.fadeIn(PanoramaActorsLayout.FINGER_FADE_IN)
                        , Actions.forever(Actions.sequence(
                                Actions.moveTo(getStage().getWidth() * PanoramaActorsLayout.STREET_HINT_MOVE_TO - hintFinger.getWidth() / 2, hintFinger.getY(), PanoramaActorsLayout.FINGER_MOVE_DURATION)
                                , Actions.scaleTo(PanoramaActorsLayout.FINGER_SCALE_TO, PanoramaActorsLayout.FINGER_SCALE_TO, PanoramaActorsLayout.FINGER_SCALE_DURATION)
                                , Actions.moveTo(-hintFinger.getWidth(), hintFinger.getY(), PanoramaActorsLayout.FINGER_MOVE_DURATION)
                                , Actions.scaleTo(PanoramaActorsLayout.FINGER_ORIGINAL_SCALE_TO, PanoramaActorsLayout.FINGER_ORIGINAL_SCALE_TO, PanoramaActorsLayout.FINGER_ORIGINAL_SCALE_DURATION)))));
            }
        }
        lastScroll = scrollpane.getScrollX();
        return false;
    }

    protected void screenResized(int width, int height) {
        if (panel != null && scrollpane != null) {
            ScaleUtil.setFullScreen(panel, getStage());
            panel.setCullingArea(new Rectangle(panel.getX(), panel.getY(), panel.getWidth(), panel.getHeight()));
            float scrollX = scrollpane.getScrollPercentX();
            panel.setY(getViewportBottomY());
            panel.clear();
            panel.addActor(prepareStreetGroup());
            prepareSwipeStreetFingerHint();
            scrollpane.pack();
            scrollpane.setScrollPercentX(scrollX);
            scrollpane.updateVisualScroll();
        }

        if (leftFinger != null && rightFinger != null && hint != null && newspaperStages != null) {
            leftFinger.setY(getViewportBottomY());
            rightFinger.setY(getViewportBottomY());
            hint.setY(getViewportBottomY() + leftFinger.getHeight() / 2);
            newspaperStages.setY(getViewportBottomY());
            prepareFingersSweepActions();
        }
    }

    @Override
    public void backButtonPressed() {
        ApplicationBus.getInstance().post(new Tablexia.ChangeScreenEvent(TablexiaSettings.LOADER_SCREEN, TablexiaApplication.ScreenTransaction.FADE)).asynchronously();
    }

}
