package cz.nic.tablexia.screen.encyclopedia;

import com.badlogic.gdx.audio.Music;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.scenes.scene2d.utils.TextureRegionDrawable;
import com.badlogic.gdx.utils.Align;
import com.badlogic.gdx.utils.viewport.ExtendViewport;
import com.badlogic.gdx.utils.viewport.Viewport;

import java.util.List;
import java.util.Map;

import cz.nic.tablexia.TablexiaSettings;
import cz.nic.tablexia.screen.AbstractTablexiaScreen;
import cz.nic.tablexia.screen.encyclopedia.assets.EncyclopediaAssets;
import cz.nic.tablexia.screen.encyclopedia.content.ContentParser;
import cz.nic.tablexia.screen.encyclopedia.content.model.Content;
import cz.nic.tablexia.screen.encyclopedia.menu.MenuItem;
import cz.nic.tablexia.screen.encyclopedia.menu.MenuWidget;
import cz.nic.tablexia.screen.encyclopedia.widget.ResizableScrollPane;
import cz.nic.tablexia.screen.encyclopedia.widget.ResizableWidgetGroup;
import cz.nic.tablexia.util.Log;
import cz.nic.tablexia.util.MusicUtil;
import cz.nic.tablexia.util.Point;
import cz.nic.tablexia.util.ui.button.ImageTablexiaButton;
import cz.nic.tablexia.util.ui.button.StandardTablexiaButton;
import cz.nic.tablexia.util.ui.button.TablexiaButton;

public class EncyclopediaScreen extends AbstractTablexiaScreen<Void> {

    private static final String SOURCE_FILE_EXTENSION           = ".json";
    private static final float  FONT_BUTTON_WIDTH               = 70;
    private static final float  FONT_BUTTON_ADAPTIVE_SIZE_RATIO = 1.3f;
    private static final float  BUTTONS_MARGIN_LEFT             = 30;
    private static final float  BUTTONS_GAP                     = 20;
    private static final float  CONTENT_RELATIVE_WIDTH          = 0.70f;
    private static final int    CONTENT_PAD_TOP                 = 30;
    private static final float  MENU_RELATIVE_HEIGHT            = 0.80f;
    private static final int    CONTENT_PAD_LEFT                = 120;

    private static final int RIGHT_BUTTONS_PAD_BOTTOM = 10;

    private static final int BUTTON_MINUS = 0;
    private static final int BUTTON_PLUS  = 1;

    private static final float MUSIC_FADE_OUT_SHORT_DURATION = 0.5f;

    private static final Color  BACKGROUND_COLOR                = TablexiaSettings.getDefaultBackgroundColor();
    private static final String FONT_PLUS_TEXT                  = "+A";
    private static final String FONT_MINUS_TEXT                 = "-A";
    private static final String MUSIC_EXTENSION                 = ".mp3";
    private static final String MENU_STATE_SCROLL_POSITION_Y    = "menuScrollPositionY";
    private static final String CONTENT_STATE_SCROLL_POSITION_Y = "contentScrollPositionY";
    private static final String MENU_SELECTED_ITEM              = "menuSelectedItem";
    private static final int    DEFAULT_SELECTED_MENU_ITEM      = 0;
    public static final String  ENCYCLOPEDIA_SPEECH             = MFX_PATH + "encyclopedia.mp3";

    private static final boolean SHOW_BOUNDING_BOXES = TablexiaSettings.getInstance().isShowBoundingBoxes();

    private Table                       rightPanelStack;
    private Table                       container;
    private Table                       contentStack;
    private Table                       content;
    private ScrollPane                  contentScrollPane;
    private Table                       rightPanel;
    private MenuWidget                  menu;
    private ScrollPane                  menuScrollPane;
    private Group                       buttonsPanel;
    private Music                       playedMusic;
    private ImageTablexiaButton			musicButton;
    private ResizeFontListener          resizeFontListener;
    private static EncyclopediaFont     actualRegularFont = EncyclopediaFont.DEFAULT_REGULAR;
    private static EncyclopediaFont     actualBoldFont    = EncyclopediaFont.DEFAULT_BOLD;
    private static EncyclopediaFont     actualHeaderFont  = EncyclopediaFont.DEFAULT_HEADER;

    public Music getPlayedMusic() {
        return playedMusic;
    }

    @Override
    public boolean hasSoftBackButton() {
        return true;
    }

    @Override
    protected Viewport createViewport() {
        return new ExtendViewport(TablexiaSettings.getWorldSize(), TablexiaSettings.getMinWorldHeight());
    }

    @Override
    protected void screenPaused(Map<String, String> screenState) {
        if (menuScrollPane.getScrollPercentY() != Float.NaN) {
            screenState.put(MENU_STATE_SCROLL_POSITION_Y, String.valueOf(menuScrollPane.getScrollPercentY()));
        }
        if (contentScrollPane.getScrollPercentY() != Float.NaN) {
            screenState.put(CONTENT_STATE_SCROLL_POSITION_Y, String.valueOf(contentScrollPane.getScrollPercentY()));
        }
        screenState.put(MENU_SELECTED_ITEM, String.valueOf(menu.getSelectedItemIndex()));

        Log.debug(this.getClass(), "SCREEN ----- SAVING -------");
        for (Map.Entry<String, String> entry : screenState.entrySet()) {
            Log.debug(this.getClass(), entry.getKey() + ": " + entry.getValue());
        }
        Log.debug(this.getClass(), "content.size: " + contentScrollPane.getMaxY());
    }

    @Override
    protected void screenLoaded(Map<String, String> screenState) {
        resizeFontListener = new ResizeFontListener();
        container = new Table();
        container.setDebug(SHOW_BOUNDING_BOXES, SHOW_BOUNDING_BOXES);
        container.setBackground(new TextureRegionDrawable(getColorTextureRegion(BACKGROUND_COLOR)));
        getStage().addActor(container);
        container.setFillParent(true);

        prepareContentLayout();
        prepareRightMenu();

        int selectedItem = DEFAULT_SELECTED_MENU_ITEM;
        if (screenState.containsKey(MENU_SELECTED_ITEM)) {
            selectedItem = Integer.valueOf(screenState.get(MENU_SELECTED_ITEM));
        }
        menu.selectItem(selectedItem);
        prepareButtons();

        if (screenState.containsKey(MENU_STATE_SCROLL_POSITION_Y)) {
            menuScrollPane.pack();
            menuScrollPane.setScrollPercentY(Float.valueOf(screenState.get(MENU_STATE_SCROLL_POSITION_Y)));
            menuScrollPane.updateVisualScroll();
        }

        if (screenState.containsKey(CONTENT_STATE_SCROLL_POSITION_Y)) {
            contentScrollPane.pack();
            contentScrollPane.setScrollPercentY(Float.valueOf(screenState.get(CONTENT_STATE_SCROLL_POSITION_Y)));
            contentScrollPane.updateVisualScroll();
        }

        contentStack.pack();
    }

    @Override
    protected void screenResized(int width, int height) {
        float contentWidth = CONTENT_RELATIVE_WIDTH * getViewportWidth();
        contentStack.setSize(contentWidth, getViewportHeight());
        contentScrollPane.setSize(contentWidth, getViewportHeight());
        float rightPanelWidth = (1 - CONTENT_RELATIVE_WIDTH) * getViewportWidth();
        rightPanelStack.setSize(rightPanelWidth, getViewportHeight());
        menuScrollPane.setSize(rightPanelWidth, getViewportHeight() * MENU_RELATIVE_HEIGHT);
        buttonsPanel.setSize(rightPanelWidth, getViewportHeight() * (1 - MENU_RELATIVE_HEIGHT));
        rightPanel.setSize(rightPanelWidth, getViewportHeight());
    }

    @Override
    protected String prepareIntroMusicPath() {
        return ENCYCLOPEDIA_SPEECH;
    }

    private void prepareContentLayout() {
        //TODO add culling
        contentStack = new Table();
        contentStack.setDebug(SHOW_BOUNDING_BOXES, SHOW_BOUNDING_BOXES);
        contentStack.setSize(CONTENT_RELATIVE_WIDTH * getViewportWidth(), getViewportHeight());
        content = new Table();
        content.padLeft(CONTENT_PAD_LEFT);
        content.align(Align.top);
        contentScrollPane = new ResizableScrollPane(content);
        contentScrollPane.setScrollingDisabled(true, false);
        contentScrollPane.setSize(CONTENT_RELATIVE_WIDTH * getViewportWidth(), getViewportHeight());

        contentStack.add(contentScrollPane).padTop(CONTENT_PAD_TOP);
        container.add(contentStack);
    }

    private void prepareRightMenu() {
        float rightPanelWidth = (1 - CONTENT_RELATIVE_WIDTH) * getSceneWidth();
        menu = new MenuWidget(this, new Point(rightPanelWidth, getSceneInnerHeight() * MENU_RELATIVE_HEIGHT), EncyclopediaAssets.BOOKMARK, new MenuWidget.OnMenuItemClickListener() {
            @Override
            public void onMenuItemClicked(MenuItem clickedMenuItem) {
                showPage(clickedMenuItem.getResourcePageName());
                updateMenuScrollPosition(clickedMenuItem.ordinal() + 1);
                disposeMusic();
            }
        });
        menuScrollPane = new ResizableScrollPane(menu);
        menuScrollPane.setScrollingDisabled(true, false);

        menuScrollPane.setSize(rightPanelWidth, getViewportHeight() * MENU_RELATIVE_HEIGHT);

        buttonsPanel = new ResizableWidgetGroup();
        buttonsPanel.setSize(rightPanelWidth, getViewportHeight() * (1 - MENU_RELATIVE_HEIGHT));

        rightPanelStack = new Table();

        rightPanel = new Table();
        rightPanel.setPosition(getViewportWidth() - rightPanelWidth, getViewportHeight());
        rightPanel.setSize(rightPanelWidth, getViewportHeight());
        rightPanel.add(menuScrollPane);
        rightPanel.row();
        rightPanel.add(buttonsPanel);
        rightPanelStack.add(rightPanel).padTop(CONTENT_PAD_TOP);

        container.add(rightPanelStack);

        menu.updateSize(rightPanelWidth);
    }

    private void prepareButtons() {
        // TODO: 1.12.15 add font resizing 
        TablexiaButton btnFontPlus = new StandardTablexiaButton(FONT_PLUS_TEXT, StandardTablexiaButton.TablexiaButtonType.BLUE)
                .setButtonBounds(BUTTONS_MARGIN_LEFT, RIGHT_BUTTONS_PAD_BOTTOM, FONT_BUTTON_WIDTH, FONT_BUTTON_WIDTH)
                .adaptiveSizeRatio(FONT_BUTTON_ADAPTIVE_SIZE_RATIO)
                .setInputListener(resizeFontListener);

        TablexiaButton btnFontMinus = new StandardTablexiaButton(FONT_MINUS_TEXT, StandardTablexiaButton.TablexiaButtonType.BLUE)
                .setButtonBounds(btnFontPlus.getX() + btnFontPlus.getWidth() + BUTTONS_GAP, RIGHT_BUTTONS_PAD_BOTTOM, FONT_BUTTON_WIDTH, FONT_BUTTON_WIDTH)
                .adaptiveSizeRatio(FONT_BUTTON_ADAPTIVE_SIZE_RATIO)
                .setInputListener(resizeFontListener);

        btnFontMinus.setUserObject(BUTTON_MINUS);
        btnFontPlus.setUserObject(BUTTON_PLUS);

        buttonsPanel.addActor(btnFontPlus);
        buttonsPanel.addActor(btnFontMinus);
    }

    private void showPage(String pageResourceName) {
        if (content.hasChildren()) {
            content.clearChildren();
        }
        FileHandle fileHandle = getFile(pageResourceName + SOURCE_FILE_EXTENSION);
        List<Content> contentList = ContentParser.parseEncyclopediaPageContent(fileHandle.readString());

        for (Content c : contentList) {
            c.addToCell(content, this, (CONTENT_RELATIVE_WIDTH * getViewportWidth()) - CONTENT_PAD_LEFT);
            content.row();
        }
    }

    public void playMusic(String music, final ImageTablexiaButton musicButton) {
        if (this.musicButton != null && this.musicButton != musicButton){
			resetMusicButtonState();
        }
        this.musicButton = musicButton;
        Music m = getMusic(EncyclopediaAssets.SFX_PATH + music + MUSIC_EXTENSION);
        m.setOnCompletionListener(new Music.OnCompletionListener() {
            @Override
            public void onCompletion(Music music) {
                musicButton.text(getText(EncyclopediaAssets.PLAY));
                musicButton.setUnchecked();
				musicButton.setIcon(getPlayIcon());
            }
        });
        m.play();
        if (m.isPlaying()) Log.debug(getClass(), "Playing");
        playedMusic = m;
    }

    public void togglePlaying() {
        if (playedMusic.isPlaying()) {
            musicButton.text(getText(EncyclopediaAssets.PLAY));
			musicButton.setIcon(getPlayIcon());
            playedMusic.pause();
        } else {
            // TODO: 5.1.16 fix button text 
            musicButton.text(getText(EncyclopediaAssets.PAUSE));
            musicButton.setIcon(getPauseIcon());
            playedMusic.play();
        }
    }

    private void resetMusicButtonState() {
        disposeMusic();
        musicButton.setUnchecked();
        musicButton.text(getText(EncyclopediaAssets.PLAY));
        musicButton.setIcon(getPlayIcon());
    }

    public void disposeMusic() {
        Music introMusic = getIntroMusic();
        if (introMusic != null && introMusic.isPlaying()) {
            MusicUtil.fadeOut(introMusic, MUSIC_FADE_OUT_DURATION);
        }

        if (playedMusic != null ) {
            if (playedMusic.isPlaying()) {
                MusicUtil.fadeOut(playedMusic,MUSIC_FADE_OUT_SHORT_DURATION,true);
            }else {
                playedMusic.dispose();
            }
            musicButton.setUnchecked();
            playedMusic = null;
        }
    }

    public void updateContentSize() {
        List<Content> contentList = ContentParser.getContentList();
        for (Content c : contentList) {
            c.refreshContent(this);
        }
        content.pack();
    }

    private void updateMenuScrollPosition(int index){
        int thirdPart = MenuItem.values().length/3;
        float position;
        if (index< thirdPart){
            position = 0f;
        }else if (index< thirdPart *2){
            position = 0.5f;
        }else {
            position = 1f;
        }
        menuScrollPane.setScrollPercentY(position);
    }

    public static EncyclopediaFont getActualRegularFont() {
        return actualRegularFont;
    }
    public static EncyclopediaFont getActualBoldFont() {
        return actualBoldFont;
    }

    public static EncyclopediaFont getHeaderActualFont() {
        return actualHeaderFont;
    }

    public void decreaseFontSize() {
        actualRegularFont = actualRegularFont.getSmaller();
        actualBoldFont = actualBoldFont.getSmaller();
        actualHeaderFont = actualHeaderFont.getSmaller();
    }

    public void increaseFontSize() {
        actualRegularFont = actualRegularFont.getLarger();
        actualBoldFont = actualBoldFont.getLarger();
        actualHeaderFont = actualHeaderFont.getLarger();
    }

	public Image getPauseIcon() {
		return new Image(getScreenTextureRegion(EncyclopediaAssets.PAUSE_ICON));
	}

	public Image getPlayIcon() {
		return new Image(getScreenTextureRegion(EncyclopediaAssets.PLAY_ICON));
	}

	private class ResizeFontListener extends ClickListener {

        @Override
        public void clicked(InputEvent event, float x, float y) {
            super.clicked(event, x, y);
            if (event.getListenerActor().getUserObject().equals(BUTTON_PLUS)) {
                increaseFontSize();
            } else {
                decreaseFontSize();
            }
            updateContentSize();
        }

    }

    @Override
    protected void screenDisposed() {
        if (playedMusic != null) {
            playedMusic.dispose();

            playedMusic = null;
        }
        super.screenDisposed();
    }
}
