package cz.nic.tablexia.screen.encyclopedia.content.model;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.utils.JsonValue;

import cz.nic.tablexia.TablexiaSettings;
import cz.nic.tablexia.screen.AbstractTablexiaScreen;
import cz.nic.tablexia.screen.encyclopedia.EncyclopediaFont;
import cz.nic.tablexia.screen.encyclopedia.EncyclopediaScreen;
import cz.nic.tablexia.util.ui.TablexiaLabel;

/**
 * Created by Václav Tarantík on 3.9.15.
 */
public class TextContent extends Content {

    private static final String JSON_TEXT_KEY      = "text";
    private static final String JSON_BOLD_KEY      = "bold";
    private static final Color  DEFAULT_FONT_COLOR = TablexiaSettings.getDefaultFontColor();

    private String  text;
    private boolean bold;
    private TablexiaLabel paragraph;

    public TextContent(JsonValue jsonValue) {
        checkJsonContent(jsonValue, JSON_TEXT_KEY);
        text = jsonValue.getString(JSON_TEXT_KEY);
        bold = jsonValue.has(JSON_BOLD_KEY) ? jsonValue.getBoolean(JSON_BOLD_KEY) : false;
    }

    @Override
    public Actor render(AbstractTablexiaScreen screen, float parentContainerWidth) {
        EncyclopediaFont fontType;
        if (isBold()){
            fontType = ((EncyclopediaScreen)screen).getActualBoldFont();
        }else{
            fontType = ((EncyclopediaScreen)screen).getActualRegularFont();
        }
        return createParagraph(fontType,screen.getText(getText()));
    }

    @Override
    void update(AbstractTablexiaScreen screen) {
        EncyclopediaFont fontType;
        if (isBold()){
            fontType = ((EncyclopediaScreen)screen).getActualBoldFont();
        }else{
            fontType = ((EncyclopediaScreen)screen).getActualRegularFont();
        }
        paragraph.setTablexiaLabelStyle(new TablexiaLabel.TablexiaLabelStyle(fontType.getFontType(), DEFAULT_FONT_COLOR));
    }

    private TablexiaLabel createParagraph(EncyclopediaFont fontType, String text){
        TablexiaLabel.TablexiaLabelStyle labelStyle = new TablexiaLabel.TablexiaLabelStyle(fontType.getFontType(), DEFAULT_FONT_COLOR);
        paragraph = new TablexiaLabel(text, labelStyle);
        paragraph.setWrap(true);

        return paragraph;
    }

    public String getText() {
        return text;
    }

    public boolean isBold() {
        return bold;
    }
}
