package cz.nic.tablexia.screen.encyclopedia.menu;

import com.badlogic.gdx.graphics.Pixmap;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.NinePatch;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.ui.VerticalGroup;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.scenes.scene2d.utils.NinePatchDrawable;
import com.badlogic.gdx.utils.Align;
import com.badlogic.gdx.utils.Array;

import java.util.HashMap;
import java.util.Map;

import cz.nic.tablexia.screen.AbstractTablexiaScreen;
import cz.nic.tablexia.util.Point;
import cz.nic.tablexia.util.ui.button.TablexiaButton;

/**
 * Created by Václav Tarantík on 26.8.15.
 */
public class MenuWidget extends VerticalGroup {
    private static final float BUTTON_TOP_BOTTOM_PADDING    =   20f;

    Array<MenuItem> menuItemValues;
    private Map<MenuItem, TablexiaButton> menuItems;
    private NinePatchDrawable       selectionDrawable;
    private AbstractTablexiaScreen  screen;
    private OnMenuItemClickListener onItemClickListener;
    private int                     selectedItemIndex = -1;

    public MenuWidget(AbstractTablexiaScreen screen, Point size, String selectionDrawableResource, OnMenuItemClickListener onItemClickListener) {
        menuItemValues = new Array<MenuItem>(MenuItem.values());
        menuItems = new HashMap<MenuItem, TablexiaButton>();
        this.screen = screen;
        selectionDrawable = new NinePatchDrawable(new NinePatch(screen.getScreenPatch(selectionDrawableResource)));
        selectionDrawable.setMinWidth(size.x);
        this.onItemClickListener = onItemClickListener;
        initItems();
    }

    private void initItems() {
        for (final MenuItem item : menuItemValues) {
            final TablexiaButton buttonMenuItem = new TablexiaButton(
                screen.getText(item.getTitle()),
                true,
                    new NinePatch(new Texture(new Pixmap(4, 4, Pixmap.Format.RGBA4444)), 0 ,0, 0, 0),
                    selectionDrawable.getPatch(),
                    null,
                    null
            );
            buttonMenuItem.adaptiveSize(false);
            buttonMenuItem.setChecked(false);
            buttonMenuItem.onlyOnClick(true);
            buttonMenuItem.getLabel().setAlignment(Align.left);
            buttonMenuItem.getLabel().setWrap(true);
            buttonMenuItem.removeListener();
            buttonMenuItem.setInputListener(new ClickListener() {
                @Override
                public void clicked(InputEvent event, float x, float y) {
                    selectItem(item.ordinal());
                }
            });
            menuItems.put(item, buttonMenuItem);
            addActor(buttonMenuItem);
        }
    }

    @Override
    protected void setStage(Stage stage) {
        super.setStage(stage);
    }

    public void updateSize(float width) {
        super.setWidth(width);

        for (final MenuItem item : menuItemValues) {
            TablexiaButton btn = menuItems.get(item);

            if(btn == null) continue;

            btn.setWidth(width);
            btn.setHeight(btn.getLabel().getTablexiaLabelStyle().getFontType().getSize() + 2 * BUTTON_TOP_BOTTOM_PADDING);
        }
    }

    public void selectItem(int position) {
        MenuItem searchedItem = menuItemValues.get(position);

        //Reset all items
        for(TablexiaButton currMenuItem : menuItems.values()) {
            currMenuItem.setUnchecked();
        }

        //Choose and highlight the one selected
        TablexiaButton menuItem = menuItems.get(searchedItem);
        if(menuItem != null) {
            menuItem.setChecked(true);

            if(selectedItemIndex != position) {
                onItemClickListener.onMenuItemClicked(menuItemValues.get(position));
                selectedItemIndex = position;
            }
        }
    }

    public interface OnMenuItemClickListener {
        void onMenuItemClicked(MenuItem clickedMenuItem);
    }

    public int getSelectedItemIndex() {
        return selectedItemIndex;
    }
}
