package cz.nic.tablexia.screen.gamemenu.pages;

import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;

import cz.nic.tablexia.screen.AbstractTablexiaScreen;

/**
 * Created by lhoracek on 3/4/15.
 * Edited by Drahomir Karchnak on 2/12/15
 */
public abstract class MenuPage extends Group {
    //see isClick method
    private static final float MAX_CLICK_RADIUS = 20.0f;

    private AbstractTablexiaScreen<int[][]> screen;

    private float lastTouchX = -1;
    private float lastTouchY = -1;

    public MenuPage(AbstractTablexiaScreen<int[][]> screen) {
        this.screen = screen;
    }

    protected AbstractTablexiaScreen<int[][]> getScreen() {
        return screen;
    }

	/**
	 * Background Texture's  width should be half the width of the page
	 * @return
	 */
	public abstract TextureRegion getBackgroundTextureRegion();

    //Tells if Paged Scroll Pane can snap to this page
    public abstract boolean isSnapToAllowed();

    public abstract void onScroll(float scroll);
    public abstract void onScrolledToPage();
    public abstract void onScrollPageExit();

    public abstract void onResize();

    public abstract void dispose();

    /**
     * Checks distance between coordinates of the last touchdown event and x, y
     * I made this because ClickListeners clicked method doesnt work that good when actor is in scrollpane
     * and ClickListeners inTapSquare didn't work on android for some reason
     * @param x X coord
     * @param y Y coord
     * @return returns true if touchup event at x, y was a click
     */
    public boolean isClick(float x, float y) {
        if(lastTouchX < 0 || lastTouchY < 0) return false;

        float deltaX = Math.abs(x - lastTouchX);
        float deltaY = Math.abs(y - lastTouchY);

        return Math.sqrt(deltaX*deltaX + deltaY*deltaY) <= MAX_CLICK_RADIUS;
    }

    public void setLastTouch(float x, float y) {
        lastTouchX = x;
        lastTouchY = y;
    }

    public float getLastTouchX() { return lastTouchX; }

    public float getLastTouchY() { return lastTouchY; }

    @Override
    public void setDebug(boolean enabled) {
        super.setDebug(enabled);
        for (Actor a : getChildren()) {
            a.setDebug(enabled);
        }
    }

    public boolean backButtonPressed() {
        return false;
    }
}
