package cz.nic.tablexia.screen.gamemenu.pages;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.actions.SequenceAction;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;

import cz.nic.tablexia.Tablexia;
import cz.nic.tablexia.TablexiaApplication;
import cz.nic.tablexia.TablexiaSettings;
import cz.nic.tablexia.bus.ApplicationBus;
import cz.nic.tablexia.loader.application.ApplicationAtlasManager;
import cz.nic.tablexia.model.UserDAO;
import cz.nic.tablexia.screen.AbstractTablexiaScreen;
import cz.nic.tablexia.screen.encyclopedia.EncyclopediaScreen;
import cz.nic.tablexia.screen.gamemenu.GameMenuAssets;
import cz.nic.tablexia.screen.gamemenu.GameMenuScreen;
import cz.nic.tablexia.screen.gamemenu.event.ShowGameMenuPageEvent;
import cz.nic.tablexia.screen.halloffame.HallOfFameScreen;
import cz.nic.tablexia.screen.profile.ProfileScreen;
import cz.nic.tablexia.screen.statistics.StatisticsScreen;
import cz.nic.tablexia.util.ui.ClickListenerWithSound;
import cz.nic.tablexia.util.ui.TablexiaNoBlendingImage;
import cz.nic.tablexia.util.ui.button.TablexiaButton;

/**
 * Created by lhoracek on 3/4/15.
 * Edited by Drahomir Karchnak
 */
public class OfficeMenuPage extends MenuPage {
    private static Vector2 HALL_OF_FAME_SIGN_POSITION   =   new Vector2(0.355f, 0.692f);
    private static Vector2 STREET_DOOR_POSITION         =   new Vector2(0.866f, 0.292f);
    private static Vector2 ENCYCLOPEDIA_POSITION        =   new Vector2(0.031f, 0.250f);
    private static Vector2 STATISTICS_POSITION          =   new Vector2(0.234f, 0.395f);
    private static Vector2 HALL_OF_FAME_DOOR_POSITION   =   new Vector2(0.354f, 0.465f);
    private static Vector2 DESK_UNPRESSED_POSITION      =   new Vector2(0.348f, 0.170f);
    private static Vector2 DESK_PRESSED_POSITION        =   new Vector2(0.348f, 0.170f);
    private static Vector2 HELP_OVERLAY_POSITION        =   new Vector2(0.158f, 0.407f);

    /*
      Colors for ClickMap
      check clickablemap.png in screen gamemenu assets folder, you'll get the idea.
     */
    public static final Color STREET_COLOR       = Color.YELLOW;
    public static final Color HALLOFFAME_COLOR   = Color.GREEN;
    public static final Color STATISTICS_COLOR   = Color.RED;
    public static final Color ENCYCLOPEDIA_COLOR = Color.BLACK;
    public static final Color PROFILE_COLOR      = Color.BLUE;

    //Time for help overlay to fadein and fadeout
    private static final float HELP_FADE_TIME     = 0.25f;

    //Help Button
    private static final float HELP_BUTTON_SCALE         = 0.85f;
    private static final float HELP_BUTTON_OFFSET_RIGHT  = 0.04f;
    private static final float HELP_BUTTON_OFFSET_BOTTOM = 0.018f;
    private static final int   HELP_BUTTON_SIZE          = 70;

    //Group for all helping elements, so we can hide/show 'em all at once
    private Group helpGroup;
    private Group helpOverlayStack; //For all fullscreen toggle hide/show elements of help

    //Stack of all office related actors
    private Group officeStack;

    //Images for office related actors
    private Image   hallOfFame;
    private Image   statistics;
    private Image   encyclopedia;
    private Image   street;
    private Image profilePressed;

    private Image profileUnpressed;

    private TablexiaButton toggleHelpButton;

    //Texture region of clickable map
    private final TextureRegion clickmap;

    //PagedScrollPane, which is parent of this menu page has Y offset, which we have to counter
    //to position all elements correctly
    private float pagedScrollPaneOffsetY;

    private float scrollXOffset = 0;

    private boolean playClickSound = false;

    public OfficeMenuPage(AbstractTablexiaScreen screen) {
        super(screen);

        pagedScrollPaneOffsetY = ((GameMenuScreen) screen).getPagedScrollPane().getY();

        createImageStack();

        //Prepares helpGroup, which is used to store helpOverlayStack and toggleHelpButton
        helpGroup = new Group();
        helpGroup.setPosition(getScreen().getSceneLeftX(), getScreen().getSceneOuterBottomY() - pagedScrollPaneOffsetY);
        helpGroup.setSize(getScreen().getSceneWidth(), getScreen().getSceneOuterHeight());

        //Prepares helpOverlayStack, which is a parent of HelpOverlay and HelpColorOverlay
        helpOverlayStack = new Group();
        helpOverlayStack.setPosition(getScreen().getSceneLeftX(), getScreen().getSceneOuterBottomY() - pagedScrollPaneOffsetY);
        helpOverlayStack.setSize(getScreen().getSceneWidth(), getScreen().getSceneOuterHeight());

        //Init
        prepareHelpOverlay();
        prepareToggleHelpButton();
        prepareInputListener();

        //Have user seen the help overlay ?
        if(TablexiaSettings.getInstance().getSelectedUser().isHelp()) {
            TablexiaSettings.getInstance().getSelectedUser().setHelp(false);
            UserDAO.updateUser(TablexiaSettings.getInstance().getSelectedUser());
            toggleHelpOverlay();
        }

        clickmap = getScreen().getScreenTextureRegion(GameMenuAssets.OFFICE_CLICKMAP);
        sizeChanged();
    }

    /**
     *  Creates, prepares and adds InputListener to OfficeMenuPage
     */
    public void prepareInputListener() {
        addListener(new ClickListenerWithSound() {
            @Override
            public boolean touchDown(InputEvent event, float x, float y, int pointer, int button) {
                super.touchDown(event, x, y, pointer, button);

                setLastTouch(x + scrollXOffset, y);

                //Don't do anything if help overlay is still visible
                if (helpOverlayStack.isVisible()) {
                    toggleHelpOverlay();
                    return false;
                }

                Color color = getTouchedColor(x, y);

                if (!helpOverlayStack.isVisible()) { //If Help is visible, ignore the click
                    if (color.equals(ENCYCLOPEDIA_COLOR)) {
                        encyclopedia.setVisible(true);
                    } else if (color.equals(STATISTICS_COLOR)) {
                        statistics.setVisible(true);
                    } else if (color.equals(HALLOFFAME_COLOR)) {
                        hallOfFame.setVisible(true);
                    } else if (color.equals(STREET_COLOR)) {
                        street.setVisible(true);
                    } else if (color.equals(PROFILE_COLOR)) {
                        profilePressed.setVisible(true);
                        profileUnpressed.setVisible(false);
                    }
                }
                return true;
            }

            @Override
            public void touchUp(InputEvent event, float x, float y, int pointer, int button) {
                super.touchUp(event, x, y, pointer, button);
                hideAllActions();
            }

            @Override
            public void onClick(InputEvent event, float x, float y) {
                playClickSound = true;

                if(!isClick(x + scrollXOffset, y)) {
                    playClickSound = false;
                    return;
                }

                Color color = getTouchedColor(x, y);

                if (color.equals(ENCYCLOPEDIA_COLOR)) {
                    ApplicationBus.getInstance().publishAsync(new Tablexia.ChangeScreenEvent(EncyclopediaScreen.class, TablexiaApplication.ScreenTransaction.FADE));
                } else if (color.equals(STATISTICS_COLOR)) {
                    ApplicationBus.getInstance().publishAsync(new Tablexia.ChangeScreenEvent(StatisticsScreen.class, TablexiaApplication.ScreenTransaction.FADE));
                } else if (color.equals(HALLOFFAME_COLOR)) {
                    ApplicationBus.getInstance().publishAsync(new Tablexia.ChangeScreenEvent(HallOfFameScreen.class, TablexiaApplication.ScreenTransaction.FADE));
                } else if (color.equals(STREET_COLOR)) {
                    ApplicationBus.getInstance().publishAsync(new ShowGameMenuPageEvent(GameMenuScreen.GameMenuPageDefinition.getGameMenuPageForNumber(1)));
                } else if (color.equals(PROFILE_COLOR)) {
                    ApplicationBus.getInstance().publishAsync(new Tablexia.ChangeScreenEvent(ProfileScreen.class, TablexiaApplication.ScreenTransaction.FADE));
                }
                else {
                    playClickSound = false;
                }
            }

            @Override
            protected void playButtonSound() {
                if(playClickSound)
                    super.playButtonSound();
            }
        });
    }

    /**
     * Initializes Toggle Help Button
     */
    private void prepareToggleHelpButton() {
        toggleHelpButton = new TablexiaButton(null, false,
                getScreen().getScreenTextureRegion(GameMenuAssets.HELP_BUTTON_RELEASE),
                getScreen().getScreenTextureRegion(GameMenuAssets.HELP_BUTTON_PRESSED),
                null,
                null)

                .setButtonSize(HELP_BUTTON_SIZE, HELP_BUTTON_SIZE)
                .setInputListener(new ClickListener() {

                    @Override
                    public void clicked(InputEvent event, float x, float y) {
                        super.clicked(event, x, y);
                        hideAllActions();
                        toggleHelpOverlay();
                    }
                });

        positionHelpButton();

        helpGroup.addActor(toggleHelpButton);
    }

    private void positionHelpButton() {
        toggleHelpButton.setPosition(getScreen().getViewportRightX() - toggleHelpButton.getWidth() - (HELP_BUTTON_OFFSET_RIGHT * getScreen().getSceneWidth()),
                getScreen().getViewportBottomY() - pagedScrollPaneOffsetY + (HELP_BUTTON_OFFSET_BOTTOM * getScreen().getSceneOuterHeight()));
    }

    /**
     * Initializes Help Overlay
     */
    private void prepareHelpOverlay() {
        Image helpColorOverlay  = new Image(ApplicationAtlasManager.getInstance().getColorTexture(ApplicationAtlasManager.COLOR_OVERLAY));
        helpColorOverlay.setSize(getScreen().getSceneWidth(), getScreen().getSceneOuterHeight());

        Image helpOverlay = createImage(GameMenuAssets.OFFICE_HELP, HELP_OVERLAY_POSITION);

        helpOverlayStack.addActor(helpColorOverlay);
        helpOverlayStack.addActor(helpOverlay);

        helpOverlayStack.setVisible(false);
        helpOverlayStack.addAction(Actions.alpha(0));

        helpGroup.addActor(helpOverlayStack);
        addActor(helpGroup);
    }

    private void createImageStack() {
        officeStack = new Group();

        officeStack.setSize(getScreen().getSceneWidth(), getScreen().getSceneOuterHeight());
        officeStack.setPosition(0, getScreen().getSceneOuterBottomY() - pagedScrollPaneOffsetY);

        officeStack.addActor(new TablexiaNoBlendingImage(getScreen().getScreenTextureRegion(GameMenuAssets.OFFICE)));
        officeStack.addActor(createImage(GameMenuAssets.OFFICE_HALLOFFAME, HALL_OF_FAME_SIGN_POSITION)); //napis
        officeStack.addActor(street = createImage(GameMenuAssets.DOOR_PRESSED, STREET_DOOR_POSITION));
        officeStack.addActor(encyclopedia = createImage(GameMenuAssets.ENCYCLOPEDIA_PRESSED, ENCYCLOPEDIA_POSITION));
        officeStack.addActor(statistics = createImage(GameMenuAssets.STATISTICS_PRESSED, STATISTICS_POSITION));
        officeStack.addActor(hallOfFame = createImage(GameMenuAssets.HALLOFFAME_PRESSED, HALL_OF_FAME_DOOR_POSITION));
        officeStack.addActor(profileUnpressed = createImage(GameMenuAssets.DESK, DESK_UNPRESSED_POSITION));
        officeStack.addActor(profilePressed = createImage(GameMenuAssets.PROFILE_PRESSED, DESK_PRESSED_POSITION));


        addActor(officeStack);
        hideAllActions();
    }

    /**
     * Hides all possibly visible actions
     */
    private void hideAllActions() {
        profileUnpressed.setVisible(true);

        profilePressed.setVisible(false);
        street.setVisible(false);
        encyclopedia.setVisible(false);
        statistics.setVisible(false);
        hallOfFame.setVisible(false);
    }

    /**
     * Helper method for creating image instances
     *
     * @param texture texture name
     * @return return image
     */
    private Image createImage(String texture, Vector2 position) {
        TextureRegion tr = getScreen().getScreenTextureRegion(texture);

        Image result = new Image(tr);
        result.setPosition(position.x * getScreen().getSceneWidth(), position.y * getScreen().getSceneOuterHeight());
        return result;
    }

    /**
     * Gets color from clickmap according to coords x and y
     * @param x X coord
     * @param y Y coord
     * @return color clicked, Color.BLACK if anything went wrong
     */
    private Color getTouchedColor(float x, float y) {
        int clickX = (int) (x / getWidth() * clickmap.getRegionWidth());
        int clickY = clickmap.getRegionHeight() - (int) (y / getHeight() * clickmap.getRegionHeight());

        if (clickX >= getScreen().getData().length ||
                clickY >= getScreen().getData()[0].length ||
                clickX < 0 || clickY < 0)
        {
            return Color.BLACK;
        }

        return new Color(getScreen().getData()[clickX][clickY]);
    }

    @Override
    public void onScrolledToPage() {
        ApplicationBus.getInstance().post(new GameMenuPage.GameMenuPageScrolledEvent(GameMenuScreen.GameMenuPageDefinition.OFFICE_PAGE)).asynchronously();
    }

    @Override
    public void onScrollPageExit() { }

    @Override
    public void onResize() {
        positionHelpButton();
    }

    @Override
    public void dispose() { }

    @Override
    public void onScroll(float offset) {
        scrollXOffset = offset;
    }

    /**
     * Toggles (FadeIn/FadeOut) help overlay
     */
    private void toggleHelpOverlay() {
        if(helpOverlayStack.isVisible()) {
            hideHelpOverlay();
        } else {
            showHelpOverlay();
        }
    }

    private void hideHelpOverlay() {
        toggleHelpButton.setUnchecked();
        helpOverlayStack.addAction( new SequenceAction(
                Actions.fadeOut(HELP_FADE_TIME),
                Actions.run( new Runnable() {
                    @Override
                    public void run() {
                        helpOverlayStack.setVisible(false);
                    }
                })
        ));
    }

    private void showHelpOverlay() {
        toggleHelpButton.setChecked();
        helpOverlayStack.addAction( new SequenceAction(
                Actions.run( new Runnable() {
                    @Override
                    public void run() {
                        helpOverlayStack.setVisible(true);
                    }
                }),
                Actions.fadeIn(HELP_FADE_TIME)
        ));
    }

	@Override
	public TextureRegion getBackgroundTextureRegion() { return null; }

    @Override
    public boolean isSnapToAllowed() {
        return true;
    }

    @Override
    public boolean backButtonPressed() {
        if (helpOverlayStack.isVisible()) {
            hideHelpOverlay();
            return true;
        }
        return false;
    }
}