package cz.nic.tablexia.screen.gamemenu.pages;

import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Vector2;

import cz.nic.tablexia.screen.AbstractTablexiaScreen;
import cz.nic.tablexia.screen.gamemenu.GameMenuAssets;
import cz.nic.tablexia.screen.gamemenu.GameMenuScreen;

/**
 * This class represents the page between the office and game menu pages
 * It works as a separator and PagedScrollPane can't snap to this page
 * Created by Drahomir Karchnak on 02/12/15.
 */
public class WallPage extends MenuPage{
    private static final Vector2 SIGN_POSITION = new Vector2(0.230f, 0.286f);

    private TextureRegion back = getScreen().getScreenTextureRegion(GameMenuAssets.WALL_BACK);
    private TextureRegion mid = getScreen().getScreenTextureRegion(GameMenuAssets.WALL_MID);
	private TextureRegion sign = getScreen().getScreenTextureRegion(GameMenuAssets.WALL_SIGN);

    private float pagedScrollPaneOffsetY;

    public WallPage(AbstractTablexiaScreen<int[][]> screen) {
        super(screen);

        //offset of the pagedScrollPane
        pagedScrollPaneOffsetY = ((GameMenuScreen) screen).getPagedScrollPane().getY();

        //Sets size and position of WallPage
        setSize(mid.getRegionWidth(), getScreen().getSceneOuterHeight());
        setPosition(getScreen().getSceneWidth(), getScreen().getSceneOuterBottomY() - getScreen().getSceneOuterBottomY());
    }

	@Override
	public TextureRegion getBackgroundTextureRegion() {
		return back;
	}

	@Override
    public void draw(Batch batch, float parentAlpha) {
        batch.setColor(batch.getColor().r, batch.getColor().g, batch.getColor().b, parentAlpha);

        //Enables Blending for foreground layer
        batch.enableBlending();

        batch.draw(mid,
				getScreen().getSceneLeftX() + getX(),
				getScreen().getSceneOuterBottomY() - pagedScrollPaneOffsetY,
				getWidth(),
				getHeight());

		batch.draw(sign,
				getScreen().getSceneLeftX() + getX() + (getWidth() * SIGN_POSITION.x),
				getScreen().getSceneOuterBottomY() - pagedScrollPaneOffsetY + (getHeight() * SIGN_POSITION.y),
				sign.getRegionWidth(),
				sign.getRegionHeight());

		super.draw(batch, parentAlpha);
    }

    @Override
    public boolean isSnapToAllowed() {
        //PagedScrollPane can't snap to this page
        return false;
    }

    @Override
    public void onScroll(float scroll) { }

    @Override
    public void onScrolledToPage() { }

    @Override
    public void onScrollPageExit() { }

    @Override
    public void onResize() { }

    @Override
    public void dispose() { }
}
