package cz.nic.tablexia.screen.halloffame.trophy;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import cz.nic.tablexia.game.GameDefinition;
import cz.nic.tablexia.game.trophy.GameTrophyDefinition;
import cz.nic.tablexia.game.trophy.ITrophyDefinition;
import cz.nic.tablexia.game.trophy.UserTrophyDefinition;
import cz.nic.tablexia.screen.halloffame.assets.HallOfFameAssets;
import cz.nic.tablexia.util.structure.Trio;

/**
 * Created by Václav Tarantík on 6.3.15.
 */
public class TrophyHelper {

    private static final String TROPHY_SUBFOLDER_NAME   = "trophy/";
    private static final String TROPHY_PREFIX           = "trophy_";

    public static final Trio<Float,Float,Float> heapProperties = new Trio<Float,Float,Float>(0.25f, 0f, 1f);

    public enum TrophyHeapAssets {

        HALLOFFAME_TROPHYHEAP_00(HallOfFameAssets.HALL_OF_FAME_TROPHY_HEAP_00, 0), //
        HALLOFFAME_TROPHYHEAP_01(HallOfFameAssets.HALL_OF_FAME_TROPHY_HEAP_01, 1), //
        HALLOFFAME_TROPHYHEAP_02(HallOfFameAssets.HALL_OF_FAME_TROPHY_HEAP_02, 2), //
        HALLOFFAME_TROPHYHEAP_03(HallOfFameAssets.HALL_OF_FAME_TROPHY_HEAP_03, 3), //
        HALLOFFAME_TROPHYHEAP_04(HallOfFameAssets.HALL_OF_FAME_TROPHY_HEAP_04, 4), //
        HALLOFFAME_TROPHYHEAP_05(HallOfFameAssets.HALL_OF_FAME_TROPHY_HEAP_05, 5), //
        HALLOFFAME_TROPHYHEAP_06(HallOfFameAssets.HALL_OF_FAME_TROPHY_HEAP_06, 6), //
        HALLOFFAME_TROPHYHEAP_07(HallOfFameAssets.HALL_OF_FAME_TROPHY_HEAP_07, 7), //
        HALLOFFAME_TROPHYHEAP_08(HallOfFameAssets.HALL_OF_FAME_TROPHY_HEAP_08, 8), //
        HALLOFFAME_TROPHYHEAP_09(HallOfFameAssets.HALL_OF_FAME_TROPHY_HEAP_09, 9), //
        HALLOFFAME_TROPHYHEAP_10(HallOfFameAssets.HALL_OF_FAME_TROPHY_HEAP_10, 10), //
        HALLOFFAME_TROPHYHEAP_15(HallOfFameAssets.HALL_OF_FAME_TROPHY_HEAP_15, 15), //
        HALLOFFAME_TROPHYHEAP_20(HallOfFameAssets.HALL_OF_FAME_TROPHY_HEAP_20, 20), //
        HALLOFFAME_TROPHYHEAP_30(HallOfFameAssets.HALL_OF_FAME_TROPHY_HEAP_30, 30), //
        HALLOFFAME_TROPHYHEAP_35(HallOfFameAssets.HALL_OF_FAME_TROPHY_HEAP_35, 35), //
        HALLOFFAME_TROPHYHEAP_40(HallOfFameAssets.HALL_OF_FAME_TROPHY_HEAP_40, 40), //
        HALLOFFAME_TROPHYHEAP_50(HallOfFameAssets.HALL_OF_FAME_TROPHY_HEAP_50, 50), //
        HALLOFFAME_TROPHYHEAP_60(HallOfFameAssets.HALL_OF_FAME_TROPHY_HEAP_60, 60);

        private String trophyHeapAsset;
        private int starCount;

        TrophyHeapAssets(String trophyHeapAsset, int starCount) {
            this.trophyHeapAsset = trophyHeapAsset;
            this.starCount = starCount;
        }

        public String getTrophyHeapAssetName() {
            return trophyHeapAsset;
        }

        public int getStarCount() {
            return starCount;
        }

        public static String getTrophyHeapAssetNameForStarCount(int starCount) {
            for (int i = TrophyHeapAssets.values().length - 1; i >= 0; i--) {
                TrophyHeapAssets trophyHeapAssets = TrophyHeapAssets.values()[i];
                if (starCount >= trophyHeapAssets.getStarCount()) {
                    return trophyHeapAssets.getTrophyHeapAssetName();
                }
            }
            return TrophyHeapAssets.values()[0].getTrophyHeapAssetName();
        }
    }

    public static final Map<ITrophyDefinition, Trio<Float,Float,Float>> trophiesProperties;//x relative position, y relative position, scale relative to original size

    static {
        trophiesProperties = new HashMap<ITrophyDefinition, Trio<Float, Float,Float>>();

        trophiesProperties.put(GameTrophyDefinition.ROBBERY_PLAY1, new Trio<Float, Float, Float>(2.4f, 0.4f, 1f));// robbery_1_finished1,17
        trophiesProperties.put(GameTrophyDefinition.ROBBERY_PLAY2, new Trio<Float, Float, Float>(2.05f, 0.4f, 1f));// robbery_5_finished,14
        trophiesProperties.put(GameTrophyDefinition.ROBBERY_PLAY3, new Trio<Float, Float, Float>(2.15f, 0.6f, 1f));// robbery_10_finished,15
        trophiesProperties.put(GameTrophyDefinition.ROBBERY_DIFF1, new Trio<Float, Float, Float>(2.25f, 0.2f, 1f));// robbery_1_limit,16
        trophiesProperties.put(GameTrophyDefinition.ROBBERY_DIFF2, new Trio<Float, Float, Float>(2.65f, 0.2f, 1f));// robbery_2_limit,19

        trophiesProperties.put(GameTrophyDefinition.PURSUIT_PLAY1, new Trio<Float, Float, Float>(1.51f, 0.62f, 1f));// pursuit_1_finished,9
        trophiesProperties.put(GameTrophyDefinition.PURSUIT_PLAY2, new Trio<Float, Float, Float>(1.35f, 0.55f, 1f));// pursuit_5_finished,7
        trophiesProperties.put(GameTrophyDefinition.PURSUIT_PLAY3, new Trio<Float, Float, Float>(1.51f, 0.33f, 1f));// pursuit_10_finished,10
        trophiesProperties.put(GameTrophyDefinition.PURSUIT_DIFF1, new Trio<Float, Float, Float>(1.74f, 0.68f, 1f));// pursuit_1_limit,11
        trophiesProperties.put(GameTrophyDefinition.PURSUIT_DIFF2, new Trio<Float, Float, Float>(1.81f, 0.39f, 1f));// pursuit_2_limit,12

        trophiesProperties.put(GameTrophyDefinition.KIDNAPPING_PLAY1, new Trio<Float, Float, Float>(0.95f, 0.65f, 1f));// kidnapping_1_finished,3
        trophiesProperties.put(GameTrophyDefinition.KIDNAPPING_PLAY2, new Trio<Float, Float, Float>(1.0f, 0.2f, 1f));// kidnapping_5_finished,4
        trophiesProperties.put(GameTrophyDefinition.KIDNAPPING_PLAY3, new Trio<Float, Float, Float>(1.15f, 0.5f, 1f));// kidnapping_10_finished,5
        trophiesProperties.put(GameTrophyDefinition.KIDNAPPING_DIFF1, new Trio<Float, Float, Float>(0.75f, 0.4f, 1f));// kidnapping_1_limit,1
        trophiesProperties.put(GameTrophyDefinition.KIDNAPPING_DIFF2, new Trio<Float, Float, Float>(0.6f, 0.7f, 1f));// kidnapping_2_limit,0

        trophiesProperties.put(GameTrophyDefinition.NIGHT_WATCH_PLAY1, new Trio<Float, Float, Float>(3.5f, 0.5f, 1f));// night_watch_1_finished,22
        trophiesProperties.put(GameTrophyDefinition.NIGHT_WATCH_PLAY2, new Trio<Float, Float, Float>(3.63f, 0.6f, 1f));// night_watch_5_finished,23
        trophiesProperties.put(GameTrophyDefinition.NIGHT_WATCH_PLAY3, new Trio<Float, Float, Float>(3.8f, 0.5f, 1f));// night_watch_10_finished,25
        trophiesProperties.put(GameTrophyDefinition.NIGHT_WATCH_DIFF1, new Trio<Float, Float, Float>(3.78f, 0.77f, 1f));// night_watch_1_limit,24
        trophiesProperties.put(GameTrophyDefinition.NIGHT_WATCH_DIFF2, new Trio<Float, Float, Float>(3.25f, 0.5f, 1f));// night_watch_2_limit,21

        trophiesProperties.put(GameTrophyDefinition.SHOOTING_RANGE_PLAY1, new Trio<Float, Float, Float>(4.0f, 0.7f, 1f));// shooting_range_1_finished,26
        trophiesProperties.put(GameTrophyDefinition.SHOOTING_RANGE_PLAY2, new Trio<Float, Float, Float>(4.1f, 0.4f, 1f));// shooting_range_5_finished,27
        trophiesProperties.put(GameTrophyDefinition.SHOOTING_RANGE_PLAY3, new Trio<Float, Float, Float>(4.5f, 0.55f, 1f));// shooting_range_10_finished,30
        trophiesProperties.put(GameTrophyDefinition.SHOOTING_RANGE_DIFF1, new Trio<Float, Float, Float>(4.35f, 0.5f, 0.6f));// shooting_range_1_limit,29
        trophiesProperties.put(GameTrophyDefinition.SHOOTING_RANGE_DIFF2, new Trio<Float, Float, Float>(4.27f, 0.7f, 0.8f));// shooting_range_2_limit,28

        trophiesProperties.put(GameTrophyDefinition.IN_THE_DARKNESS_PLAY1, new Trio<Float, Float, Float>(4.7f, 0.43f, 0.7f));// in_the_darkness_1_finished,32
        trophiesProperties.put(GameTrophyDefinition.IN_THE_DARKNESS_PLAY2, new Trio<Float, Float, Float>(5.05f, 0.77f, 0.7f));// in_the_darkness_5_finished,33
        trophiesProperties.put(GameTrophyDefinition.IN_THE_DARKNESS_PLAY3, new Trio<Float, Float, Float>(4.85f, 0.1f, 0.7f));// in_the_darkness_10_finished.34
        trophiesProperties.put(GameTrophyDefinition.IN_THE_DARKNESS_DIFF1, new Trio<Float, Float, Float>(4.85f, 0.65f, 0.7f));// in_the_darkness_1_limit,31
        trophiesProperties.put(GameTrophyDefinition.IN_THE_DARKNESS_DIFF2, new Trio<Float, Float, Float>(5.23f, 0.4f, 0.7f));// in_the_darkness_2_limit,35

        //USER TROPHIES
        trophiesProperties.put(UserTrophyDefinition.CONSECUTIVELY_DAYSBACK_3, new Trio<Float, Float, Float>(0.87f, 0.38f, 1f));//user_consecutively3days,2
        trophiesProperties.put(UserTrophyDefinition.CONSECUTIVELY_DAYSBACK_5, new Trio<Float, Float, Float>(1.3f, 0.75f, 1f));//user_consecutively5days,6
        trophiesProperties.put(UserTrophyDefinition.CONSECUTIVELY_DAYSBACK_8, new Trio<Float, Float, Float>(2.45f, 0.6f, 1f));//user_consecutively8days,18
        trophiesProperties.put(UserTrophyDefinition.ONEDAYALLGAMES_0STARS, new Trio<Float, Float, Float>(3.05f, 0.55f, 1f));//user_consecutivelyallgames0stars,20
        trophiesProperties.put(UserTrophyDefinition.CONSECUTIVELYALLGAMES_MEDIUMDIFFICULTY, new Trio<Float, Float, Float>(1.92f, 0.65f, 1f));//user_consecutivelyallgames2stars,13
        trophiesProperties.put(UserTrophyDefinition.CONSECUTIVELYALLGAMES_HARDDIFFICULTY, new Trio<Float, Float, Float>(1.3f, 0.05f, 1f));//user_consecutivelyallgames3stars,8

    }

    public enum TrophyType{
        EMPTY,
        SMALL,
        FULL
    }

    public static List<String> getTrophyNamesList() {
        List<String> trophyNamesToBeLoaded = new ArrayList<String>();
        //common trophies
        for(GameTrophyDefinition trophyDefinition: GameTrophyDefinition.values()){
            for(TrophyType trophyType : TrophyType.values()){
                trophyNamesToBeLoaded.add(getFullPath(trophyDefinition.getTrophyNameKey(),trophyType));
            }
        }
        //user trophies
        for(UserTrophyDefinition trophyDefinition: UserTrophyDefinition.values()){
            for(TrophyType trophyType : TrophyType.values()){
                trophyNamesToBeLoaded.add(getFullPath(trophyDefinition.getTrophyNameKey(),trophyType));
            }
        }
        return trophyNamesToBeLoaded;
    }

    public static String getFullPath(String imageName, TrophyType trophyType){
        return TROPHY_SUBFOLDER_NAME + TROPHY_PREFIX + imageName + "_" + trophyType.toString().toLowerCase();
    }

    public static List<ITrophyDefinition> getPossibleTrophiesForGame(GameDefinition gameDef) {
        List<ITrophyDefinition> list = new ArrayList<ITrophyDefinition>();

        list.addAll(Arrays.asList(UserTrophyDefinition.values()));

        for (GameTrophyDefinition trophy : GameTrophyDefinition.values()) {
            if (trophy.getGameDefinition().equals(gameDef)) {
                list.add(trophy);
            }
        }
        return list;
    }

    public static ITrophyDefinition getTrophyDefinitionByName(String name) {
        ITrophyDefinition trophy = null;

        try {
            trophy = GameTrophyDefinition.valueOf(name);
        } catch (IllegalArgumentException ex) {
            // we don't find trophy in game setion
        }

        try {
            trophy = UserTrophyDefinition.valueOf(name);
        } catch (IllegalArgumentException ex) {
            // we don't find trophy in user setion
        }

        return trophy;
    }

    public enum MouseTexts {
        MOUSE_TEXTS_0       (0,     "mouse_0_1", "mouse_0_2", "mouse_0_3", "mouse_0_4"),
        MOUSE_TEXTS_4       (4,     "mouse_4_1"),
        MOUSE_TEXTS_5       (5,     "mouse_5_1"),
        MOUSE_TEXTS_6       (6,     "mouse_6_1"),
        MOUSE_TEXTS_9       (9,     "mouse_9_1"),
        MOUSE_TEXTS_13      (13,    "mouse_13_1"),
        MOUSE_TEXTS_14      (14,    "mouse_14_1"),
        MOUSE_TEXTS_16      (16,    "mouse_16_1"),
        MOUSE_TEXTS_17      (17,    "mouse_17_1"),
        MOUSE_TEXTS_19      (19,    "mouse_19_1"),
        MOUSE_TEXTS_21      (21,    "mouse_21_1", "mouse_21_2", "mouse_21_3"),
        MOUSE_TEXTS_25      (25,    "mouse_25_1"),
        MOUSE_TEXTS_28      (28,    "mouse_28_1"),
        MOUSE_TEXTS_29      (29,    "mouse_29_1"),
        MOUSE_TEXTS_30      (30,    "mouse_30_1"),
        MOUSE_TEXTS_31      (31,    "mouse_31_1"),
        MOUSE_TEXTS_32      (32,    "mouse_32_1"),
        MOUSE_TEXTS_33      (33,    "mouse_33_1"),
        MOUSE_TEXTS_34      (34,    "mouse_34_1"),
        MOUSE_TEXTS_36      (36,    "mouse_36_1", "mouse_36_2", "mouse_36_3");

        public static final String MOUSE_RANKUP_TEXT = "mouse_rankup";

        private String[] texts;
        private int numOfStars;
        private int actualIndex = 0;

        MouseTexts(int numOfStars, String... texts) {
            this.texts = texts;
            this.numOfStars = numOfStars;
        }

        public String getNextText() {
            String result = null;

            if (actualIndex < texts.length) {
                result = texts[actualIndex];
                actualIndex++;
            }

            return result;
        }

        public int getNumOfStars() {
            return numOfStars;
        }

        public void resetTextCounter() {
            actualIndex = 0;
        }

        public static void resetAllTextCounters() {
            for (MouseTexts text : values()) {
                text.resetTextCounter();
            }
        }

        public static String getMouseText(int trophyCount) {
            MouseTexts finalText = MOUSE_TEXTS_0;

            for (MouseTexts text : values()) {
                if(trophyCount >= text.getNumOfStars() && text.getNumOfStars() > finalText.getNumOfStars()) {
                    finalText = text;
                }
            }

            return finalText.getNextText();
        }
    }
}
