package cz.nic.tablexia.screen.profile;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.utils.Drawable;
import com.badlogic.gdx.scenes.scene2d.utils.TextureRegionDrawable;
import com.badlogic.gdx.utils.Align;

import java.util.Map;

import cz.nic.tablexia.game.ranksystem.UserRankManager;
import cz.nic.tablexia.loader.application.ApplicationAtlasManager;
import cz.nic.tablexia.loader.application.ApplicationFontManager;
import cz.nic.tablexia.loader.application.ApplicationTextManager;
import cz.nic.tablexia.menu.user.UserAvatarDefinition;
import cz.nic.tablexia.screen.createuser.FormScreen;
import cz.nic.tablexia.shared.model.User;
import cz.nic.tablexia.util.ScaleUtil;
import cz.nic.tablexia.util.ui.TablexiaLabel;
import cz.nic.tablexia.util.ui.TablexiaProgressBar;

public class ProfileScreen extends FormScreen {
    private static final String PROGRESS_BAR_FILL             = "gfx/progressbar_fill";
    private static final float  PROGRESS_BAR_WIDTH            = 0.8f;
    private static final float  PROGRESS_BAR_BOTTOM_OFFSET    = 0.28f;
    private static final float  PROGRESS_BAR_SPEED            = 3.0f;

    private static final ApplicationFontManager.FontType_NEW RANK_NAME_FONT_TYPE = ApplicationFontManager.FontType_NEW.BOLD_20;

    private static final String PROGRESS_STATUS_ARROW         = "gfx/progress_arrow";
    private static final float  PROGRESS_STATUS_WIDTH         = 0.8f;
    private static final float  PROGRESS_STATUS_ARROWS_WIDTH  = 0.45f;
    private static final float  PROGRESS_STATUS_BADGE_HEIGHT   = 0.2f;
    private static final float  PROGRESS_STATUS_BOTTOM_OFFSET = 0.33f;

    private static final String USER_UUID_ERROR               = "user_uuid_error";
    private static final String USER_UUID_DESCRIPTION_TEXT    = "user_uuid_description";

    private static final float  BADGE_HEIGHT                            = 0.4f;
    private static final float  BADGE_POSITON_Y_SCALE                   = 0.52f;
    private static final float  USER_UUID_DESCRIPTION_POSITION_Y_SCALE  = 0.08f;
    private static final float  USER_UUID_POSITION_Y_SCALE              = 0.05f;
    private static final float  USER_UUID_PADDING                       = 0.055f; //Percent from page width

    private TablexiaProgressBar progressBar;

    @Override
    protected void screenLoaded(Map<String, String> screenState) {
        super.screenLoaded(screenState);
        pen.setVisible(false);
        stamp.setVisible(false);
        inkpad.setVisible(false);
        ink.setVisible(false);

        mugshotFrame.clearListeners();
        buttonPlus.setVisible(false);
        buttonMinus.setVisible(false);
        nameField.clearListeners();
        switchGender.setVisible(false);
        clearName.setVisible(false);

        badge.setY(getSceneInnerBottomY() + getSceneInnerHeight() * BADGE_POSITON_Y_SCALE);

        Drawable d = new TextureRegionDrawable(new TextureRegion(getApplicationTextureRegion(UserAvatarDefinition.getAvatar(getSelectedUser().getAvatar()))));
        mugshotImage.setDrawable(d);
        nameField.setText(getSelectedUser().getName());
        ageValue.setText(String.valueOf(getSelectedUser().getAge()));

        imageFemale.setVisible(getSelectedUser().getGender().equals(User.Gender.FEMALE));
        imageMale.setVisible(getSelectedUser().getGender().equals(User.Gender.MALE));
        float center = (imageMale.getX() + imageFemale.getX()) / 2;
        imageFemale.setX(center);
        imageMale.setX(center);
        signaturePane.setSignature(getSelectedUser().getSignature());
        stampIt();

        prepareProgressBar();
        prepareProgressStatus();

        getStage().addActor(createUserUuidDescription());

        TablexiaLabel userUuidLabel = createUserUuidLabel();
        if(userUuidLabel != null)
            getStage().addActor(createUserUuidLabel());
    }

    @Override
    public boolean hasSoftBackButton() {
        return true;
    }

    @Override
    protected void screenResized(int width, int height) {
        super.screenResized(width, height);
    }

    @Override
    protected boolean validationEnabled() {
        return false;
    }

    @Override
    protected String prepareScreenName() {
        return "form";
    }

    private TablexiaLabel createUserUuidDescription() {
        TablexiaLabel resultLabel;

        String resultLabelText = getSelectedUser().getUuid() == null || getSelectedUser().getUuid().isEmpty() ? getText(USER_UUID_ERROR) : getText(USER_UUID_DESCRIPTION_TEXT);

        resultLabel = new TablexiaLabel(
                resultLabelText,
                new TablexiaLabel.TablexiaLabelStyle(ApplicationFontManager.FontType_NEW.BOLD_16, Color.BLACK)
        );

        resultLabel.setPosition(
                book.getX() + book.getWidth() / 2 * USER_UUID_PADDING,
                getStage().getWidth() * USER_UUID_DESCRIPTION_POSITION_Y_SCALE
        );
        resultLabel.setWidth(book.getWidth() / 2f - 2 * (book.getWidth() / 2) * USER_UUID_PADDING);
        resultLabel.setAlignment(Align.center);
        resultLabel.setWrap(true);
        return resultLabel;
    }

    private TablexiaLabel createUserUuidLabel() {
        if(getSelectedUser().getUuid() == null || getSelectedUser().getUuid().isEmpty())
            return null;

        TablexiaLabel label = new TablexiaLabel(getSelectedUser().getUuid(), new TablexiaLabel.TablexiaLabelStyle(ApplicationFontManager.FontType_NEW.BOLD_16, Color.BLACK));
        label.setPosition(
                book.getX(),
                getStage().getWidth() * USER_UUID_POSITION_Y_SCALE
        );
        label.setWidth(book.getWidth() / 2f);
        label.setAlignment(Align.center);
        return label;
    }

    @Override
    protected Actor prepareBadge() {
        UserRankManager.UserRank rank = UserRankManager.getInstance().getRank(getSelectedUser());
        badge = ScaleUtil.createImageToHeight(ApplicationAtlasManager.getInstance().getTextureRegion(rank.getBadgeImageKey()), BADGE_HEIGHT * getSceneInnerHeight());
        float pageWidth = (book.getWidth()/2);
        badge.setPosition(book.getX() + pageWidth/2 - badge.getWidth()/2, badge.getY());
        return badge;
    }

    private void prepareProgressBar() {
        this.progressBar = new TablexiaProgressBar(
                ApplicationAtlasManager.getInstance().getTextureRegion(ApplicationAtlasManager.PROGRESS_BAR),
                getScreenTextureRegion(PROGRESS_BAR_FILL)
        );

        float pageWidth = book.getWidth() / 2f;

        this.progressBar.setWidth(pageWidth * PROGRESS_BAR_WIDTH);
        this.progressBar.setPosition(
                book.getX() + (pageWidth / 2f) - (progressBar.getWidth() / 2),
                book.getY() + book.getHeight() * PROGRESS_BAR_BOTTOM_OFFSET
        );

        UserRankManager.RankProgress resultRankProgress = UserRankManager.getInstance().getTotalUserRankProgress(getSelectedUser());

        this.progressBar.setSmooth(false);
        this.progressBar.setPercent(0);
        this.progressBar.setSmooth(true);
        this.progressBar.setSpeed(PROGRESS_BAR_SPEED);
        this.progressBar.setPercent(resultRankProgress.getPercentDone());

        getStage().addActor(progressBar);
    }

    private void prepareProgressStatus () {
        final float pageWidth = book.getWidth()/2f;
        final float groupWidth = pageWidth * PROGRESS_STATUS_WIDTH;

        Group group = new Group();
        group.setWidth(groupWidth);
        group.setPosition(book.getX() + pageWidth/2f - group.getWidth()/2f, book.getY() + book.getHeight() * PROGRESS_STATUS_BOTTOM_OFFSET);

        UserRankManager.UserRank currRank = UserRankManager.getInstance().getRank(getSelectedUser());
        UserRankManager.UserRank nextRank = UserRankManager.UserRank.getNextRank(currRank);

        Image currBadge = ScaleUtil.createImageToHeight(ApplicationAtlasManager.getInstance().getTextureRegion(currRank.getBadgeIconKey()), PROGRESS_STATUS_BADGE_HEIGHT * groupWidth);
        group.addActor(currBadge);

        Image progress = ScaleUtil.createImageToWidth(getScreenTextureRegion(PROGRESS_STATUS_ARROW), PROGRESS_STATUS_ARROWS_WIDTH * groupWidth);
        progress.setPosition(group.getWidth()/2 - progress.getWidth()/2, currBadge.getY() + currBadge.getHeight()/2 - progress.getHeight()/2);
        group.addActor(progress);

        Image nextBadge = ScaleUtil.createGreyScaleImageToHeight(ApplicationAtlasManager.getInstance().getTextureRegion(nextRank.getBadgeIconKey()), PROGRESS_STATUS_BADGE_HEIGHT * groupWidth);
        nextBadge.setX(group.getWidth() - nextBadge.getWidth());
        group.addActor(nextBadge);

        if(currRank.hasBadgeName()) {
            TablexiaLabel rankName = new TablexiaLabel(ApplicationTextManager.getInstance().getText(currRank.getNameKey()), new TablexiaLabel.TablexiaLabelStyle(RANK_NAME_FONT_TYPE, Color.BLACK));
            rankName.setSize(group.getWidth(), rankName.getHeight());
            rankName.setAlignment(Align.center);
            rankName.setY(progress.getY() + progress.getHeight());
            group.addActor(rankName);
        }

        getStage().addActor(group);
    }

	@Override
	public void backButtonPressed() {
		backToInitialScreen();
	}
}
