package cz.nic.tablexia.screen.statistics;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.NinePatch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.InputListener;
import com.badlogic.gdx.scenes.scene2d.ui.Cell;
import com.badlogic.gdx.scenes.scene2d.ui.Container;
import com.badlogic.gdx.scenes.scene2d.ui.HorizontalGroup;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;
import com.badlogic.gdx.scenes.scene2d.ui.Stack;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.ui.VerticalGroup;
import com.badlogic.gdx.scenes.scene2d.ui.WidgetGroup;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.scenes.scene2d.utils.NinePatchDrawable;
import com.badlogic.gdx.utils.Align;
import com.badlogic.gdx.utils.Scaling;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import cz.nic.tablexia.TablexiaSettings;
import cz.nic.tablexia.game.GameDefinition;
import cz.nic.tablexia.game.difficulty.GameDifficulty;
import cz.nic.tablexia.loader.application.ApplicationAtlasManager;
import cz.nic.tablexia.loader.application.ApplicationFontManager;
import cz.nic.tablexia.loader.application.ApplicationTextManager;
import cz.nic.tablexia.menu.game.GameMenuDefinition;
import cz.nic.tablexia.model.game.GameDAO;
import cz.nic.tablexia.screen.AbstractTablexiaScreen;
import cz.nic.tablexia.screen.statistics.enums.GraphStyle;
import cz.nic.tablexia.screen.statistics.interfaces.GameClickListener;
import cz.nic.tablexia.screen.statistics.views.GraphPane;
import cz.nic.tablexia.shared.model.Game;
import cz.nic.tablexia.util.Log;
import cz.nic.tablexia.util.Point;
import cz.nic.tablexia.util.ScaleUtil;
import cz.nic.tablexia.util.ui.ClickListenerWithSound;
import cz.nic.tablexia.util.ui.Switch;
import cz.nic.tablexia.util.ui.TablexiaLabel;
import cz.nic.tablexia.util.ui.TablexiaNoBlendingImage;
import cz.nic.tablexia.util.ui.button.TablexiaButton;
import cz.nic.tablexia.util.ui.dialog.TablexiaComponentDialog;
import cz.nic.tablexia.util.ui.dialog.TablexiaComponentDialogFactory;
import cz.nic.tablexia.util.ui.dialog.components.AdaptiveSizeDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.ResizableSpaceContentDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.SpeechDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.TablexiaDialogComponentAdapter;
import cz.nic.tablexia.util.ui.dialog.components.TextContentDialogComponent;
import cz.nic.tablexia.util.ui.dialog.components.TouchCloseDialogComponent;

public class StatisticsScreen extends AbstractTablexiaScreen<Void> {
    private static final ApplicationFontManager.FontType_NEW DEFAULT_FONT_TYPE    = ApplicationFontManager.FontType_NEW.BOLD_16;
    private static final ApplicationFontManager.FontType_NEW DIFFICULTY_FONT_TYPE = ApplicationFontManager.FontType_NEW.BOLD_20;
    private static final ApplicationFontManager.FontType_NEW BOOKMARK_FONT_TYPE   = ApplicationFontManager.FontType_NEW.BOLD_26;
    private static final ApplicationFontManager.FontType_NEW TITLE_FONT_TYPE      = ApplicationFontManager.FontType_NEW.BOLD_30;

    private static final float      GRAPH_DIALOG_WIDTH  = 270;
    private static final float      GRAPH_DIALOG_HEIGHT = 180;

    private static final float COMPONENTS_PADDING = 10f, COMPONENTS_THRESHOLD_PADDING = 52f, TITLE_PADDING = 50f;
    private static final float DRAWER_SCALE = 0.6f;
    private static final float SWITCH_GRAPH_POSITION_X_SCALE = 0.30f, SWITCH_GRAPH_POSITION_Y_SCALE = 0.88f, SWITCH_GRAPH_PADDING_X = 52f, SWITCH_GRAPH_PADDING_Y = 5f;
    private static final float DIFFICULTIES_SCALE = 0.8f, DIFFICULTIES_POSITION_X_SCALE = 0.63f, DIFFICULTIES_POSITION_Y_SCALE = 0.9f;
    private static final float NOTEBOOK_POSITION_SCALE = 0.3f, NOTEBOOK_WIDTH_SCALE = 0.7f, NOTEBOOK_HEIGHT_SCALE = 0.85f;
    private static final float BOOKMARK_SIZE_SCALE = 0.6f, BOOKMARKS_POSITION_X_SCALE = 0.24f, BOOKMARKS_POSITION_Y_SCALE = 0.7f;

    public static final String                                  GFX_PATH                = "gfx/";
    public static final String                                  STATISTICS_SPEECH       = MFX_PATH + "statistics.mp3";
    private static final String[]                               BOOKMARKS_TEXTURES      = {"1", "2", "3", "2", "1", "3"};
    private static final String                                 SELECTED_SWITCH_KEY     = "selected_switch", SELECTED_GAME_KEY = "selected_game", DIFFICULTY_EASY_KEY = "diff_easy", DIFFICULTY_MEDIUM_KEY = "diff_medium", DIFFICULTY_HARD_KEY = "diff_hard";
    private static final GameDefinition                         DEFAULT_GAME            = GameDefinition.ROBBERY;
    private static final int                                    DIFFICULTY_BUTTON_SIZE  = 70;
    private static final ApplicationFontManager.FontType_NEW    DIALOG_BOLD_FONT        = ApplicationFontManager.FontType_NEW.BOLD_16;

    private Group content;
    private Image background;
    private TablexiaLabel titleLabel;
    private Image imageDrawer;
    private WidgetGroup switchGraphType;
    private Container<Group> notebook;
    private Stack graphs;
    private WidgetGroup bookmarks, difficulties;
    private GraphStyle graphStyle;
    private Set<GameDifficulty> selectedGameDifficulties;
    private GameDifficulty lastSelectedDifficulty;
    private GameDefinition selectedGameDefinition;
    private Switch switchGraph;
    private ScrollPane scrollPane;
    private float distanceScrolled;
    private Table graphPaneHolder;
    private GraphPane graphPane;
    private Map<GameDifficulty, TablexiaButton> difficultyButtons;
    private TablexiaComponentDialog dialog;

    @Override
    public boolean hasSoftBackButton() {
        return true;
    }

    @Override
    protected void screenPaused(Map<String, String> screenState) {
        screenState.put(DIFFICULTY_EASY_KEY, String.valueOf(selectedGameDifficulties.contains(GameDifficulty.EASY)));
        screenState.put(DIFFICULTY_MEDIUM_KEY, String.valueOf(selectedGameDifficulties.contains(GameDifficulty.MEDIUM)));
        screenState.put(DIFFICULTY_HARD_KEY, String.valueOf(selectedGameDifficulties.contains(GameDifficulty.HARD)));
        screenState.put(SELECTED_GAME_KEY, Integer.toString(selectedGameDefinition.getGameNumber()));
        screenState.put(SELECTED_SWITCH_KEY, String.valueOf(graphStyle == GraphStyle.DAILY ? 0 : 2));
    }

    @Override
    protected void screenLoaded(Map<String, String> screenState) {
        getStage().setDebugAll(TablexiaSettings.getInstance().isShowBoundingBoxes());
        content = new Group();

        content.addActor(background = new TablexiaNoBlendingImage(getApplicationTextureRegion(ApplicationAtlasManager.BACKGROUND_WOODEN)));
        content.addActor(titleLabel = new TablexiaLabel(getText("statistics_label"), new TablexiaLabel.TablexiaLabelStyle(TITLE_FONT_TYPE, Color.BLACK)));

        content.addActor(imageDrawer = new Image(getScreenTextureRegion(GFX_PATH + "cardfile")));
        // TODO move in action??

        graphs = new Stack();
        scrollPane = new ScrollPane(null);
        graphPaneHolder = new Table();

        selectedGameDifficulties = new HashSet<GameDifficulty>();

        switchGraph = new Switch(getScreenTextureRegion(GFX_PATH + "switch_background"), getScreenTextureRegion(GFX_PATH + "switch_left"), getScreenTextureRegion(GFX_PATH + "switch_center"), getScreenTextureRegion(GFX_PATH + "switch_right"));
        switchGraph.setSwitchSelectedListener(switchListener);
        switchGraph.setDisabledStep(1);
        switchGraphType = new WidgetGroup();
        TablexiaLabel left  = new TablexiaLabel(getText("statistics_games"), new TablexiaLabel.TablexiaLabelStyle(DEFAULT_FONT_TYPE, Color.BLACK));
        TablexiaLabel right = new TablexiaLabel(getText("statistics_average"), new TablexiaLabel.TablexiaLabelStyle(DEFAULT_FONT_TYPE, Color.BLACK));
        left.addListener(leftButtonListener);
        right.addListener(rightButtonListener);
        switchGraphType.addActor(left);
        switchGraphType.addActor(right);
        switchGraphType.addActor(switchGraph);
        switchGraph.setX(left.getWidth() + SWITCH_GRAPH_PADDING_X / 2);
        right.setX(left.getWidth() + switchGraph.getWidth() + SWITCH_GRAPH_PADDING_X);
        left.setY(SWITCH_GRAPH_PADDING_Y);
        right.setY(SWITCH_GRAPH_PADDING_Y);
        left.setAlignment(Align.bottomRight);
        right.setAlignment(Align.bottomLeft);
        content.addActor(switchGraphType);

        setSelectedGraphType(screenState);

        difficultyButtons = new HashMap<GameDifficulty, TablexiaButton>();

        float componentsPadding = isUnderScaleThreshold() ? COMPONENTS_THRESHOLD_PADDING : COMPONENTS_PADDING;
        difficulties = new HorizontalGroup().space(componentsPadding);

        for (GameDifficulty gameDifficulty : GameDifficulty.getVisibleGameDifficultyList()) {
            String diffString = gameDifficulty.name().toLowerCase();
            TablexiaButton diffButton = new TablexiaButton(null, false,
                        getScreenTextureRegion(GFX_PATH + "difficultyradio_" + diffString + "_unpressed"),
                        getScreenTextureRegion(GFX_PATH + "difficultyradio_" + diffString + "_pressed"),
                        getScreenTextureRegion(GFX_PATH + "difficultyradio_" + diffString + "_pressed"),
                        null)
                    .setButtonSize(DIFFICULTY_BUTTON_SIZE, DIFFICULTY_BUTTON_SIZE)
                    .checkable(true)
                    .setInputListener(difficultyClickListener)
                    .adaptiveSize(true);
            difficulties.addActor(diffButton);
            if(!isUnderScaleThreshold()) {
                TablexiaLabel diffLabel = new TablexiaLabel(ApplicationTextManager.getInstance().getText("gamedifficulty_" + diffString), new TablexiaLabel.TablexiaLabelStyle(DIFFICULTY_FONT_TYPE, Color.BLACK));
                difficulties.addActor(diffLabel);
            }
            diffButton.setUserObject(gameDifficulty);
            setDifficultyButtonByDifficulty(gameDifficulty, screenState, diffButton);
            difficultyButtons.put(gameDifficulty,diffButton);
        }
        difficulties.setScale(DIFFICULTIES_SCALE);
        content.addActor(difficulties);

        content.addActor(notebook = new Container<Group>());
        NinePatch chartBackground = getScreenPatch(GFX_PATH + "chart_background");
        notebook.setBackground(new NinePatchDrawable(chartBackground));
        notebook.setActor(graphs);
        notebook.fill();

        bookmarks = new VerticalGroup().space(COMPONENTS_PADDING);
        boolean gameFromDBLoaded = false;
        for (int i = 0; i < GameMenuDefinition.values().length; i++) {
            GameMenuDefinition gameMenuDefinition = GameMenuDefinition.values()[i];

            TablexiaButton button = new TablexiaButton(
                gameMenuDefinition.getTitle(),
                true,
                getScreenTextureRegion(GFX_PATH + "gamelist_listitem_background" + BOOKMARKS_TEXTURES[i] + "_inactive"),
                getScreenTextureRegion(GFX_PATH + "gamelist_listitem_background" + BOOKMARKS_TEXTURES[i] + "_active"),
                null,
                null
            );
            button.adaptiveSize(false);
            button.setChecked(false);
            button.onlyOnClick(true);
            button.getLabel().setTablexiaLabelStyle(new TablexiaLabel.TablexiaLabelStyle(BOOKMARK_FONT_TYPE, Color.BLACK));
            button.setUserObject(gameMenuDefinition);
            button.setInputListener(bookmarkClickListener);
            button.setWidth(getScreenTextureRegion(GFX_PATH + "gamelist_listitem_background" + BOOKMARKS_TEXTURES[i] + "_inactive").getRegionWidth());
            bookmarks.addActor(button);

            //if user selected some graphs on last session, select them
            if(selectGame(screenState, gameMenuDefinition.getGameDefinition()) && !gameFromDBLoaded) gameFromDBLoaded = true;
        }

        bookmarks.setScale(BOOKMARK_SIZE_SCALE);
        content.addActor(bookmarks);

        //if not loaded from DB, select first available or default game and difficulty
        if(!gameFromDBLoaded)selectFirstGameAndDifficultyAvailable();

        getStage().addActor(content);
        screenResized(0, 0);
    }

    @Override
    protected void screenVisible(Map<String, String> screenState) {
        super.screenVisible(screenState);
        datasetChanged();
    }

    @Override
    protected String prepareIntroMusicPath() {
        return STATISTICS_SPEECH;
    }

    private void selectFirstGameAndDifficultyAvailable() {

        List<Game> games;
        boolean gameSelected = false;
        for(int i = 1; i < GameMenuDefinition.values().length+1; i++) {
            games = getGames(GameDefinition.getGameDefinitionForGameNumber(i));
            if(games.size() > 0) {

                GameDifficulty difficulty;
                for(Game game : games) {
                    difficulty = GameDifficulty.getByGame(game);
                    if(difficulty.hasScore()) {
                        selectGame(GameDefinition.getGameDefinitionForGameNumber(i));
                        setDifficultyButtonChecked(difficultyButtons.get(difficulty), true, difficulty);
                        gameSelected = true;
                        break;
                    }
                }
            }
            if(gameSelected) break;
        }

        if(!gameSelected) {
            selectGame(DEFAULT_GAME);
            setDifficultyButtonChecked((TablexiaButton)difficulties.getChildren().get(0),true,GameDifficulty.DEFAULT_DIFFICULTY);
        }
    }

    private void setDifficultyButtonByDifficulty(GameDifficulty difficulty, Map<String, String> screenState, TablexiaButton button) {

        switch (difficulty) {
            case EASY:
                if(screenState.containsKey(DIFFICULTY_EASY_KEY)) setDifficultyButtonChecked(button, Boolean.parseBoolean(screenState.get(DIFFICULTY_EASY_KEY)), GameDifficulty.EASY);
                break;

            case MEDIUM:
                if(screenState.containsKey(DIFFICULTY_MEDIUM_KEY)) setDifficultyButtonChecked(button, Boolean.parseBoolean(screenState.get(DIFFICULTY_MEDIUM_KEY)), GameDifficulty.MEDIUM);
                break;

            case HARD:
                if(screenState.containsKey(DIFFICULTY_HARD_KEY)) setDifficultyButtonChecked(button, Boolean.parseBoolean(screenState.get(DIFFICULTY_HARD_KEY)), GameDifficulty.HARD);
                break;
        }
    }

    private void setDifficultyButtonChecked(TablexiaButton button, boolean selected, GameDifficulty difficulty) {
        if (button != null) {
            button.setChecked(selected);
            if(selected) selectedGameDifficulties.add(difficulty);
        }
    }

    /** used when loading selected game bookmark from DB */
    private boolean selectGame(Map<String, String> screenState, GameDefinition definition) {
        if (screenState.containsKey(SELECTED_GAME_KEY) &&
                Integer.parseInt(screenState.get(SELECTED_GAME_KEY)) == definition.getGameNumber()){
           selectGame(definition);
            return true;
        }
        return false;
    }

    /**used when selecting first available or default game */
    private void selectGame(GameDefinition definition) {
         ((TablexiaButton) bookmarks.getChildren().get(definition.getGameNumber()-1)).setChecked(true);
        selectedGameDefinition = definition;
    }

    private void setSelectedGraphType(Map<String,String> screenState) {
        if(screenState.containsKey(SELECTED_SWITCH_KEY)) graphStyle = (Integer.parseInt(screenState.get(SELECTED_SWITCH_KEY)) == 0 ? GraphStyle.DAILY : GraphStyle.AVERAGE);
        switchGraph.switchToStep(graphStyle == GraphStyle.AVERAGE ? 2 : 0);
    }

    private InputListener leftButtonListener = new ClickListenerWithSound() {
        @Override
        public void clicked(InputEvent event, float x, float y) {
            super.clicked(event, x, y);
            switchGraph.switchToStep(0);
        }
    };

    private InputListener rightButtonListener = new ClickListenerWithSound() {
        @Override
        public void onClick(InputEvent event, float x, float y) {
            switchGraph.switchToStep(2);
        }
    };

    private Switch.DragSwitchListener.SwitchSelectedListener switchListener = new Switch.DragSwitchListener.SwitchSelectedListener() {
        @Override
        public void stepSelected(int step) {
            graphStyle = (step == 0 ? GraphStyle.DAILY : GraphStyle.AVERAGE);

            Log.info(((Object) this).getClass(), "Selected graph type step " + step + " " + graphStyle);

            scrollPane.scrollTo(0, 0, 0, 0);
            datasetChanged();

        }
    };

    private InputListener difficultyClickListener = new ClickListener() {

        @Override
        public void clicked(InputEvent event, float x, float y) {
            super.clicked(event, x, y);
            GameDifficulty gameDifficulty = (GameDifficulty) event.getListenerActor().getUserObject();
            boolean selected = ((TablexiaButton) event.getListenerActor()).isChecked();

            alternateSelectedDifficulty(gameDifficulty);

            Log.info(((Object) this).getClass(), (selected ? "Selected" : "Unselected") + " difficulty " + gameDifficulty.name());

            datasetChanged();

            //When changing difficulty, leave same scrolled distance
            scrollPane.scrollTo(distanceScrolled, 0, 0, 0);
        }
    };

    private InputListener bookmarkClickListener = new ClickListener() {
        public void clicked(InputEvent event, float x, float y) {
            GameMenuDefinition gameMenuDefinition = (GameMenuDefinition) event.getListenerActor().getUserObject();

            Log.info(((Object) this).getClass(), "Selected game " + gameMenuDefinition.getGameDefinition().getTitle());
            for (Actor game : bookmarks.getChildren()) {
                ((TablexiaButton) game).setChecked(true);
                if (!game.equals(event.getListenerActor())) {
                    ((TablexiaButton) game).setChecked(false);
                }
            }

            if(selectedGameDefinition == gameMenuDefinition.getGameDefinition()) {
                return;
            }

            selectedGameDefinition = gameMenuDefinition.getGameDefinition();

            datasetChanged();

            //When changing game, scroll to start
            scrollPane.scrollTo(0, 0, 0, 0);
        }
    };

    private GameClickListener graphClickListener = new GameClickListener() {

        @Override
        public void onAverageGameClicked(InputEvent event, float x, float y, Game game, float averageScore, String dateText) {
            GameDefinition gameDefinition = GameDefinition.getGameDefinitionForGameNumber(game.getGameNumber());
            createPointDetailDialog(x,
                                    y,
                                    null,
                                    gameDefinition.getTitle(),
                                    GameDifficulty.getGameDifficultyForDifficultyNumber(game.getGameDifficulty()).getTextDescription(),
                                    dateText,
                                    String.format("%s %s",
                                                    ApplicationTextManager.getInstance().getText(GameDefinition.getGameDefinitionForGameNumber(game.getGameNumber()).getStatisticsAverageScoreDialogText()),
                                                    getCorrectedScore(selectedGameDefinition, averageScore)));
        }

        @Override
        public void onGameClicked(InputEvent event, float x, float y, final Game game) {
            GameDefinition gameDefinition = GameDefinition.getGameDefinitionForGameNumber(game.getGameNumber());
            createPointDetailDialog(x,
                                    y,
                                    new TablexiaDialogComponentAdapter() {

                                        private static final int    CUPS_TABLE_SPACING  = 2;
                                        private static final float  CUP_WIDTH_RATIO     = 1f / 6;
                                        private static final float  CUP_HEIGHT_RATIO    = 1f / 5;

                                        private List<Cell> cupCells;

                                        @Override
                                        public void prepareContent(Cell content) {
                                            Table table = new Table();
                                            table.defaults().space(CUPS_TABLE_SPACING);

                                            cupCells = new ArrayList<Cell>();
                                            content.setActor(table);

                                            TextureRegion[] cups = getRatingStars(game);

                                            for(int i = 0; i < cups.length; i++) {
                                                Image cup;
                                                cup = new Image(cups[i]);
                                                cup.setScaling(Scaling.fit);
                                                cupCells.add(table.add(cup));
                                            }

                                            resize();
                                        }

                                        @Override
                                        public void sizeChanged() {
                                            resize();
                                        }

                                        private void resize() {
                                            for (Cell cell: cupCells) {
                                                cell.width(getDialog().getWidth() * CUP_WIDTH_RATIO).height(getDialog().getHeight() * CUP_HEIGHT_RATIO);
                                            }
                                        }

                                    },
                                    gameDefinition.getTitle(),
                                    GameDifficulty.getGameDifficultyForDifficultyNumber(game.getGameDifficulty()).getTextDescription(),
                                    game.getGameStartDate(true),
                                    String.format("%s %s",
                                            ApplicationTextManager.getInstance().getText(gameDefinition.getStatisticsScoreDialogText()),
                                            getCorrectedScore(selectedGameDefinition, game)));
        }

        private void createPointDetailDialog(float x, float y, final TablexiaDialogComponentAdapter cupsComponent, String gameTitle, String gameDifficulty, String date, String score) {
            final float absoluteClickX = getGraphDialogsX(x);
            final float absoluteClickY = getGraphDialogsY(y);
            if(isPointOutsideTheView(absoluteClickX)) return;

            ArrayList<TablexiaDialogComponentAdapter> components = new ArrayList<TablexiaDialogComponentAdapter>();
            components.add(new TouchCloseDialogComponent());
            components.add(new AdaptiveSizeDialogComponent());
            components.add(new SpeechDialogComponent(
                            new Point(absoluteClickX, absoluteClickY),
                            SpeechDialogComponent.ArrowType.ARROW) {

                              /**Overriden methods, where instead of size of whole stage is used size of
                               * notebook, where are graph shown*/
                                @Override
                               public boolean isDialogOnTheLeftHalfOfScreen(float dialogX) {
                                 return absoluteClickX < (notebook.getX() + (notebook.getWidth()/2));
                                }

                               @Override
                               public boolean isDialogOnTheTopHalfOfScreen(float dialogY) {
                                   return absoluteClickY < (notebook.getY() + (notebook.getHeight()/2));
                               }

                                @Override
                              public boolean isDialogOutOfScreenLeftRight() {
                                    return  getDialog().getOutterRightX() > notebook.getX() + notebook.getWidth() - STAGE_CULLING_PADDING ||
                                            getDialog().getOutterLeftX() < notebook.getX() - STAGE_CULLING_PADDING;
                                }

                              @Override
                               public boolean isDialogOutOfScreenTopDown() {
                                  return  getDialog().getInnerTopY() > notebook.getY() + notebook.getHeight() - STAGE_CULLING_PADDING||
                                          getDialog().getOutterBottomY() < notebook.getY() - STAGE_CULLING_PADDING;
                              }
                           }
            );
            if (cupsComponent != null) {
                components.add(new ResizableSpaceContentDialogComponent());
                components.add(cupsComponent);
            }
            components.add(new ResizableSpaceContentDialogComponent());
            components.add(new TextContentDialogComponent(gameTitle, DIALOG_BOLD_FONT, false));
            components.add(new TextContentDialogComponent(String.format("%s %s", ApplicationTextManager.getInstance().getText(ApplicationTextManager.ApplicationTextsAssets.GAME_DIFFICULTY_NAME), gameDifficulty, false), DIALOG_BOLD_FONT, false));
            components.add(new TextContentDialogComponent(date, DIALOG_BOLD_FONT, false));
            components.add(new TextContentDialogComponent(score, DIALOG_BOLD_FONT, false));
            components.add(new ResizableSpaceContentDialogComponent());

            dialog = TablexiaComponentDialogFactory.getInstance().createDialog(
                    getStage(),
                    TablexiaComponentDialog.TablexiaDialogType.BUBBLE_SQUARE,
                    components.toArray(new TablexiaDialogComponentAdapter[]{}));
            dialog.show(GRAPH_DIALOG_WIDTH, GRAPH_DIALOG_HEIGHT);
        }
    };

    private TextureRegion[] getRatingStars(Game game) {

        TextureRegion[] textures = new TextureRegion[3];
        TextureRegion starAchievedTexture = getScreenTextureRegion(GFX_PATH + "ratingstar_achieved");
        TextureRegion starNotAchievedTexture = getScreenTextureRegion(GFX_PATH + "ratingstar_not_achieved");
        int starsAchieved = GameDAO.getGameResult(game).getStarCount();

        for(int i = 0; i < starsAchieved; i++) {
            textures[i] = starAchievedTexture;
        }

        for(int i = 0; i < 3 - starsAchieved; i++) {
            textures[starsAchieved + i] = starNotAchievedTexture;
        }

        return textures;
    }

    /**Checks if surface of point is seen / hidden (50% of it) */
    private boolean isPointOutsideTheView(float clickX) {
        if(clickX >  notebook.getX() + graphs.getX() + graphs.getWidth() || clickX < notebook.getX() + graphs.getX()) return true;
        return false;
    }

    /**In some games is time used as score (less means better).
    In order to calculate graph points, it is converted to 1/time.
    This function makes the opposite to show real time.
    Other scores are converted to text representing minutes and seconds */
    private String getCorrectedScore(GameDefinition gameDefinition, Game game) {
        return getCorrectedScore(gameDefinition, gameDefinition.getGameResultResolver().getGameScoreResult(game));
    }

    private String getCorrectedScore(GameDefinition gameDefinition, float score) {

        if(gameDefinition == GameDefinition.PURSUIT) return millisecondsToTimeText(score);

        return String.valueOf(Math.round(score));
    }

    private String millisecondsToTimeText(float time) {
        int minutes = (int) (time/1000) / 60;
        int seconds = (int) ((time/1000) % 60);

        if(minutes == 0) {
            return seconds + "s";
        } else if (seconds == 0){
            return minutes + "m ";
        } else {
            return minutes + "m " + seconds + "s";
        }
    }

    /**returning absolute value of clicked position on X axis on screen*/
    private float getGraphDialogsX(float clickX) {
        return notebook.getX() + graphs.getX() + (clickX - scrollPane.getScrollX()) + GraphPane.POINT_SIZE / 2;
    }

    /**returning absolute value of clicked position on Y axis on screen*/
    private float getGraphDialogsY(float clickY) {
         return notebook.getY() + graphs.getY() + clickY + GraphPane.POINT_SIZE / 2 + content.getY();
    }

    private void alternateSelectedDifficulty(GameDifficulty newDifficulty) {
        if (selectedGameDifficulties.contains(newDifficulty)) {
            selectedGameDifficulties.remove(newDifficulty);
        } else {
            selectedGameDifficulties.add(newDifficulty);
        }
        lastSelectedDifficulty = newDifficulty;
    }

    private void datasetChanged() {
        graphs.clear();
        hideDialog();

        if(selectedGameDefinition != null) {
            graphPane = drawGraph(getGames(selectedGameDefinition));

            //if is graphPane wide, put it into scrollPane
            if(graphPane.getWidth() <= GraphPane.MIN_GRAPH_WIDTH){
                distanceScrolled = 0;
                graphs.add(graphPane);
            } else {
                graphPaneHolder.clear();
                graphPaneHolder.add(graphPane).width(graphPane.getWidth()).height(graphPane.getHeight());

                distanceScrolled = scrollPane.getScrollX();

                scrollPane.setWidget(graphPaneHolder);
                scrollPane.setOverscroll(true, false);

                graphs.add(scrollPane);
            }
            position();
        }
    }

    private GraphPane drawGraph(List<Game> games) {
        graphPane = new GraphPane(games, graphStyle, this, selectedGameDefinition, selectedGameDifficulties, lastSelectedDifficulty, graphClickListener);
        lastSelectedDifficulty = null;
        return graphPane;
    }


    public void position() {
        titleLabel.setPosition(TITLE_PADDING, getStage().getHeight() - TITLE_PADDING);
        background.setBounds(0, 0, getStage().getWidth(), getStage().getHeight());
        ScaleUtil.setImageHeight(imageDrawer, getStage().getHeight() * DRAWER_SCALE);
        imageDrawer.setPosition(0, (getStage().getHeight() / 2) - (imageDrawer.getHeight() / 2));
        switchGraphType.setPosition(getStage().getWidth() * SWITCH_GRAPH_POSITION_X_SCALE, getStage().getHeight() * SWITCH_GRAPH_POSITION_Y_SCALE);
        difficulties.setPosition(getStage().getWidth() * DIFFICULTIES_POSITION_X_SCALE, getStage().getHeight() * DIFFICULTIES_POSITION_Y_SCALE);
        notebook.setBounds(getStage().getWidth() * NOTEBOOK_POSITION_SCALE, 0, getStage().getWidth() * NOTEBOOK_WIDTH_SCALE, getStage().getHeight() * NOTEBOOK_HEIGHT_SCALE);
        bookmarks.setPosition(getStage().getWidth() * BOOKMARKS_POSITION_X_SCALE, getStage().getHeight() * BOOKMARKS_POSITION_Y_SCALE);

        scrollPane.setSize(graphs.getWidth(), graphs.getHeight());
        if (graphPaneHolder.getCells().size > 0) graphPaneHolder.getCells().get(0).width(graphPane.getWidth()).height(graphs.getHeight());
        if (graphPane != null) graphPaneHolder.setSize(graphPane.getWidth(), graphPane.getHeight());
    }

    private List<Game> getGames(GameDefinition definition) {
        return GameDAO.getGamesForUserAndDefinition(getSelectedUser().getId(), definition);
    }

    private void hideDialog() {
        if(dialog != null) dialog.hide();
    }

    @Override
    public void screenResized(int width, int height) {

        if(graphPane != null) graphPane.resetHeight();
        ScaleUtil.setFullscreenBounds(content, getStage());
        position();
        hideDialog();
    }
}