package cz.nic.tablexia.screen.viewport;

import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.ui.Image;

import java.util.Map;

import cz.nic.tablexia.loader.TablexiaAbstractFileManager;
import cz.nic.tablexia.loader.TablexiaTextureManager;
import cz.nic.tablexia.screen.AbstractTablexiaScreen;

public class FullscreenImageDebugScreen extends AbstractTablexiaScreen<Void> {

    private static final String BASE_PATH = "gfx/";
    private static final String LOADER_BACKGROUND   = BASE_PATH + "screen_loader_background.jpg";
    private static final String LOADER_SMALL_HAND   = BASE_PATH + "screen_loader_smallhand.png";
    private static final String LOADER_BIG_HAND     = BASE_PATH + "screen_loader_bighand.png";

    @Override
    public boolean canBePaused() {
        return false;
    }

    @Override
    protected String prepareScreenAtlasPath(String screenAssetsPath, String screenName) {
        // no atlas loading
        return null;
    }

    @Override
    protected String prepareScreenTextResourcesAssetName() {
        // no text loading
        return null;
    }

    @Override
    protected void screenLoaded(Map<String, String> screenState) {
        TablexiaTextureManager textureManager = new TablexiaTextureManager(TablexiaAbstractFileManager.AssetsStorageType.INTERNAL);
        textureManager.loadTexture(LOADER_BACKGROUND);
        textureManager.loadTexture(LOADER_SMALL_HAND);
        textureManager.loadTexture(LOADER_BIG_HAND);

        textureManager.finishLoading();

        Image background = new Image(textureManager.getTexture(LOADER_BACKGROUND));
        background.setPosition(0, 0);
        background.setSize(getStage().getWidth(), getStage().getHeight());
        getStage().addActor(background);

        Image smallhand = new Image(textureManager.getTexture(LOADER_SMALL_HAND));
        smallhand.setPosition(getStage().getWidth() * 0.22f, getStage().getHeight() * 0.64f);
        smallhand.setSize(getStage().getWidth() * 0.01f, getStage().getHeight() * 0.06f);
        getStage().addActor(smallhand);
        smallhand.setOrigin(smallhand.getWidth() / 2, 0);
        smallhand.addAction(Actions.forever(Actions.rotateBy(-360, 6)));

        Image bighand = new Image(textureManager.getTexture(LOADER_BIG_HAND));
        bighand.setPosition(getStage().getWidth() * 0.22f, getStage().getHeight() * 0.64f);
        bighand.setSize(getStage().getWidth() * 0.01f, getStage().getHeight() * 0.06f);
        getStage().addActor(bighand);
        bighand.setOrigin(bighand.getWidth() / 2, 0);
        bighand.addAction(Actions.forever(Actions.rotateBy(-360, 0.5f)));
    }
}
