package cz.nic.tablexia.screen.viewport;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.InputListener;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Table;

import java.util.Map;

import cz.nic.tablexia.TablexiaSettings;
import cz.nic.tablexia.loader.application.ApplicationFontManager;
import cz.nic.tablexia.screen.AbstractTablexiaScreen;
import cz.nic.tablexia.util.Log;
import cz.nic.tablexia.util.ui.TablexiaLabel;

/**
 * Created by lhoracek on 4/13/15.
 */
public class ViewportDebugScreen extends AbstractTablexiaScreen<Void> {

    @Override
    protected String prepareScreenTextResourcesAssetName() {
        // no text loading
        return null;
    }

    @Override
    protected void screenLoaded(Map<String, String> screenState) {
        getStage().setDebugAll(true);

        float sceneWidth = getSceneRightX() - getSceneLeftX();
        float sceneOuterHeight = getSceneOuterTopY() - getSceneOuterBottomY();
        float sceneInnerHeight = getSceneInnerTopY() - getSceneInnerBottomY();

        Image outerScreen = new Image(getColorTextureRegion(Color.YELLOW));
        outerScreen.setSize(sceneWidth, sceneOuterHeight);
        outerScreen.setPosition(getSceneLeftX(), getSceneOuterBottomY());
        getStage().addActor(outerScreen);

        Image innerScreen = new Image(getColorTextureRegion(Color.RED));
        innerScreen.setSize(sceneWidth, sceneInnerHeight);
        innerScreen.setPosition(getSceneLeftX(), getSceneInnerBottomY());
        getStage().addActor(innerScreen);

        int size = 10;
        for (int i = 0; i < size; i++) {
            for (int j = 0; j < size; j++) {
                TextureRegion checker = getScreenTextureRegion("checker");
                Image check = new Image(checker);
                check.setPosition(500 - (check.getWidth() * size / 2) + (i * check.getWidth()), TablexiaSettings.getMinWorldHeight() / 2 - (check.getWidth() * size / 2) + (j * check.getWidth()));
                getStage().addActor(check);
            }
        }

        getStage().addListener(new InputListener() {
            Image point;

            @Override
            public boolean touchDown(InputEvent event, float x, float y, int pointer, int button) {
                Log.debug(getClass(), "Touchdown");
                point = new Image(getScreenTextureRegion("point"));
                getStage().addActor(point);
                point.setPosition(x - point.getWidth() / 2, y - point.getHeight() / 2);
                return true;
            }


            @Override
            public void touchDragged(InputEvent event, float x, float y, int pointer) {
                if (point != null) {
                    point.setPosition(x - point.getWidth() / 2, y - point.getHeight() / 2);
                    Log.debug(getClass(), "Touch coords " + (x - point.getWidth() / 2) + ":" + (y - point.getHeight() / 2));
                }
            }

            @Override
            public void touchUp(InputEvent event, float x, float y, int pointer, int button) {
                if (point != null) {
                    Log.debug(getClass(), "Liftoff");
                    point.remove();
                    point = null;
                }
            }
        });

        Table t = new Table();
        for(ApplicationFontManager.FontType_NEW fnt : ApplicationFontManager.FontType_NEW.values()) {
            t.add().setActor(new TablexiaLabel("ABCDEFGH - tohle je Teeeest 123123123....", new TablexiaLabel.TablexiaLabelStyle(fnt, Color.BLACK)));
            t.row();
        }
        t.setPosition(400, getSceneInnerTopY());
        getStage().addActor(t);
    }
}
