package cz.nic.tablexia.sync.work;

import com.badlogic.gdx.Net;
import com.badlogic.gdx.net.HttpRequestHeader;
import com.badlogic.gdx.net.HttpStatus;

import cz.nic.tablexia.model.game.GameDAO;
import cz.nic.tablexia.model.screen.ScreenDAO;
import cz.nic.tablexia.shared.model.User;
import cz.nic.tablexia.sync.RestSynchronizationService;
import cz.nic.tablexia.util.Log;

/**
 * Created by frantisek on 15.1.16.
 */
public class ConfirmDataReceived extends SyncWork {

    public ConfirmDataReceived(User user) {
        setUser(user);
    }

    @Override
    public String getUrl() {
        return RestSynchronizationService.getConfirmUserUrl(getUser().getUuid());
    }

    @Override
    public void send(RestSynchronizationService sync) {
        sync.setHeader(HttpRequestHeader.ContentType, "application/json");
        sync.send();
    }

    @Override
    protected void onResponse(Net.HttpResponse httpResponse) {
        GameDAO.markGamesAsSync(getUser().getGames());
        ScreenDAO.markScreensAsSync(getUser().getScreens());
        Log.debug(this.getClass(), String.format("Marking %d of games as synchronized", getUser().getGames() != null ? getUser().getGames().size() : 0));
        RestSynchronizationService.doSyncWork(new DownloadUser(getUser().getUuid()));
    }

    @Override
    protected int getExpectedSuccessResponseCode() {
        return HttpStatus.SC_OK;
    }
}
