package cz.nic.tablexia.sync.work;

import com.badlogic.gdx.Net;
import com.badlogic.gdx.net.HttpRequestHeader;
import com.badlogic.gdx.net.HttpStatus;
import com.badlogic.gdx.utils.Json;

import cz.nic.tablexia.game.ranksystem.UserRankManager;
import cz.nic.tablexia.model.UserDAO;
import cz.nic.tablexia.shared.model.User;
import cz.nic.tablexia.sync.RestSynchronizationService;
import cz.nic.tablexia.util.Log;

/**
 * Created by frantisek on 18.1.16.
 */
public class DownloadUser extends SyncWork {

    private final String uuid;

    public DownloadUser(String uuid) {
        this.uuid = uuid;
    }

    @Override
    public String getUrl() {
        return RestSynchronizationService.getDownloadUserUrl(uuid);
    }

    @Override
    public void send(RestSynchronizationService sync) {
        sync.setHeader(HttpRequestHeader.ContentType, "application/json");
        sync.send();
    }

    @Override
    protected void onResponse(Net.HttpResponse httpResponse) {
        String userJson = httpResponse.getResultAsString();
        Json json = new Json();
        json.setIgnoreUnknownFields(true);
        User user = json.fromJson(User.class, userJson);

        if (user == null) {
            Log.err(this.getClass(), "Failed to deserialize user from JSON");
            return;
        }

        long start = System.currentTimeMillis();
        user.setId(UserDAO.importUser(user));
        long finish = System.currentTimeMillis();

        setUser(user);

        UserRankManager.getInstance().forceRefreshUserRank(user);

        Log.debug(this.getClass(), "Duration: " + getTime(start, finish));
    }

    @Override
    protected int getExpectedSuccessResponseCode() {
        return HttpStatus.SC_OK;
    }
}
