package cz.nic.tablexia.util;

import com.badlogic.gdx.graphics.Pixmap;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.utils.reflect.ClassReflection;
import com.badlogic.gdx.utils.reflect.ReflectionException;
import com.google.common.base.Splitter;

import java.util.Map;

import cz.nic.tablexia.screen.AbstractTablexiaScreen;

public class Utility {

    public static AbstractTablexiaScreen<?> getScreenForScreenClass(Class<? extends AbstractTablexiaScreen<?>> screenClass, Map<String, String> initialState) {
        try {
            AbstractTablexiaScreen<?> abstractTablexiaScreen = (AbstractTablexiaScreen<?>) ClassReflection.getConstructors(screenClass)[0].newInstance(new Object[]{});
            if (initialState != null) {
                abstractTablexiaScreen.setInitialState(initialState);
            }
            return abstractTablexiaScreen;
        } catch (ReflectionException e) {
            Log.err(Utility.class, "Cannot instantiate screen class: " + screenClass + " ! (Do you have only one parameter less constructor in screen class?)", e);
        }
        return null;
    }

    public static Map<String, String> createChecksumMapFromString(String checksums) {
        if (checksums == null) {
            return null;
        }
        return Splitter.on(',').trimResults().withKeyValueSeparator(":").split(checksums);
    }

    public static String transformLocalAssetsPath(String localAssetsPath) {
        return localAssetsPath.replace('/', '.');
    }

    public static int[][] createColorMap(TextureRegion clickmap) {
        int clickmapWidth = clickmap.getRegionWidth();
        int clickmapHeight = clickmap.getRegionHeight();

        Pixmap pixmap = clickmap.getTexture().getTextureData().consumePixmap();

        int [][] colors = new int[clickmapWidth][clickmapHeight];
        for (int i = 0; i < clickmapWidth; i++) {
            for (int j = 0; j < clickmapHeight; j++) {
                colors[i][j] = pixmap.getPixel(clickmap.getRegionX() + i, clickmap.getRegionY() + j);
            }
        }

        pixmap.dispose();
        return colors;
    }

    public static int[][] createColorMap(Texture clickmap) {
        return createColorMap(new TextureRegion(clickmap));
    }

}
