package cz.nic.tablexia.util.assetmanager;

import com.badlogic.gdx.files.FileHandle;

import java.util.HashMap;

import cz.nic.tablexia.loader.TablexiaAbstractFileManager;
import cz.nic.tablexia.util.Log;

/**
 * Created by lhoracek on 4/3/15.
 */
public class DirectoryAsset extends HashMap<String, String> {
    private final String basePath;

    public DirectoryAsset(String path){
        basePath = path;
        loadFilesInDir(basePath);
    }

    private void loadFilesInDir(String path) {
        for (FileHandle entry : getDirHandle(path).list()) {
            loadFiles("", entry);
        }
    }

    private void loadFiles(String path, FileHandle fh) {
        if (fh.isDirectory()) {
            Log.debug(getClass(), "Is directory: " + fh.name());
            for (FileHandle entry : fh.list()) {
                loadFiles(path + fh.name() + "/", entry);
            }
        } else {
            add(path, fh);
        }
    }

    private FileHandle getDirHandle(String path) {
        FileHandle dirHandle;
        dirHandle = TablexiaAbstractFileManager.getFileStoragePathFileHandle(TablexiaAbstractFileManager.AssetsStorageType.EXTERNAL, path);
        return dirHandle;
    }

    public String add(String path, FileHandle fh) {
        String fullPath = basePath + "/" + path + fh.name();
        String key = path + fh.nameWithoutExtension();
        Log.debug(getClass(), "Adding asset: " + key + " : " + fullPath);
        return super.put(key, fullPath);
    }

    @Override
    public String get(Object key) {
        String value = super.get(key);
        if(value == null){
            Log.err(getClass(), "Asset not prepared: " + key);
            throw new IllegalArgumentException("Asset not prepared: " + key);
        }
        return value;
    }
}
