package cz.nic.tablexia.util.listener;

import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.scenes.scene2d.Action;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.InputListener;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;

import java.util.LinkedHashMap;

import cz.nic.tablexia.game.games.pursuit.action.RotateAndMovePieceInPosition;
import cz.nic.tablexia.game.games.pursuit.action.RotatePieceToClosestAngle;
import cz.nic.tablexia.game.games.pursuit.helper.ArithmeticsHelper;
import cz.nic.tablexia.game.games.pursuit.model.Grid;
import cz.nic.tablexia.game.games.pursuit.model.PuzzlePiece;
import cz.nic.tablexia.util.Log;
import cz.nic.tablexia.util.Point;

/**
 * Created by Vaclav Tarantik on 6/18/15.
 */
public class DragAndRotateActorListener extends InputListener {
    private static final int DRAGGED_PIECE_ZINDEX = 25;
    public static final float ROTATION_ANIMATION_DURATION = 0.3f;
    public static final float MOVETO_ANIMATION_DURATION = 0.2f;
    private float grabX, grabY;
    private float initialFingersAngle;
    private Actor draggedActor;
    private Actor parentActor;
    private IOnRotationFinished iOnRotationFinished;
    private float actorOriginalRotation;
    private boolean performingAction;
    private boolean pieceAnimationDone;
    private RotateAndMovePieceInPosition pieceTranslationAction;

    private LinkedHashMap<Integer, Point> activePointers;

    public DragAndRotateActorListener(IOnRotationFinished iOnRotationFinished) {
        this.iOnRotationFinished = iOnRotationFinished;
        activePointers = new LinkedHashMap<Integer, Point>();
    }

    public void selected(float x, float y) {
    }

    public void moved(float x, float y, Actor draggedPiece) {
    }

    public void dropped(float x, float y, Actor draggedPiece) {
    }

	public void rotated(float angle, Actor rotatedPiece) {
	}

    @Override
    public boolean touchDown(InputEvent event, float x, float y, int pointer, int button) {
        if (button != 1) {
            activePointers.put(pointer, new Point(x, y));
        }
        if (activePointers.size() == 1) {
            selected(x, y);
        }
        if (draggedActor != null && !performingAction) {
            //mouse right button clicked
            if (button == 1) {
                draggedActor.setRotation((draggedActor.getRotation() + 90) % 360);
				rotated((draggedActor.getRotation() + 90), draggedActor);
            }
            if (activePointers.size() == 1) {
                draggedActor.setZIndex(DRAGGED_PIECE_ZINDEX);
                draggedActor.setScale(1.2f);
                Vector2 recalculatedCoords = new Vector2(x, y);
                draggedActor.stageToLocalCoordinates(recalculatedCoords);
                grabX = recalculatedCoords.x;
                grabY = recalculatedCoords.y;

            } else if (activePointers.size() == 2) {
                initialFingersAngle = (float) ArithmeticsHelper.getAngleBetweenTwoPoints(activePointers.get(activePointers.keySet().toArray()[0]), activePointers.get(activePointers.keySet().toArray()[1]));
                actorOriginalRotation = draggedActor.getRotation();
                Log.info(getClass(), "Actors initial angle: " + actorOriginalRotation);
            }
        }
        return true;
    }

    @Override
    public void touchDragged(InputEvent event, float x, float y, int pointer) {
        super.touchDragged(event, x, y, pointer);
        if (draggedActor != null && !performingAction) {
            if (activePointers.size() == 1) {
                //updating current pointer position
                activePointers.put(pointer, new Point(x, y));
                Vector2 recalculatedCoords = new Vector2(x, y);
                draggedActor.stageToLocalCoordinates(recalculatedCoords);
                float bx = draggedActor.getX() + (recalculatedCoords.x - grabX);
                float by = draggedActor.getY() + (recalculatedCoords.y - grabY);
                draggedActor.setPosition(bx, by);
                moved(bx, by, draggedActor);
            } else if (activePointers.size() > 1) {
                //not updating the first finger coordinates when rotating so the piece does not move
                activePointers.put(pointer, new Point(x, y));
                Point firstFingerCoordinates = activePointers.get(activePointers.keySet().toArray()[0]);
                Point secondFingerCoordinates = activePointers.get(activePointers.keySet().toArray()[1]);
                double angle = ArithmeticsHelper.getAngleBetweenTwoPoints(firstFingerCoordinates, secondFingerCoordinates);
				float actorRotation = draggedActor.getRotation() + (float) (angle - initialFingersAngle);
                draggedActor.setRotation(actorRotation);
                initialFingersAngle = (float) angle;

				rotated(actorRotation - actorOriginalRotation, draggedActor);
            }
        }
    }

    @Override
    public void touchUp(InputEvent event, final float x, final float y, int pointer, int button) {
        if (button != 1) {
            if (draggedActor != null && !performingAction && activePointers.size()>=1) {
                //storing original fingers positions and deciding which finger was released
                boolean draggingFingerLifted = ((Integer) (activePointers.keySet().toArray()[0]) == pointer);
                boolean rotatingFingerLifted = false;
                if (activePointers.size() > 1) {
                    rotatingFingerLifted = (Integer) activePointers.keySet().toArray()[1] == pointer;
                }
                //removing lifted finger pointer after storing its position
                activePointers.remove(pointer);
                if (activePointers.size() == 0) {
                    //if user removed all fingers before animation finished, piece is stopped and animated back to its original position
                    if (!isPieceAnimationDone()) {
                        if (pieceTranslationAction != null) {
                            pieceTranslationAction.interrupt();
                        }
                        Point pieceOriginalPosition = ((Grid) parentActor).getPieceGridPositionInPixels(((PuzzlePiece) draggedActor).getActualPosition());
                        parentActor.addAction(new RotateAndMovePieceInPosition((PuzzlePiece) draggedActor, actorOriginalRotation, pieceOriginalPosition, iOnRotationFinished));
                    }
                    parentActor.addAction(Actions.after(new Action() {
                        @Override
                        public boolean act(float delta) {
                            dropped(x, y, draggedActor);
                            return true;
                        }
                    }));

                } else if (activePointers.size() > 0) {
                    //if user raised the dragging finger, animate piece under second finger
                    Point currentFirstPointer = activePointers.get(activePointers.keySet().toArray()[0]);
                    if (draggingFingerLifted) {
                        setPieceAnimationDone(false);
                        Vector2 recalculatedCoords = new Vector2(currentFirstPointer.x, currentFirstPointer.y);
                        draggedActor.stageToLocalCoordinates(recalculatedCoords);
                        float bx = draggedActor.getX() + (recalculatedCoords.x - (draggedActor.getWidth() / 2));
                        float by = draggedActor.getY() + (recalculatedCoords.y - (draggedActor.getHeight() / 2));
                        pieceTranslationAction = new RotateAndMovePieceInPosition((PuzzlePiece) draggedActor, actorOriginalRotation, new Point(bx, by), iOnRotationFinished);
						rotated(ArithmeticsHelper.getAngleInRotationDirection(actorOriginalRotation, draggedActor.getRotation()), draggedActor);
                        parentActor.addAction(pieceTranslationAction);
                    } else if (rotatingFingerLifted) {
                        parentActor.addAction(new RotatePieceToClosestAngle((PuzzlePiece) draggedActor, actorOriginalRotation, iOnRotationFinished));
						rotated(ArithmeticsHelper.getAngleInRotationDirection(actorOriginalRotation, draggedActor.getRotation()), draggedActor);
                    }
                }
            } else {
                activePointers.remove(pointer);
            }
        }

    }

    public interface IOnRotationFinished {
        void onDragAndRotationFinished();
    }

    public void setDraggedActor(Actor actor) {
        this.draggedActor = actor;
    }

    public Actor getDraggedActor() {
        return draggedActor;
    }

    public void setPerformingAction(boolean performingAction) {
        this.performingAction = performingAction;
    }

    public void setParentActor(Actor parentActor) {
        this.parentActor = parentActor;
    }

    public void setPieceAnimationDone(boolean pieceAnimationDone) {
        this.pieceAnimationDone = pieceAnimationDone;
    }

    public boolean isPieceAnimationDone() {
        return pieceAnimationDone;
    }
}
