package cz.nic.tablexia.util.ui;

import com.badlogic.gdx.graphics.g2d.Animation;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.utils.TextureRegionDrawable;

/**
 * Created by Matyáš Latner.
 */
public class AnimatedImage extends Image {

    protected   Animation   animation;
    private     float       stateTime = 0;

    private     boolean     started;
    private     boolean     loop;
    private     Runnable    finishHandler;

    public AnimatedImage(Animation animation, boolean started) {
        super(animation.getKeyFrame(0));
        this.animation = animation;
        this.started = started;
        this.loop = false;
    }

    public void startAnimation(Runnable finishHandler) {
        this.finishHandler = finishHandler;
        stateTime = 0;
        started = true;
        loop = false;
    }

    public void startAnimationLoop() {
        stateTime = 0;
        started = true;
        loop = true;
    }

    public void stopAnimation() {
        started = false;
        loop = false;
    }

    @Override
    public void act(float delta) {
        if (started) {
            stateTime = stateTime + delta;
            if (loop) {
                ((TextureRegionDrawable)getDrawable()).setRegion(animation.getKeyFrame(stateTime += delta, true));
            } else {
                ((TextureRegionDrawable)getDrawable()).setRegion(animation.getKeyFrame(stateTime, false));
                if (finishHandler != null && animation.isAnimationFinished(stateTime)) {
                    finishHandler.run();
                }
            }
        } else {
            ((TextureRegionDrawable)getDrawable()).setRegion(animation.getKeyFrame(0));
        }
        super.act(delta);
    }
}
