package cz.nic.tablexia.util.ui;

import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;

/**
 * Created by drahomir on 4/4/16.
 */
public class ScrollPaneWithBorders extends ScrollPane{

    private float lastScrollX;
    private Image leftBorder, rightBorder;

    private boolean singleBorder = false;

    public ScrollPaneWithBorders(Actor widget, Image border) {
        this(widget, border, null);
    }

    public ScrollPaneWithBorders(Actor widget, Image leftBorder, Image rightBorder) {
        super(widget);
        lastScrollX = getVisualScrollX();

        this.leftBorder = leftBorder;
        this.rightBorder = rightBorder;

        leftBorder.setVisible(false);

        if(rightBorder != null) {
            rightBorder.setVisible(false);
        }
        else {
            singleBorder = true;
            this.rightBorder = leftBorder;
        }
    }

    @Override
    public void act(float delta) {
        super.act(delta);

        if(lastScrollX != getVisualScrollX()) {
            lastScrollX = getVisualScrollX();

            if(isRightEdge()) {
                rightBorder.setVisible(true);
                float posX = getX() + getWidth() - (getVisualScrollX() - (getWidget().getWidth() - getWidth()));
                rightBorder.setPosition(MathUtils.floor(posX), getY());
                rightBorder.setSize(rightBorder.getWidth(), getHeight());
            }
            else if(isLeftEdge()) {
                leftBorder.setVisible(true);
                float posX = getX() - leftBorder.getWidth() - getVisualScrollX();
                leftBorder.setPosition(MathUtils.ceil(posX), getY());
                leftBorder.setSize(leftBorder.getWidth(), getHeight());
            }
            else {
                leftBorder.setVisible(false);
                rightBorder.setVisible(false);
            }
        }
    }

    @Override
    public void draw(Batch batch, float parentAlpha) {
        super.draw(batch, parentAlpha);

        if(getLeftBorder().isVisible())
            getLeftBorder().draw(batch, parentAlpha);

        if(getRightBorder().isVisible())
            getRightBorder().draw(batch, parentAlpha);
    }

    public boolean isSingleBorder() {
        return singleBorder;
    }

    public Image getLeftBorder() {
        return leftBorder;
    }

    public Image getRightBorder() {
        return rightBorder;
    }
}
