package cz.nic.tablexia.util.ui;

import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.scenes.scene2d.Actor;

public class TablexiaProgressBar extends Actor {
    private float fillOffsetX;
    private float fillOffsetY;

    private float percent;
    private float visualPercent;
    private float speed;
    private boolean smooth;

    private TextureRegion progressbar;
    private TextureRegion fill;

    public TablexiaProgressBar(TextureRegion progressbar, TextureRegion fill) {
        this.progressbar = progressbar;
        this.fill = fill;

        setWidth(progressbar.getRegionWidth());
    }

    @Override
    public void setWidth(float width) {
        float height = width * (progressbar.getRegionHeight() / (float) progressbar.getRegionWidth());

        super.setSize(width, height);

        float scale_x = width / progressbar.getRegionWidth();
        float scale_y = height / progressbar.getRegionHeight();

        fillOffsetX = (this.progressbar.getRegionWidth()  - this.fill.getRegionWidth()) / 2 * scale_x;
        fillOffsetY = (this.progressbar.getRegionHeight() - this.fill.getRegionHeight()) / 2 * scale_y;
    }

    @Override
    public void setHeight(float height) {}

    @Override
    public void setBounds(float x, float y, float width, float height) {
        setPosition(x, y);
        setWidth(width);
    }

    @Override
    @Deprecated
    public void setSize(float width, float height) {
        setWidth(width);
    }

    @Override
    public void act(float delta) {
        super.act(delta);

        if(smooth) {
            if (visualPercent < percent) {
                visualPercent += 1/speed * delta;

                if (visualPercent > percent) visualPercent = percent;
            } else if (visualPercent > percent) {
                visualPercent -= 1/speed * delta;

                if (visualPercent < percent) visualPercent = percent;
            }
        }
    }

    public void setPercent(float percent) {
        this.percent = MathUtils.clamp(percent, 0, 1);

        if(!smooth)
            visualPercent = this.percent;
    }

    public void setSmooth(boolean smooth) {
        this.smooth = smooth;
    }

    public void setSpeed(float speed) {
        this.speed = speed;
    }

    @Override
    public void draw(Batch batch, float parentAlpha) {
        float origAlpha = batch.getColor().a;

        batch.setColor(batch.getColor().r, batch.getColor().g, batch.getColor().b, parentAlpha);

        if(visualPercent != 0) {
            int origRegionWidth = fill.getRegionWidth();
            fill.setRegionWidth((int) (fill.getRegionWidth() * visualPercent));
            batch.draw(
                    fill,
                    getX() + fillOffsetX,
                    getY() + fillOffsetY,
                    (getWidth() - 2 * fillOffsetX) * visualPercent,
                    getHeight() - 2 * fillOffsetY
            );
            fill.setRegionWidth(origRegionWidth);
        }

        batch.draw(progressbar, getX(), getY(), getWidth(), getHeight());

        batch.setColor(batch.getColor().r, batch.getColor().g, batch.getColor().b, origAlpha);
    }

    public float getFillOffsetX() {
        return fillOffsetX;
    }

    public float getFillOffsetY() {
        return fillOffsetY;
    }
}