package cz.nic.tablexia.util.ui.button;

import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.utils.Align;

import cz.nic.tablexia.util.ScaleUtil;
import cz.nic.tablexia.util.ui.TablexiaLabel;

/**
 * Created by frantisek on 18.2.16.
 */
public class ImageTablexiaButton extends StandardTablexiaButton {
	private static final int	DEFAULT_ICON_ALIGN			= Align.left;
	private static final float 	DEFAULT_IMAGE_HEIGHT_RATIO 	= 2f / 4;
	private static final int 	IMAGE_CONTENT_SPACING 		= 20;

	private int					iconAlign 					= DEFAULT_ICON_ALIGN;
	private final Group 		ICON_LAYER;
	private Image				icon;

	public ImageTablexiaButton(String text, Image icon, int align) {
		super(text);
		ICON_LAYER = new Group();
		init(icon, align);
	}

	public ImageTablexiaButton(String text, Image icon) {
		this(text, icon, DEFAULT_ICON_ALIGN);
	}

	public ImageTablexiaButton(String text, Image icon, int align, TablexiaButtonType tablexiaButtonType) {
		this(text, icon, align, tablexiaButtonType, false);
	}

	public ImageTablexiaButton(String text, Image icon, TablexiaButtonType tablexiaButtonType, boolean wrapText) {
		this(text, icon, DEFAULT_ICON_ALIGN, tablexiaButtonType, wrapText);
	}

	public ImageTablexiaButton(String text, Image icon, int align, TablexiaButtonType tablexiaButtonType, boolean wrapText) {
		super(text, tablexiaButtonType, wrapText);
		ICON_LAYER = new Group();
		init(icon, align);
	}

	private void init(Image icon, int align) {
		addActor(ICON_LAYER);
		setIcon(icon, align);
	}


	public TablexiaButton setIcon(Image icon) {
		return setIcon(icon, iconAlign);
	}

	public TablexiaButton setIcon(Image icon, int align) {
		this.icon = icon;
		this.iconAlign = align;

		ICON_LAYER.clear();
		if (icon == null) {
			return this;
		}

		ICON_LAYER.addActor(icon);
		adjustBounds();
		return this;
	}

	private TablexiaButton setSizeForIcon(Image icon, int align) {
		ScaleUtil.setImageHeight(icon, getInnerHeight() * DEFAULT_IMAGE_HEIGHT_RATIO);
		switch (align) {
			case Align.left:
				icon.setX(getLabel().getX() + ((getLabel().getWidth() - getLabel().getPrefWidth())/2) - icon.getWidth() - IMAGE_CONTENT_SPACING / 2);
				break;
			case Align.right:
				icon.setX(getLabel().getX() + getLabel().getWidth() - (getLabel().getWidth() - getLabel().getPrefWidth())/2 + IMAGE_CONTENT_SPACING / 2);
				break;
		}
		icon.setY(getActualPatch().getPadBottom() + (getInnerHeight() - icon.getHeight()) / 2);
		return this;
	}

	@Override
	protected void adjustBounds() {
		if (icon != null) {
			setSizeForIcon(icon, iconAlign);
		}
		super.adjustBounds();
	}

	@Override
	public float getContentWidth() {
		float iconWidth = 0;
		if (icon != null) {
			iconWidth = icon.getDrawable().getMinWidth();
		}
		return super.getContentWidth() + iconWidth + IMAGE_CONTENT_SPACING;
	}

	@Override
	protected void prepareTextSize(TablexiaLabel label) {
		super.prepareTextSize(label);

		float iconWidth = 0;
		if (icon != null) {
			iconWidth = icon.getWidth();
		}

		label.setWidth(label.getWidth() - iconWidth / 2 - IMAGE_CONTENT_SPACING / 2);
	}

	@Override
	protected void prepareTextPosition(TablexiaLabel label) {
		float iconWidth = 0;
		if (icon != null) {
			iconWidth = icon.getWidth();
		}

		switch (iconAlign) {
			case Align.left:
				label.setPosition(getActualPatch().getPadLeft(), getActualPatch().getPadBottom());
				label.setPosition(label.getX() + iconWidth / 2 + IMAGE_CONTENT_SPACING / 2, label.getY());
				break;

			case Align.right:
				label.setPosition(getWidth() - label.getWidth() - getActualPatch().getPadRight(), getActualPatch().getPadBottom());
				label.setPosition(label.getX() - iconWidth / 2 - IMAGE_CONTENT_SPACING / 2, label.getY());
				break;
		}

	}
}
