package cz.nic.tablexia.util.ui.dialog.components;

import com.badlogic.gdx.scenes.scene2d.InputListener;
import com.badlogic.gdx.scenes.scene2d.ui.Image;

import cz.nic.tablexia.loader.application.ApplicationInternalTextureManager;
import cz.nic.tablexia.loader.application.ApplicationTextManager;
import cz.nic.tablexia.util.ui.button.StandardTablexiaButton;

/**
 * Created by Matyáš Latner.
 */
public class PositiveNegativeButtonContentDialogComponent extends TwoButtonContentDialogComponent {

    private static final PositiveNegativeButtonType DEFAULT_POSITIVE_NEGATIVE_BUTTON_TYPE = PositiveNegativeButtonType.YES_NO;

    public enum PositiveNegativeButtonType {
        YES_NO              (ApplicationTextManager.ApplicationTextsAssets.SYSTEM_YES,      ApplicationTextManager.ApplicationTextsAssets.SYSTEM_NO),
        CONFIRM_DECLINE     (ApplicationTextManager.ApplicationTextsAssets.SYSTEM_CONFIRM,  ApplicationTextManager.ApplicationTextsAssets.SYSTEM_DECLINE),
        AGAIN_EXIT          (ApplicationTextManager.ApplicationTextsAssets.SYSTEM_RETRY,    ApplicationTextManager.ApplicationTextsAssets.SYSTEM_EXIT),
		AGAIN_BACK          (ApplicationTextManager.ApplicationTextsAssets.SYSTEM_RETRY,    ApplicationTextManager.ApplicationTextsAssets.SYSTEM_BACK);

        private final String positiveText;
        private final String negativeText;

        PositiveNegativeButtonType(String positiveText, String negativeText) {
            this.positiveText = positiveText;
            this.negativeText = negativeText;
        }

        public String getPositiveText() {
            return positiveText;
        }

        public String getNegativeText() {
            return negativeText;
        }
    }

    public PositiveNegativeButtonContentDialogComponent(InputListener yesInputListener) {
        this(yesInputListener, DEFAULT_POSITIVE_NEGATIVE_BUTTON_TYPE);
    }

    public PositiveNegativeButtonContentDialogComponent(InputListener yesInputListener, PositiveNegativeButtonType positiveNegativeButtonType) {
        this(yesInputListener, null, positiveNegativeButtonType);
    }

    public PositiveNegativeButtonContentDialogComponent(InputListener yesInputListener, InputListener noInputListener) {
        this(yesInputListener, noInputListener, DEFAULT_POSITIVE_NEGATIVE_BUTTON_TYPE);
    }

    public PositiveNegativeButtonContentDialogComponent(InputListener yesInputListener, boolean yesButtonUseOnce, InputListener noInputListener, boolean noButtonUseOnce) {
        this(yesInputListener, yesButtonUseOnce, noInputListener, noButtonUseOnce, DEFAULT_POSITIVE_NEGATIVE_BUTTON_TYPE);
    }    

    public PositiveNegativeButtonContentDialogComponent(InputListener yesInputListener, boolean yesButtonUseOnce, InputListener noInputListener, boolean noButtonUseOnce, PositiveNegativeButtonType positiveNegativeButtonType) {
        super(  ApplicationTextManager.getInstance().getText(positiveNegativeButtonType.getNegativeText()),
                ApplicationTextManager.getInstance().getText(positiveNegativeButtonType.getPositiveText()),
                new Image(ApplicationInternalTextureManager.getInstance().getTexture(ApplicationInternalTextureManager.BUTTON_NO_ICON)),
                new Image(ApplicationInternalTextureManager.getInstance().getTexture(ApplicationInternalTextureManager.BUTTON_YES_ICON)),
                StandardTablexiaButton.TablexiaButtonType.RED,
                yesButtonUseOnce,
                StandardTablexiaButton.TablexiaButtonType.GREEN,
                noButtonUseOnce,
                noInputListener,
                yesInputListener
                );
    }

    public PositiveNegativeButtonContentDialogComponent(InputListener yesInputListener, InputListener noInputListener, PositiveNegativeButtonType positiveNegativeButtonType) {
        super(  ApplicationTextManager.getInstance().getText(positiveNegativeButtonType.getNegativeText()),
                ApplicationTextManager.getInstance().getText(positiveNegativeButtonType.getPositiveText()),
                new Image(ApplicationInternalTextureManager.getInstance().getTexture(ApplicationInternalTextureManager.BUTTON_NO_ICON)),
                new Image(ApplicationInternalTextureManager.getInstance().getTexture(ApplicationInternalTextureManager.BUTTON_YES_ICON)),
                StandardTablexiaButton.TablexiaButtonType.RED,
                StandardTablexiaButton.TablexiaButtonType.GREEN,
                noInputListener,
                yesInputListener
                );
    }
}