package cz.nic.tablexia.util.ui.dialog.components;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.InputListener;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.ui.Cell;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.utils.Drawable;
import com.badlogic.gdx.utils.Align;

import java.util.Timer;
import java.util.TimerTask;

import cz.nic.tablexia.TablexiaSettings;
import cz.nic.tablexia.loader.application.ApplicationAtlasManager;
import cz.nic.tablexia.loader.application.ApplicationFontManager;
import cz.nic.tablexia.util.ui.ComponentScaleUtil;
import cz.nic.tablexia.util.ui.TextFieldWithPlaceholder;
import cz.nic.tablexia.util.ui.dialog.TablexiaComponentDialog;

/**
 * Created by Matyáš Latner.
 */
public class TextFiledDialogComponent extends TablexiaDialogComponentAdapter {
    private static final Color                                  DEFAULT_FONT_COLOR                  = Color.BLACK;
    private static final ApplicationFontManager.FontType_NEW    DEFAULT_FONT_TYPE                   = ApplicationFontManager.FontType_NEW.REGULAR_20;
    private static final int                                    FONT_SIZE_RESIZE_THRESHOLD_STEP     = 10;
    private static final String                                 DEFAULT_PLACEHOLDER_TEXT            = "";
    private static final int                                    DEFAULT_TEXTFIELD_ALIGN             = Align.center;
	private static final float									DEFAULT_TEXT_FIELD_WIDTH_PERCENT	= 1;
    
    public static final int                                     DIALOG_REPLACE_TIMEOUT = 7000;

    private boolean                                         isScaled;
    private final Color                                     fontColor;
    private Integer                                         maximumLength;
    private String                                          placeholderText;
    private ApplicationFontManager.FontType_NEW             fontType;
    private ApplicationFontManager.FontType_NEW             actualFontType;
	private float 											textFieldWidthPercent 	= DEFAULT_TEXT_FIELD_WIDTH_PERCENT;

    private TextFieldWithPlaceholder                        textField;
    private Cell                                            content;

    private Timer                                           timer;
    private boolean                                         timerSet;

    private InputListener                                   repositionOnTypeListener, repositionOnTouchListener;

    public TextFiledDialogComponent() {
        this(null, DEFAULT_PLACEHOLDER_TEXT, DEFAULT_FONT_TYPE, DEFAULT_FONT_COLOR);
    }

    public TextFiledDialogComponent(String placeholderText) {
        this(null, placeholderText);
    }

    public TextFiledDialogComponent(Integer maximumLength, String placeholderText) {
        this(maximumLength, placeholderText, DEFAULT_FONT_TYPE, DEFAULT_FONT_COLOR);
    }

    public TextFiledDialogComponent(Integer maximumLength, String placeholderText, ApplicationFontManager.FontType_NEW fontType, Color fontColor) {
        this.placeholderText    = placeholderText;
        this.fontType           = fontType;
        this.actualFontType     = fontType;
        this.fontColor          = fontColor;
        this.maximumLength      = maximumLength;
        timerSet                = false;
        textField = prepareTextField(placeholderText);
    }

    public String getText() {
        return textField.getText();
    }

	public float getTextFieldWidthPercent() {
		return textFieldWidthPercent;
	}

	public void setTextFieldWidthPercent(float textFieldWidthPercent) {
		this.textFieldWidthPercent = textFieldWidthPercent;
	}

	@Override
    public void prepareContent(Cell content) {
        this.content = content;
		content.width(getDialog().getWidth() * getTextFieldWidthPercent());
        content.setActor(textField);
    }

    @Override
    public void adaptiveSizeThresholdChanged() {
        if (ComponentScaleUtil.isUnderThreshold()) {
            if (!isScaled) {
                actualFontType = ApplicationFontManager.FontType_NEW.getProximateFontTypeForSize(fontType.getSize() + FONT_SIZE_RESIZE_THRESHOLD_STEP, fontType.isBold());
                isScaled = true;
            }
        } else {
            if (isScaled) {
                actualFontType = fontType;
                isScaled = false;
            }
        }
        textField.setTablexiaTextFieldStyle(prepareTextFieldStyle());
        content.width(getDialog().getWidth() * getTextFieldWidthPercent());
    }

    private TextFieldWithPlaceholder prepareTextField(String placeholderText) {
        TextFieldWithPlaceholder fieldWithPlaceholder = new TextFieldWithPlaceholder("", prepareTextFieldStyle(), placeholderText, DEFAULT_TEXTFIELD_ALIGN);
        if (maximumLength != null) {
            fieldWithPlaceholder.setMaxLength(maximumLength);
        }
        return fieldWithPlaceholder;
    }

    public TextFieldWithPlaceholder.TablexiaTextFieldStyle prepareTextFieldStyle() {
        return new TextFieldWithPlaceholder.TablexiaTextFieldStyle(actualFontType, fontColor, createCursorForTextField(), null, null);
    }

    private Drawable createCursorForTextField() {
        Texture texture = ApplicationAtlasManager.getInstance().getColorTexture(Color.BLACK);
        return new Image(texture).getDrawable();
    }

    public void setRepositionOnFocusOnMobileDevices(boolean repositionOnFocus) {
        if(repositionOnFocus && TablexiaSettings.getInstance().isRunningOnMobileDevice()) {
            if (repositionOnTypeListener == null) prepareOnTypeListener();
            if (repositionOnTouchListener == null) prepareOnTouchListener();
            textField.addListener(repositionOnTypeListener);
            textField.addListener(repositionOnTouchListener);
        }
        else {
            if (repositionOnTypeListener != null) textField.removeListener(repositionOnTypeListener);
            if (repositionOnTouchListener != null) textField.removeListener(repositionOnTouchListener);
        }
    }

    private void prepareOnTypeListener() {
        this.repositionOnTypeListener = new InputListener() {
            private static final int COMMON_INT_CODE_FOR_ENTER = 13;
            private static final int ANDROID_INT_CODE_FOR_ENTER = 10;

            @Override
            public boolean keyTyped(InputEvent event, char character) {
                if (timerSet) timer.cancel();
                // have to check also (int)character because of some bug in iOS keyboard. It returns code Input.Keys.ENTER for every key after "done" button pressed
                if ((event.getKeyCode() != Input.Keys.ENTER || ((int) character != ANDROID_INT_CODE_FOR_ENTER || (int) character != COMMON_INT_CODE_FOR_ENTER))) {
                    resetTimer();
                } else {
                    repositionDialog(DialogPosition.CENTER);
                }
                return false;
            }
        };
    }
        
    
    public void prepareOnTouchListener(){
        this.repositionOnTouchListener = new InputListener(){

            @Override
            public boolean touchDown(InputEvent event, float x, float y, int pointer, int button) {
                if (timerSet) timer.cancel();
                repositionDialog(DialogPosition.TOP);
                resetTimer();
                return super.touchDown(event, x, y, pointer, button);
            }
        };
    }

    private void resetTimer() {
        timer = new Timer();
        timer.schedule(new TimerTask() {
            @Override
            public void run() {
                repositionDialog(DialogPosition.CENTER);
                timerSet = false;
            }
        }, DIALOG_REPLACE_TIMEOUT);
        timerSet = true;
    }

    private void repositionDialog(final DialogPosition dialogPosition) {
        TablexiaComponentDialog dialog = getDialog();
        Stage stage = getStage();
        if(dialog == null || (stage == null)) return;

        dialog.setPosition(
                TablexiaSettings.getViewportLeftX(stage) + TablexiaSettings.getViewportWidth(stage) * 0.5f - dialog.getInnerWidth() * 0.5f,
                TablexiaSettings.getViewportBottomY(stage) + TablexiaSettings.getViewportHeight(stage) * dialogPosition.getShift() - dialog.getOutterHeight() * dialogPosition.getShift()
        );

        Gdx.app.postRunnable(new Runnable() {
            @Override
            public void run() {
                Gdx.input.setOnscreenKeyboardVisible(dialogPosition.isShowKeyboard());
            }
        });
    }

    private enum DialogPosition {
        TOP(1f, true),
        CENTER(0.5f, false);

        private float   shift;
        private boolean showKeyboard;

        DialogPosition(float shift, boolean showKeyboard) {
            this.shift = shift;
            this.showKeyboard = showKeyboard;
        }

        public float getShift() {
            return shift;
        }

        public boolean isShowKeyboard() {
            return showKeyboard;
        }
    }
}
