package cz.nic.tablexia.util.ui.dialog.components;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.scenes.scene2d.ui.Cell;
import com.badlogic.gdx.utils.Align;

import cz.nic.tablexia.loader.application.ApplicationFontManager;
import cz.nic.tablexia.util.ui.TablexiaLabel;

/**
 * Created by Matyáš Latner.
 */
public class TwoColumnTextContentDialogComponent extends TextContentDialogComponent {

    private static final Color                               DEFAULT_FONT_COLOR              = Color.BLACK;
    private static final ApplicationFontManager.FontType_NEW DEFAULT_FONT_TYPE               = ApplicationFontManager.FontType_NEW.REGULAR_20;
    private static final int                                 DEFAULT_ALIGN_1                 = Align.center;
    private static final int                                 DEFAULT_ALIGN_2                 = Align.center;

    private final   String                                  text2;
    private int                                             align2;
    private TablexiaLabel                                   textLabel;

    public TwoColumnTextContentDialogComponent(String text1, String text2) {
        this(text1, text2, DEFAULT_ALIGN_1, DEFAULT_ALIGN_2);
    }

    public TwoColumnTextContentDialogComponent(String text1, String text2, Integer align1, Integer align2) {
        this(text1, text2, DEFAULT_FONT_TYPE, DEFAULT_FONT_COLOR, align1, align2);
    }

    public TwoColumnTextContentDialogComponent(String text1, String text2, ApplicationFontManager.FontType_NEW fontType, Color fontColor, Integer align1, Integer align2) {
        super(text1, fontType, fontColor, align1, Float.valueOf(0f));
        this.text2 = text2;
        this.align2 = align2.intValue();
    }

    @Override
    public void adaptiveSizeThresholdChanged() {
        super.adaptiveSizeThresholdChanged();
        textLabel.setTablexiaLabelStyle(new TablexiaLabel.TablexiaLabelStyle(getActualFontType(), getFontColor()));
    }

    @Override
    protected void prepareTextLabel(Cell content) {
        super.prepareTextLabel(content);
        textLabel = new TablexiaLabel(text2, new TablexiaLabel.TablexiaLabelStyle(getActualFontType(), getFontColor()));
        textLabel.setWrap(true);
        textLabel.setAlignment(align2);
        content.getTable().add().setActor(textLabel).fillX();
    }
}
